@@ -43,6 +43,7 @@ struct _GbDefaultIconProviderPrivate
 	GdkPixbuf *default_folder_open_pixbuf_data;
 	GdkPixbuf *site_pixbuf_data;
 	GdkPixbuf *separator_pixbuf_data;
+	gboolean show_default_icon_site;
 };
 
 /**
@@ -86,6 +87,9 @@ gb_default_icon_provider_init (GbDefaultIconProvider *dip)
 	GbDefaultIconProviderPrivate *p = g_new0 (GbDefaultIconProviderPrivate, 1);
 	dip->priv = p;
 	
+	g_object_ref (dummy);
+	gtk_object_sink (GTK_OBJECT (dummy));
+
 	p->folder_pixbuf_data =
 		gtk_widget_render_icon (dummy, GALEON_STOCK_FOLDER, GTK_ICON_SIZE_MENU, NULL);
 	p->folder_open_pixbuf_data =
@@ -99,7 +103,9 @@ gb_default_icon_provider_init (GbDefaultIconProvider *dip)
 	p->separator_pixbuf_data  =
 		gtk_widget_render_icon (dummy, GALEON_STOCK_SEPARATOR, GTK_ICON_SIZE_MENU, NULL);
 
-	gtk_widget_destroy (dummy);
+	g_object_unref (dummy);
+
+	p->show_default_icon_site = FALSE;
 }
 
 static void
@@ -131,7 +137,7 @@ static GdkPixbuf *
 gb_default_icon_provider_icon_provider_get_icon_impl (GbIconProvider *ip, GbBookmark *b)
 {
 	GbDefaultIconProviderPrivate *p = GB_DEFAULT_ICON_PROVIDER (ip)->priv;
-	GdkPixbuf *ret;
+	GdkPixbuf *ret = NULL;
 
 	DEBUG_MSG (("getting icon for %s\n", b->name));
 
@@ -152,7 +158,10 @@ gb_default_icon_provider_icon_provider_get_icon_impl (GbIconProvider *ip, GbBook
 	}
 	else
 	{
-		ret = p->site_pixbuf_data;
+		if (p->show_default_icon_site)
+		{
+			ret = p->site_pixbuf_data;
+		}
 	}
 
 	if (gb_bookmark_is_alias (b))
@@ -160,6 +169,6 @@ gb_default_icon_provider_icon_provider_get_icon_impl (GbIconProvider *ip, GbBook
 		ret = gb_util_make_alias_icon (ret);
 	}
 	
-	return g_object_ref (ret);
+	return ret ? g_object_ref (ret) : NULL;
 }
 
