@@ -42,6 +42,7 @@
 #include "gul-general.h"
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkvbox.h>
@@ -833,6 +834,18 @@ hpaned_position_middle_hack (GtkPaned *paned)
 	gtk_paned_set_position (paned, pos);
 }
 
+static gboolean
+gb_editor_key_press_event_cb (GtkWidget *widget, GdkEventKey *event)
+{
+	if ((event->state & GDK_CONTROL_MASK) || (event->state & GDK_MOD1_MASK) || (event->keyval == GDK_Delete) || (event->keyval == GDK_KP_Delete))
+	{
+		GtkWidget *entry = gtk_window_get_focus (GTK_WINDOW(widget));
+		if (GTK_IS_EDITABLE(entry) && gtk_widget_event (entry, (GdkEvent*)event))
+			return TRUE;
+	}
+	return FALSE;
+}
+
 static void
 gb_editor_init_widgets_impl (GbEditor *e)
 {
@@ -939,6 +952,10 @@ gb_editor_init_widgets_impl (GbEditor *e)
 	gul_state_monitor_window (p->main_widget, "Bookmarks",
 			          STATE_BOOKMARKS_WIDTH_DEFAULT,
 				  STATE_BOOKMARKS_HEIGHT_DEFAULT);
+
+	/* keyboard accelerator handling crack */
+	g_signal_connect (p->main_widget, "key-press-event",
+			  G_CALLBACK (gb_editor_key_press_event_cb), NULL);
 }
 
 GbEditor *
