@@ -28,7 +28,6 @@
 #include "bookmarks-util.h"
 #include "xbel.h"
 #include "gul-gobject-misc.h"
-#include "gul-state.h"
 #include <libgnomeui/gnome-druid.h>
 #include <libgnomeui/gnome-druid-page-standard.h>
 #include <libgnomeui/gnome-druid-page-edge.h>
@@ -90,9 +89,6 @@ static void 		gb_import_druid_page_5_prepare_cb	(GnomeDruidPage *druid_page,
 static void		gb_import_druid_location_list_selection_changed_cb (GtkTreeSelection *ts, 
 									    GbImportDruid *d);
 static void		gb_import_druid_location_entry_changed_cb (GtkEditable *e, GbImportDruid *d);
-static gboolean		gb_import_druid_window_configure_event_cb (GtkWidget *widget,
-								   GdkEventConfigure *event,
-								   GbImportDruid *d);
 static void		gb_import_druid_merge_button_clicked_cb	(GtkButton *b, GbImportDruid *d);
 
 
@@ -158,6 +154,7 @@ gb_import_druid_init_widgets (GbImportDruid *d)
 	g_return_if_fail (p->window == NULL);
 	
 	p->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_resizable (GTK_WINDOW (p->window), FALSE);
 	gtk_window_set_title (GTK_WINDOW (p->window), 
 			      gb_import_druid_get_string (d, "window title"));
 
@@ -377,11 +374,6 @@ gb_import_druid_show (GbImportDruid *d)
 	g_return_if_fail (GB_IS_BOOKMARK_SET (p->dest));
 	g_return_if_fail (GTK_IS_WINDOW (p->window));
 	
-	/* window size saving crack */
-	gul_state_load_window (p->window, "bookmarks_import_druid", 600, 430, FALSE);
-	g_signal_connect (p->window, "configure_event", 
-			  G_CALLBACK (gb_import_druid_window_configure_event_cb), d);
-
 	gtk_widget_show (GTK_WIDGET (p->window));
 }
 
@@ -579,15 +571,6 @@ gb_import_druid_location_entry_changed_cb (GtkEditable *e, GbImportDruid *d)
 	p->location = sel;
 }
 
-static gboolean
-gb_import_druid_window_configure_event_cb (GtkWidget *widget,
-					   GdkEventConfigure *event,
-					   GbImportDruid *d)
-{
-	gul_state_save_window (widget, "bookmarks_import_druid");
-	return FALSE;
-}
-
 static void
 gb_import_druid_merge_button_clicked_cb (GtkButton *b, GbImportDruid *d)
 {
