@@ -29,6 +29,16 @@
 //#define DEBUG_MSG(x) g_print x
 #define DEBUG_MSG(x)
 
+#define DEBUG_PATH(path) \
+	{ \
+		gchar *s_patch = gtk_tree_path_to_string (path); \
+		g_print (#path " = %s\n", s_patch); \
+		g_free (s_patch); \
+	}
+
+
+	
+
 #define VALID_ITER(iter, tree_model)	(iter!= NULL && iter->user_data != NULL \
 					&& tree_model->stamp == iter->stamp)
 
@@ -86,8 +96,11 @@ static gboolean		gb_tree_model_row_drop_possible	(GtkTreeDragDest *drag_dest,
 							 GtkSelectionData *selection_data);
 
 /* helper functions */
-static gboolean		gb_tree_model_is_root		(GbTreeModel *m, GbFolder *f, int *i);
+static gboolean		gb_tree_model_is_root		(GbTreeModel *m, GbBookmark *f, int *i);
+static GbBookmark *	gb_tree_model_get_nth_root	(GbTreeModel *m, int i);
+
 static void		gb_tree_model_descendant_modified_cb (GbFolder *f, GbBookmark *b, gpointer data);
+static void		gb_tree_model_modified_cb (GbBookmark *b, gpointer data);
 static void 		gb_tree_model_descendant_added_cb (GbFolder *f, GbFolder *p, GbBookmark *b,
 							   int pos, gpointer data);
 static void 		gb_tree_model_descendant_removed_cb (GbFolder *f, GbFolder *p, GbBookmark *b, 
@@ -198,16 +211,163 @@ gb_tree_model_drag_dest_init (GtkTreeDragDestIface *iface)
 	iface->row_drop_possible = gb_tree_model_row_drop_possible;
 }
 
+static gboolean
+gb_tree_model_default_check_func (GbBookmark *b, gpointer data)
+{
+	return FALSE;
+}
+
 static void
 gb_tree_model_init (GbTreeModel *btree_model)
 {
 	btree_model->roots = NULL;
+	btree_model->only_folders = FALSE;
+	btree_model->hide_roots = FALSE;
+	btree_model->check_func = gb_tree_model_default_check_func;
+	btree_model->check_func_data = NULL;
 	do
 	{
 		btree_model->stamp = g_random_int ();
 	}
 	while (btree_model->stamp == 0);
 }
+#include <string.h>
+
+static void
+gb_tree_model_set_only_folders_recursively_emit_signals (GbTreeModel *tm, GbBookmark *lb, int rootindex)
+{
+	GtkTreeIter iter;
+	GtkTreePath *path;
+#if 0
+	static int ind = 0;
+#define D(msg,n,path) { \
+	gchar *t = g_strdup (n); \
+	int j;\
+	if (strlen (t) > 10) t[10] = '\0'; \
+	for (j = 0; j < ind; ++j) g_print ("    ");\
+	g_print (msg, t); \
+	for (j = 10 - ind; j > 0; --j) g_print ("    ");\
+	DEBUG_PATH (path); \
+	g_free (t);\
+	ind++;\
+}
+#endif
+	
+	iter.user_data = lb;
+	iter.user_data2 = GINT_TO_POINTER (rootindex);
+	iter.stamp = tm->stamp;
+	
+	path = gb_tree_model_get_path ((GtkTreeModel *) tm, &iter);
+
+#if 0
+	{
+		GtkTreeIter ij;
+		gb_tree_model_get_iter ((GtkTreeIter *) tm, &ij, path);
+		g_assert (ij.stamp == iter.stamp);
+		g_assert (ij.user_data == iter.user_data);
+		D ("+%-10s ", lb->name, path);
+	}
+#endif	
+	gtk_tree_model_row_has_child_toggled ((GtkTreeModel *) tm,
+					      path,
+					      &iter);
+	
+	if (GB_IS_FOLDER (lb) && (!gb_bookmark_is_alias (lb) || gb_tree_model_get_nth_root (tm, rootindex) == lb))
+	{
+		GbBookmark *bi;
+		gtk_tree_path_append_index (path, 0);
+		for (bi = GB_FOLDER (lb)->child; bi; bi = bi->next)
+		{
+			GtkTreeIter iterbi;
+			iterbi.user_data = bi;
+			iterbi.stamp = tm->stamp;
+
+			if (GB_IS_FOLDER (bi))
+			{
+				gtk_tree_model_row_has_child_toggled ((GtkTreeModel *) tm,
+								      path,
+								      &iterbi);
+				gtk_tree_path_next (path);
+			}
+			else
+			{
+				if (tm->only_folders)
+				{
+					gtk_tree_model_row_deleted ((GtkTreeModel *) tm, path);
+				}
+				else
+				{
+					
+					gtk_tree_model_row_inserted ((GtkTreeModel *) tm, path, &iterbi);
+					gtk_tree_path_next (path);
+				}
+			}
+		}
+		gtk_tree_path_up (path);
+
+		for (bi = GB_FOLDER (lb)->child; bi; bi = bi->next)
+		{
+			if (GB_IS_FOLDER (bi))
+			{
+				gb_tree_model_set_only_folders_recursively_emit_signals (tm, bi, rootindex);
+			}
+		}
+	}
+	
+	gtk_tree_path_free (path);
+#if 0
+	ind--;
+#endif
+}
+
+void
+gb_tree_model_set_only_folders (GbTreeModel *tm, gboolean value)
+{
+	if (tm->only_folders != value)
+	{
+		int i = 0;
+		GSList *l;
+		tm->only_folders = value;
+		
+		for (l = tm->roots; l; l = l->next)
+		{
+			{
+				int j;
+				g_assert (gb_tree_model_is_root (tm, l->data, &j));
+				g_assert (i == j);
+			}
+			gb_tree_model_set_only_folders_recursively_emit_signals (tm, l->data, i);
+			++i;
+		}
+	}
+}
+
+gboolean
+gb_tree_model_get_only_folders (GbTreeModel *tm)
+{
+	return tm->only_folders;
+}
+
+void
+gb_tree_model_set_hide_roots (GbTreeModel *tm, gboolean value)
+{
+	if (tm->hide_roots != value)
+	{
+		GSList *l;
+		tm->hide_roots = value;
+		
+		for (l = tm->roots; l; l = l->next)
+		{
+			/* TODO: should notify the listeners of the lots of changes! */
+		}
+	}
+}
+
+gboolean
+gb_tree_model_get_hide_roots (GbTreeModel *tm)
+{
+	return tm->hide_roots;
+}
 
 /**
  * gb_tree_model_new:
@@ -265,12 +425,17 @@ gb_tree_model_set_roots (GbTreeModel *btree_model, GSList *roots)
 
 	for (l = btree_model->roots; l != NULL; l = l->next)
 	{
-		g_signal_connect (l->data, "descendant-modified", 
-				  G_CALLBACK (gb_tree_model_descendant_modified_cb), btree_model);
-		g_signal_connect (l->data, "descendant-added", 
-				  G_CALLBACK (gb_tree_model_descendant_added_cb), btree_model);
-		g_signal_connect (l->data, "descendant-removed", 
-				  G_CALLBACK (gb_tree_model_descendant_removed_cb), btree_model);
+		if (GB_IS_FOLDER (l->data))
+		{
+			g_signal_connect (l->data, "descendant-modified", 
+					  G_CALLBACK (gb_tree_model_descendant_modified_cb), btree_model);
+			g_signal_connect (l->data, "descendant-added", 
+					  G_CALLBACK (gb_tree_model_descendant_added_cb), btree_model);
+			g_signal_connect (l->data, "descendant-removed", 
+					  G_CALLBACK (gb_tree_model_descendant_removed_cb), btree_model);
+		}
+		g_signal_connect (l->data, "modified", 
+				  G_CALLBACK (gb_tree_model_modified_cb), btree_model);
 		g_object_ref (l->data);
 	}
 }
@@ -327,6 +492,8 @@ gb_tree_model_get_column_type (GtkTreeModel *tree_model,
 	case GB_TREE_MODEL_COL_URL:
 		return G_TYPE_STRING;
 		break;
+	case GB_TREE_MODEL_COL_CHECK:
+		return G_TYPE_BOOLEAN;
 	default:
 		g_assert_not_reached ();
 		return G_TYPE_INVALID;
@@ -349,29 +516,54 @@ gb_tree_model_get_iter (GtkTreeModel *tree_model,
 	indices = gtk_tree_path_get_indices (path);
 	depth = gtk_tree_path_get_depth (path);
 	
-	g_return_val_if_fail (depth > 0, FALSE);
+	iter->stamp = btree_model->stamp;
+	iter->user_data = btree_model->roots;
+	iter->user_data2 = NULL; /* does not really matter here */
 	
-	parent.stamp = btree_model->stamp;
-	parent.user_data = btree_model->roots;
-	
-	if (! gb_tree_model_iter_nth_child (tree_model, iter, &parent, indices[0]))
-		return FALSE;
-	
-	for (i = 1; i < depth; i++)
+	for (i = 0; i < depth; i++)
 	{
 		parent = *iter;
 		if (! gb_tree_model_iter_nth_child (tree_model, iter, &parent, indices[i]))
+		{
 			return FALSE;
+		}
 	}
 	return TRUE;
 }
 
+static int 
+gb_tree_model_bookmark_child_index (GbTreeModel *btree_model, GbBookmark *child)
+{
+	GbFolder *parent = child->parent;
+
+	if (btree_model->only_folders)
+	{
+		GbBookmark *c;
+		gint i = 0;
+		for  (c = parent->child; c && c != child; c = c->next)
+		{
+			if (GB_IS_FOLDER (c))
+			{
+				++i;
+			}
+		}
+		return i;
+	}
+	else
+	{
+		return gb_folder_get_child_index (parent, child);
+	}
+}
+
 static GtkTreePath *
 gb_tree_model_get_path (GtkTreeModel *tree_model,
 			GtkTreeIter *iter)
 {
 	GtkTreePath *retval;
 	GbBookmark *b_it;
+	int rootindex;
+	GbBookmark *root;
+	GbBookmark *realroot;
 	gint i = 0;
 	GbTreeModel *btree_model = (GbTreeModel *) tree_model;
 	
@@ -386,11 +578,15 @@ gb_tree_model_get_path (GtkTreeModel *tree_model,
 	}
 	
 	b_it = iter->user_data;
+	rootindex = GPOINTER_TO_INT (iter->user_data2);
+	root = gb_tree_model_get_nth_root (btree_model, rootindex);
+	realroot = gb_bookmark_real_bookmark (root);
 
-	if (gb_tree_model_is_root (btree_model, (GbFolder *) b_it, &i))
+	if (root == b_it || realroot == b_it)
 	{
 		retval = gtk_tree_path_new ();
-		gtk_tree_path_append_index (retval, i);
+		g_return_val_if_fail (!btree_model->hide_roots, retval);
+		gtk_tree_path_append_index (retval, rootindex);
 	}
 	else
 	{
@@ -398,21 +594,48 @@ gb_tree_model_get_path (GtkTreeModel *tree_model,
 		GtkTreeIter tmp_iter = *iter;
 
 		g_assert (parent != NULL);
+		g_assert (!gb_bookmark_is_alias (parent));
+
 		tmp_iter.user_data = parent;
+		tmp_iter.user_data2 = GINT_TO_POINTER (rootindex);
 		
-		retval = gb_tree_model_get_path (tree_model,
-						 &tmp_iter);
-		if (retval != NULL) 
+		if (btree_model->hide_roots && ((((GbBookmark *) parent) == root)
+						||((GbBookmark *) parent) == realroot))
 		{
-			i = gb_folder_get_child_index (parent, b_it);
-			gtk_tree_path_append_index (retval, i);
+			int nprevs = 0;
+			GSList *sli;
+			GtkTreeIter riter;
+			int rrootindex = 0;
+			riter.stamp = btree_model->stamp;
+			for (sli = btree_model->roots; 
+			     sli && gb_bookmark_real_bookmark (sli->data) != (GbBookmark *) parent; 
+			     sli = sli->next)
+			{
+				riter.user_data = sli->data;
+				riter.user_data2 = GINT_TO_POINTER (rrootindex);
+				nprevs += gb_tree_model_iter_n_children (tree_model, &riter);
+				rrootindex++;
+			}
+			g_assert (rrootindex == rootindex);
+			retval = gtk_tree_path_new ();
+			gtk_tree_path_append_index (retval, 
+						    nprevs + gb_tree_model_bookmark_child_index (btree_model, b_it));
+		}
+		else
+		{
+			retval = gb_tree_model_get_path (tree_model, &tmp_iter);
+		
+			if (retval != NULL) 
+			{
+				i = gb_tree_model_bookmark_child_index (btree_model, b_it);
+				gtk_tree_path_append_index (retval, i);
+			}
 		}
 	}
 
 	return retval;
 }
 
-
 static void
 gb_tree_model_get_value (GtkTreeModel *tree_model,
 			 GtkTreeIter *iter,
@@ -421,9 +644,10 @@ gb_tree_model_get_value (GtkTreeModel *tree_model,
 {
 	GbBookmark *b;
 	GdkPixbuf *pb;
-	g_return_if_fail (GB_IS_TREE_MODEL (tree_model));
+	GbTreeModel *btree_model = (GbTreeModel *) tree_model;
+	g_return_if_fail (GB_IS_TREE_MODEL (btree_model));
 	g_return_if_fail (iter != NULL);
-	g_return_if_fail (iter->stamp == GB_TREE_MODEL (tree_model)->stamp);
+	g_return_if_fail (iter->stamp == btree_model->stamp);
 	g_return_if_fail (GB_IS_BOOKMARK (iter->user_data));
 	g_return_if_fail (column < GB_TREE_MODEL_NUM_COLUMS);
 	
@@ -453,6 +677,10 @@ gb_tree_model_get_value (GtkTreeModel *tree_model,
 			g_value_set_string (value, ((GbSite *) b)->url);
 		}
 		break;
+	case GB_TREE_MODEL_COL_CHECK:
+		g_value_init (value, G_TYPE_BOOLEAN);
+		g_value_set_boolean (value, btree_model->check_func (b, btree_model->check_func_data));
+		break;
 	default:
 		g_assert_not_reached ();
 		break;
@@ -464,20 +692,27 @@ gb_tree_model_iter_next (GtkTreeModel *tree_model,
 			 GtkTreeIter *iter)
 {
 	GbBookmark *b;
-	gint i;
+	int rootindex;
 	GbTreeModel *btree_model = (GbTreeModel *) tree_model;
 	g_return_val_if_fail (iter != NULL, FALSE);
 	g_return_val_if_fail (iter->user_data != NULL, FALSE);
 	g_return_val_if_fail (iter->stamp == GB_TREE_MODEL (tree_model)->stamp, FALSE);
 	
 	b = iter->user_data;
+	rootindex = GPOINTER_TO_INT (iter->user_data2);
 	
-	if (gb_tree_model_is_root (btree_model, (GbFolder *) b, &i))
+	if (b == gb_tree_model_get_nth_root (btree_model, rootindex))
 	{
-		GSList *nl = g_slist_nth (btree_model->roots, i + 1);
-		if (nl != NULL) 
+		GbBookmark *nb;
+		++rootindex;
+		nb = gb_tree_model_get_nth_root (btree_model, rootindex);
+
+		g_return_val_if_fail (!btree_model->hide_roots, FALSE);
+		
+		if (nb != NULL) 
 		{
-			iter->user_data = nl->data;
+			iter->user_data = nb;
+			iter->user_data2 = GINT_TO_POINTER (rootindex);
 			return TRUE;
 		}
 		else
@@ -487,15 +722,34 @@ gb_tree_model_iter_next (GtkTreeModel *tree_model,
 	}
 	else
 	{
-		GbBookmark *n;
-		n = b->next;
+		GbBookmark *n = b->next;
+	search:
+		if (btree_model->only_folders)
+		{
+			for ( ; n && !GB_IS_FOLDER (n); n = n->next) ;
+		}
+		
 		if (n != NULL) 
 		{
 			iter->user_data = n;
+			iter->user_data2 = GINT_TO_POINTER (rootindex);
 			return TRUE;
 		}
 		else
 		{
+			if (btree_model->hide_roots
+			    && ((GbBookmark *) b->parent) == gb_tree_model_get_nth_root (btree_model, rootindex))
+			{
+				GbBookmark *nb;
+				++rootindex;
+				nb = gb_tree_model_get_nth_root (btree_model, rootindex);
+				
+				if (nb != NULL && GB_IS_FOLDER (nb)) 
+				{
+					n = GB_FOLDER (nb)->child;
+					goto search;
+				}
+			}
 			return FALSE;
 		}
 	}
@@ -506,28 +760,62 @@ gb_tree_model_iter_children (GtkTreeModel *tree_model,
 			     GtkTreeIter *iter,
 			     GtkTreeIter *parent)
 {
-	GbBookmark *c1;
 	GbTreeModel *btree_model = (GbTreeModel *) tree_model;
+	GbBookmark *c1;
+	int rootindex;
+
 	g_return_val_if_fail (parent == NULL || parent->user_data != NULL, FALSE);
 	g_return_val_if_fail (parent == NULL || parent->stamp == btree_model->stamp, FALSE);
 	
-	if (!parent) 
+	if (!parent || parent->user_data == btree_model->roots) 
 	{
-		c1 = btree_model->roots->data;
+		if (btree_model->hide_roots)
+		{
+			GSList *li;
+			rootindex = 0;
+			c1 = NULL;
+			for (li = btree_model->roots; li; li = li->next)
+			{
+				if (GB_IS_FOLDER (li->data))
+				{
+					c1 = GB_FOLDER (li->data)->child;
+					break;
+				}
+				++rootindex;
+			}
+		}
+		else
+		{
+			c1 = btree_model->roots->data;
+			rootindex = 0;
+		}
 	}
 	else
 	{
 		GbFolder *p = parent->user_data;
-		if (!gb_bookmark_is_alias (p))
-			c1 = p->child;
+		rootindex = GPOINTER_TO_INT (parent->user_data2);
+		if (!gb_bookmark_is_alias (p) || gb_tree_model_get_nth_root (btree_model, rootindex) == (GbBookmark *) p)
+		{
+			if (btree_model->only_folders)
+			{
+				for (c1 = p->child; c1 && !GB_IS_FOLDER (c1); c1 = c1->next) ;
+			}
+			else
+			{
+				c1 = p->child;
+			}
+		}
 		else 
+		{
 			c1 = NULL;
+		}
 	}
 	
 	if (c1)
 	{
 		iter->stamp = GB_TREE_MODEL (tree_model)->stamp;
 		iter->user_data = c1;
+		iter->user_data2 = GINT_TO_POINTER (rootindex);
 		return TRUE;
 	}
 	else
@@ -540,91 +828,208 @@ static gboolean
 gb_tree_model_iter_has_child (GtkTreeModel *tree_model,
 			      GtkTreeIter *iter)
 {
+	GbTreeModel *btree_model = (GbTreeModel *) tree_model;
 	GbFolder *b_it;
-	g_return_val_if_fail (GB_IS_TREE_MODEL (tree_model), FALSE);
-	g_return_val_if_fail (iter->stamp == GB_TREE_MODEL (tree_model)->stamp, FALSE);
+	int rootindex;
+	
+	g_return_val_if_fail (GB_IS_TREE_MODEL (btree_model), FALSE);
+	g_return_val_if_fail (iter->stamp == btree_model->stamp, FALSE);
 	g_return_val_if_fail (iter->user_data != NULL, FALSE);
 	
 	if (!GB_IS_FOLDER (iter->user_data))
 		return FALSE;
 
 	b_it = iter->user_data;
-	
-	return b_it->child != NULL && !gb_bookmark_is_alias (b_it);
+	rootindex = GPOINTER_TO_INT (iter->user_data2);
+
+	if (gb_bookmark_is_alias (b_it) && gb_tree_model_get_nth_root (btree_model, rootindex) != (GbBookmark *) b_it)
+	{
+		return FALSE;
+	}
+	else if (btree_model->only_folders)
+	{
+		GbBookmark *b;
+		for (b = b_it->child; b; b = b->next)
+		{
+			if (GB_IS_FOLDER (b))
+			{
+				return TRUE;
+			}
+		}
+		return FALSE;
+	}
+	else
+	{
+		return b_it->child != NULL;
+	}
+}
+
+static int 
+gb_tree_model_bookmark_n_children (GbTreeModel *btree_model, GbBookmark *b, int rootindex)
+{
+	int i = 0;
+	if (GB_IS_FOLDER (b)
+	    && (!gb_bookmark_is_alias (b) || gb_tree_model_get_nth_root (btree_model, rootindex) == b))
+	{
+		gboolean no_foders = ! btree_model->only_folders;
+		GbBookmark *c = GB_FOLDER (b)->child;
+		while (c != NULL)
+		{
+			if (no_foders || GB_IS_FOLDER (c))
+			{
+				i++;
+			}
+			c = c->next;
+		}
+	}
+	return i;
 }
 
 static gint
 gb_tree_model_iter_n_children (GtkTreeModel *tree_model,
 			       GtkTreeIter *iter)
 {
+	GbTreeModel *btree_model = (GbTreeModel *) tree_model;
+	int rootindex;
+
 	g_return_val_if_fail (GB_IS_TREE_MODEL (tree_model), 0);
 	g_return_val_if_fail (iter == NULL || iter->user_data != NULL, 0);
 	
-	if (iter == NULL)
+	rootindex = GPOINTER_TO_INT (iter->user_data2);
+
+	if (iter == NULL || iter->user_data == GB_TREE_MODEL (tree_model)->roots)
 	{
-		GSList *l = GB_TREE_MODEL (tree_model)->roots;
-		return g_slist_length (l);
+		if (btree_model->hide_roots)
+		{
+			GSList *l;
+			int nch = 0;
+			for (l = btree_model->roots; l; l = l->next)
+			{
+				nch += gb_tree_model_bookmark_n_children (btree_model, l->data, rootindex);
+			}
+			return nch;
+		}
+		else
+		{
+			if (GB_TREE_MODEL (tree_model)->only_folders)
+			{
+				int i = 0;
+				GSList *l;
+
+				for (l = btree_model->roots; l; l = l->next)
+				{
+					if (GB_IS_FOLDER (l->data))
+					{
+						++i;
+					}
+				}
+				return i;
+			}
+			else
+			{
+				return g_slist_length (btree_model->roots);
+			}
+		}
 	}
 	else 
 	{
-		GbFolder *b = iter->user_data;
-		int i = 0;
-		if (GB_IS_FOLDER (b) && !gb_bookmark_is_alias (b))
-		{
-			GbBookmark *c = b->child;
-			while (c != NULL)
-			{
-				i++;
-				c = c->next;
-			}
-		}
-		return i;
+		return gb_tree_model_bookmark_n_children (btree_model, iter->user_data, rootindex);
 	}
 }
 
+static GbBookmark *
+gb_tree_model_bookmark_nth_child (GbTreeModel *btree_model, GbFolder *parent, gint index)
+{
+	gboolean no_folders = ! btree_model->only_folders;
+	int i = 0;
+	GbBookmark *c;
+		
+	for (c = parent->child; c; c = c->next)
+	{
+		if (i == index && (no_folders || GB_IS_FOLDER (c)))
+		{
+			break;
+		}
+		else if (no_folders || GB_IS_FOLDER (c))
+		{
+			i++;
+		}
+	}
+	return c;
+}
+
 static gboolean
 gb_tree_model_iter_nth_child (GtkTreeModel *tree_model,
 			      GtkTreeIter *iter,
 			      GtkTreeIter *parent,
 			      gint n)
 {
+	GbTreeModel *btree_model = (GbTreeModel *) tree_model;
 	g_return_val_if_fail (GB_IS_TREE_MODEL (tree_model), FALSE);
 	g_return_val_if_fail (parent == NULL || parent->user_data != NULL, FALSE);
 	g_return_val_if_fail (iter != NULL, FALSE);
 	
 	if (parent == NULL || parent->user_data == GB_TREE_MODEL (tree_model)->roots)
 	{
-		GSList *l = g_slist_nth (GB_TREE_MODEL (tree_model)->roots, n);
-		if (l != NULL)
+		if (btree_model->hide_roots)
 		{
-			iter->user_data = l->data;
-			iter->stamp = GB_TREE_MODEL (tree_model)->stamp;
-			return TRUE;
+			GbBookmark *c = NULL;
+			GSList *sli;
+			int rootindex = 0;
+			for (sli = btree_model->roots; sli && !c && n >= 0; sli = sli->next)
+			{
+				c = gb_tree_model_bookmark_nth_child (btree_model, sli->data, n);
+				if (c)
+				{
+					iter->stamp = btree_model->stamp;
+					iter->user_data = c;
+					iter->user_data2 = GINT_TO_POINTER (rootindex);
+					return TRUE;
+				}
+				else
+				{
+					n -= gb_tree_model_bookmark_n_children (btree_model, sli->data, rootindex);
+					++rootindex;
+				}
+			}
+			return FALSE;
 		}
 		else
 		{
-			return FALSE;
+			GSList *l = g_slist_nth (btree_model->roots, n);
+			if (l != NULL)
+			{
+				iter->user_data = l->data;
+				iter->user_data2 = GINT_TO_POINTER (n);
+				iter->stamp = btree_model->stamp;
+				return TRUE;
+			}
+			else
+			{
+				return FALSE;
+			}
 		}
 	}
 	else 
 	{
 		GbFolder *b = parent->user_data;
-		int i = 0;
-		if (GB_IS_FOLDER (b) && !gb_bookmark_is_alias (b))
+		int rootindex = GPOINTER_TO_INT (parent->user_data2);
+		if (GB_IS_FOLDER (b) && (!gb_bookmark_is_alias (b)
+					 || gb_tree_model_get_nth_root (btree_model, rootindex) == (GbBookmark *) b))
 		{
-			GbBookmark *c = b->child;
-			while (c != NULL)
+			GbBookmark *c = gb_tree_model_bookmark_nth_child (btree_model, b, n);
+
+			if (c)
 			{
-				if (i == n) 
-				{
-					iter->user_data = c;
-					iter->stamp = GB_TREE_MODEL (tree_model)->stamp;
-					return TRUE;
-				}
-				i++;
-				c = c->next;
+				iter->user_data = c;
+				iter->user_data2 = parent->user_data2;
+				iter->stamp = btree_model->stamp;
+				return TRUE;
+			}
+			else
+			{
+				return FALSE;
 			}
-			return FALSE;
 		}
 		else 
 		{
@@ -638,28 +1043,48 @@ gb_tree_model_iter_parent (GtkTreeModel *tree_model,
 			   GtkTreeIter *iter,
 			   GtkTreeIter *child)
 {
+	GbTreeModel *btree_model = (GbTreeModel *) tree_model;
 	g_return_val_if_fail (iter != NULL, FALSE);
 	g_return_val_if_fail (child != NULL, FALSE);
 	g_return_val_if_fail (child->user_data != NULL, FALSE);
 	g_return_val_if_fail (child->stamp == GB_TREE_MODEL (tree_model)->stamp, FALSE);
 	
-	if (child->user_data == GB_TREE_MODEL (tree_model)->roots)
+	if (child->user_data == btree_model->roots)
 	{
 		return FALSE;
 	}
-	else if (gb_tree_model_is_root (GB_TREE_MODEL (tree_model), child->user_data, NULL))
-	{
-		iter->user_data = GB_TREE_MODEL (tree_model)->roots;
-		iter->stamp = GB_TREE_MODEL (tree_model)->stamp;
-		return TRUE;
-	}
 	else 
 	{
-		GbFolder *parent = GB_FOLDER (GB_BOOKMARK (child->user_data)->parent);
-		g_assert (parent != NULL);
-		iter->user_data = parent;
-		iter->stamp = GB_TREE_MODEL (tree_model)->stamp;
-		return TRUE;
+		GbBookmark *bit = child->user_data;
+		int rootindex = GPOINTER_TO_INT (child->user_data2);
+		
+		if (gb_tree_model_get_nth_root (btree_model, rootindex) == bit)
+		{
+			iter->user_data = btree_model->roots;
+			iter->stamp = btree_model->stamp;
+			
+			g_return_val_if_fail (!btree_model->hide_roots, TRUE);
+			return TRUE;
+		}
+		else 
+		{
+			GbFolder *parent = GB_FOLDER (bit->parent);
+			g_assert (parent != NULL);
+			
+			if (btree_model->hide_roots
+			    && ((GbBookmark *) parent) == gb_tree_model_get_nth_root (btree_model, rootindex))
+			{
+				iter->user_data = btree_model->roots;
+				iter->stamp = btree_model->stamp;
+			}
+			else
+			{
+				iter->user_data = parent;
+				iter->user_data2 = GINT_TO_POINTER (rootindex);
+				iter->stamp = btree_model->stamp;
+			}
+			return TRUE;
+		}
 	}
 }
 
@@ -679,11 +1104,18 @@ gb_tree_model_iter_depth (GbTreeModel *btree_model,
 {
 	gint depth = 0;
 	GbBookmark *b_it;
+	GbBookmark *root;
+	gint rootindex;
+
 	g_return_val_if_fail (GB_IS_TREE_MODEL (btree_model), 0);
 	g_return_val_if_fail (VALID_ITER (iter, btree_model), 0);
 	
 	b_it = iter->user_data;
-	while (b_it->parent && !gb_tree_model_is_root (btree_model, (GbFolder *) b_it, NULL))
+	rootindex = GPOINTER_TO_INT (iter->user_data2);
+	root = gb_tree_model_get_nth_root (btree_model, rootindex);
+	while (b_it->parent && !(b_it == root
+				 || (btree_model->hide_roots 
+				     && ((GbBookmark *) b_it->parent) == root)))
 	{
 		depth++;
 		b_it = (GbBookmark *) b_it->parent;
@@ -711,7 +1143,7 @@ gb_tree_model_drag_data_delete (GtkTreeDragSource *drag_source,
 	if (gb_tree_model_get_iter (GTK_TREE_MODEL (tm), &iter, path))
 	{
 		GbBookmark *b = gb_tree_model_bookmark_from_iter (tm, &iter);
-		gb_bookmark_unparent (b);
+		gb_bookmark_unparent_safe (b);
 		return TRUE;
 	}
 	else
@@ -777,9 +1209,7 @@ gb_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 		GtkTreeIter src_iter;
 		GbBookmark *srcb;
 		
-		if (!gtk_tree_model_get_iter (src_model,
-					      &src_iter,
-					      src_path))
+		if (!gtk_tree_model_get_iter (src_model, &src_iter, src_path))
 		{
 			gtk_tree_path_free (src_path);
 			return FALSE;
@@ -810,20 +1240,19 @@ gb_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 		return FALSE;
 	}
 
-	{
-		gchar *s = gtk_tree_path_to_string (dest);
-		DEBUG_MSG (("dest = %s\n", s));
-		g_free (s);
-	}
-
+	/*
+	  DEBUG_PATH (dest);
+	*/
+		
 	/* get the destination */
 	{
 		GtkTreePath *dest_parent_path = gtk_tree_path_copy (dest);
 		GtkTreeIter dest_parent_iter; 
 		if (gtk_tree_path_up (dest_parent_path))
 		{
-			if (gtk_tree_model_get_iter (GTK_TREE_MODEL (gtm), 
-						     &dest_parent_iter, dest_parent_path))
+			if ((gtk_tree_path_get_depth (dest_parent_path) > 0)
+			    && (gtk_tree_model_get_iter (GTK_TREE_MODEL (gtm),
+							 &dest_parent_iter, dest_parent_path)))
 			{
 				bparent = gb_tree_model_bookmark_from_iter (gtm, &dest_parent_iter);
 			}
@@ -841,8 +1270,11 @@ gb_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 
 	if (!bparent)
 	{
+		
 		bparent = gb_tree_model_get_first_root (gtm);
 		DEBUG_MSG (("Defaulting the parent to the first root...\n"));
+
+		/* this is broken, specially when hide roots is TRUE */
 	}
 
 	g_return_val_if_fail (bparent, FALSE);
@@ -869,6 +1301,21 @@ gb_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 		bindex = -1;
 	}
 	
+	if (bindex > 0 && gtm->only_folders)
+	{
+		gint realindex = 0;
+		GbBookmark *c;
+		for (c = GB_FOLDER (bparent)->child; c && bindex > 0; c = c->next)
+		{
+			++realindex;
+			if (GB_IS_FOLDER (c))
+			{
+				--bindex;
+			}
+		}
+		bindex = realindex;
+	}
+
 	DEBUG_MSG (("will insert into %s (at %d)\n", bparent->name, bindex));
 
 	gb_folder_add_child (GB_FOLDER (bparent), b, bindex);
@@ -949,16 +1396,21 @@ gb_tree_model_row_drop_possible (GtkTreeDragDest *drag_dest,
 	
 }
 
-
+static GbBookmark *
+gb_tree_model_get_nth_root (GbTreeModel *m, int i)
+{
+	GSList *sli = g_slist_nth (m->roots, i);
+	return sli ? sli->data : NULL;
+}
 
 static gboolean
-gb_tree_model_is_root (GbTreeModel *m, GbFolder *f, int *i)
+gb_tree_model_is_root (GbTreeModel *m, GbBookmark *b, int *i)
 {
 	GSList *l = m->roots;
 	int j = 0;
 	while (l != NULL)
 	{
-		if (l->data == f)
+		if (l->data == b)
 		{
 			if (i != NULL)
 			{
@@ -966,7 +1418,7 @@ gb_tree_model_is_root (GbTreeModel *m, GbFolder *f, int *i)
 			}
 			return TRUE;
 		}
-		i++;
+		j++;
 		l = l->next;
 	}
 	return FALSE;
@@ -975,45 +1427,125 @@ gb_tree_model_is_root (GbTreeModel *m, GbFolder *f, int *i)
 GbBookmark *
 gb_tree_model_bookmark_from_iter (GbTreeModel *m, GtkTreeIter *iter)
 {
-	return GB_BOOKMARK (iter->user_data);
+	if (iter->user_data == m->roots)
+	{
+		return NULL;
+	}
+	else
+	{
+		return GB_BOOKMARK (iter->user_data);
+	}
 }
 
-void
+gboolean
 gb_tree_model_iter_from_bookmark (GbTreeModel *model, GbBookmark *b, GtkTreeIter *iter)
 {
-	iter->stamp = model->stamp;
-	iter->user_data = b;
+	int rootindex = 0;
+	GSList *sli;
+	gboolean ret;
+
+	if (model->only_folders && !GB_IS_FOLDER (b))
+	{
+		return FALSE;
+	}
+
+	for (sli = model->roots; sli; sli = sli->next)
+	{
+		GbBookmark *r = sli->data;
+		if (r == b || (GB_IS_FOLDER (r) && gb_folder_is_ancestor (GB_FOLDER (r), b)))
+		{
+			ret = TRUE;
+			break;
+		}
+		else
+		{
+			++rootindex;
+		}
+	}
+
+	if (ret)
+	{
+		iter->stamp = model->stamp;
+		iter->user_data = b;
+		iter->user_data2 = GINT_TO_POINTER (rootindex);
+	}
+	return ret;
+}
+
+static void
+gb_tree_model_row_changed_helper (GbTreeModel *m, GtkTreeIter *it)
+{
+	GtkTreePath *p = gb_tree_model_get_path ((GtkTreeModel *) m, it);
+	gtk_tree_model_row_changed ((GtkTreeModel *) m, p, it);
+	gtk_tree_path_free (p);
 }
 
 static void 
 gb_tree_model_descendant_modified_cb (GbFolder *f, GbBookmark *b, gpointer data)
 {
 	GbTreeModel *m = data;
-	GtkTreeIter it;
-	GtkTreePath *p;
-	it.user_data = b;
-	it.stamp = m->stamp;
-	p = gb_tree_model_get_path ((GtkTreeModel *) m, &it);
-	gtk_tree_model_row_changed ((GtkTreeModel *) m, p, &it);
-	gtk_tree_path_free (p);
+
+	if (!m->only_folders || GB_IS_FOLDER (b))
+	{
+		int rootindex = g_slist_index (m->roots, f);
+		GtkTreeIter it;
+		
+		g_assert (rootindex >= 0);
+
+		it.user_data = b;
+		it.user_data2 = GINT_TO_POINTER (rootindex);
+		it.stamp = m->stamp;
+
+		gb_tree_model_row_changed_helper (m, &it);
+	}
+}
+
+static void
+gb_tree_model_modified_cb (GbBookmark *b, gpointer data)
+{
+	GbTreeModel *m = data;
+
+	if (!m->hide_roots)
+	{
+		int rootindex = g_slist_index (m->roots, b);
+		GtkTreeIter it;
+
+		it.user_data = b;
+		it.user_data2 = GINT_TO_POINTER (rootindex);
+		it.stamp = m->stamp;
+
+		gb_tree_model_row_changed_helper (m, &it);
+	}	
+}
+
+static void
+gb_tree_model_row_inserted_helper (GbTreeModel *m, GtkTreeIter *it)
+{
+	GtkTreePath *path = gb_tree_model_get_path ((GtkTreeModel *) m, it);
+	gtk_tree_model_row_inserted ((GtkTreeModel *) m, path, it);
+	gtk_tree_path_free (path);
 }
 
 static void
 gb_tree_model_descendant_added_cb (GbFolder *f, GbFolder *p, GbBookmark *b, int pos, gpointer data)
 {
-	if (!gb_bookmark_is_alias (p)) 
+	GbTreeModel *m = data;
+	int rootindex = g_slist_index (m->roots, f);
+	GtkTreeIter it;
+
+	g_assert (rootindex >= 0);
+		
+	if ((!gb_bookmark_is_alias (p) || f == p) 
+	    && (!m->only_folders || GB_IS_FOLDER (b)))
 	{
-		GbTreeModel *m = data;
-		GtkTreeIter it;
-		GtkTreePath *path;
 		it.user_data = b;
+		it.user_data2 = GINT_TO_POINTER (rootindex);
 		it.stamp = m->stamp;
-		path = gb_tree_model_get_path ((GtkTreeModel *) m, &it);
-		gtk_tree_model_row_inserted ((GtkTreeModel *) m, path, &it);
-		gtk_tree_path_free (path);
+		
+		gb_tree_model_row_inserted_helper (m, &it);
 
 		/* probably this could be done less inneficient */
-		if (GB_IS_FOLDER (b) && !gb_bookmark_is_alias (b))
+		if (GB_IS_FOLDER (b) && !gb_bookmark_is_alias (b)) /* note that b can't be root */
 		{
 			GbBookmark *bi;
 			int i = 0;
@@ -1024,34 +1556,88 @@ gb_tree_model_descendant_added_cb (GbFolder *f, GbFolder *p, GbBookmark *b, int
 			}
 		}
 	}
+
+	/* the following is needed because the check_func value may change when a child is added */
+	if (!(m->hide_roots && f == p))
+	{
+		it.user_data = p;
+		it.user_data2 = GINT_TO_POINTER (rootindex);
+		it.stamp = m->stamp;
+
+		gb_tree_model_row_changed_helper (m, &it);
+	}
 }
 
 static void
 gb_tree_model_descendant_removed_cb (GbFolder *f, GbFolder *p, GbBookmark *b, int pos, gpointer data)
 {
 	GbTreeModel *m = data;
-	if (!gb_bookmark_is_alias (p))
+	int rootindex = g_slist_index (m->roots, f);
+	GtkTreeIter it;
+	
+	g_assert (rootindex >= 0);
+	
+	if (!(m->only_folders && !GB_IS_FOLDER (b))
+	    && (!gb_bookmark_is_alias (p) || p == f))
 	{
-		GtkTreeIter it;
 		GtkTreePath *path;
-		it.user_data = p;
-		it.stamp = m->stamp;
-		path = gb_tree_model_get_path ((GtkTreeModel *) m, &it);
-		gtk_tree_path_append_index (path, pos);
+		gint realpos;
 
-		gtk_tree_model_row_deleted ((GtkTreeModel *) m, path);
-
-		if (p->child == NULL)
+		if (m->only_folders)
 		{
-			gtk_tree_path_up (path);
-			gtk_tree_model_row_has_child_toggled ((GtkTreeModel *) m, path, &it);
+			int i = 0;
+			GbBookmark *x;
+			realpos = 0;
+			
+			for (x = p->child; x && i < pos; x = x->next)
+			{
+				++i;
+				if (GB_IS_FOLDER (x))
+				{
+					++realpos;
+				}
+			}
 		}
+		else
+		{
+			realpos = pos;
+		}
+
+		if (f == p && m->hide_roots)
+		{
+			it.user_data = m->roots;
+		}
+		else 
+		{
+			it.user_data = p;
+			it.user_data2 = GINT_TO_POINTER (rootindex);
+		}
+		
+		it.stamp = m->stamp;
+			
+		path = gb_tree_model_get_path ((GtkTreeModel *) m, &it);
+		
+		gtk_tree_path_append_index (path, realpos);
+		
+		gtk_tree_model_row_deleted ((GtkTreeModel *) m, path);
+		
+		gtk_tree_path_up (path);
+		gtk_tree_model_row_has_child_toggled ((GtkTreeModel *) m, path, &it);
+		
 		gtk_tree_path_free (path);
 	}
+
+	/* the following is needed because the check_func value may change when a child is removed */
+	if (!(m->hide_roots && f == p))
+	{
+		it.user_data = p;
+		it.user_data2 = GINT_TO_POINTER (rootindex);
+		it.stamp = m->stamp;
+
+		gb_tree_model_row_changed_helper (m, &it);
+	}
 }
 
-
-
 GbBookmark *
 gb_tree_model_get_first_root (GbTreeModel *model)
 {
@@ -1071,4 +1657,16 @@ gb_tree_model_get_set (GbTreeModel *model)
 	return model->set;
 }
 
+const GSList *
+gb_tree_model_get_roots (GbTreeModel *model)
+{
+	return model->roots;
+}
+
+void
+gb_tree_model_set_check (GbTreeModel *model, GbTreeModelCheckFunc cf, gpointer data)
+{
+	model->check_func = cf ? cf : gb_tree_model_default_check_func;
+	model->check_func_data = data;
+}
 
