@@ -29,7 +29,8 @@ typedef enum
 	GB_BAF_DEFAULT,
 	GB_BAF_NEW_TAB_OR_WINDOW,
 	GB_BAF_NEW_WINDOW,
-	GB_BAF_NEW_TAB
+	GB_BAF_NEW_TAB,
+	GB_BAF_EDITOR
 } GbBookmarkActivatedFlags;
 
 /**
