@@ -31,10 +31,16 @@
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkliststore.h>
 #include <gtk/gtktreeviewcolumn.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkdialog.h>
+#include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtkprogressbar.h>
+
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-dialog-util.h>
 #include <libgnomeui/gnome-dialog.h>
-#include <gtk/gtkprogressbar.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
 /* WHACK WHACK WHACK FIXME please
