@@ -824,7 +824,8 @@ embed_popup_reload_frame_cmd (BonoboUIComponent *uic,
                       	      GaleonEmbedPopup *popup, 
                               const char* verbname)
 {
-	/* FIXME implement */
+	galeon_embed_reload(popup->priv->embed,
+			   EMBED_RELOAD_FRAME);
 }
 
 static void 
