@@ -1,11 +1,9 @@
 #define CONF_MOUSE_WHEEL_NOKEY_ACTION "/apps/galeon/UI/Mouse/wheel_nokey_action"
 #define CONF_MOUSE_WHEEL_NOKEY_DEF_STEP "/apps/galeon/UI/Mouse/wheel_nokey_default_step"
 #define CONF_MOUSE_WHEEL_NOKEY_STEP "/apps/galeon/UI/Mouse/wheel_nokey_step"
-#define CONF_MOUSE_WHEEL_STEP_SIZE "/apps/galeon/UI/Mouse/wheel_step_size"
 #define CONF_NETWORK_SOCKS_PROXY_VERSION "/apps/galeon/Advanced/Network/socks_proxy_version"
 #define CONF_NETWORK_PROXY_KA "/apps/galeon/Advanced/Network/proxy_keepalive"
 #define CONF_NETWORK_DISK_CACHE "/apps/galeon/Advanced/Network/disk_cache_size"
-#define CONF_NETWORK_MEMORY_CACHE "/apps/galeon/Advanced/Network/mem_cache_size"
 #define CONF_NETWORK_CACHE_COMPARE "/apps/galeon/Advanced/Network/cache_compare"
 #define CONF_NETWORK_USER_AGENT "/apps/galeon/Advanced/Network/user_agent"
 #define CONF_NETWORK_HTTP_VERSION "/apps/galeon/Advanced/Network/http_version"
