@@ -22,6 +22,7 @@
 
 #include "galeon-embed.h"
 #include "galeon-marshal.h"
+#include <string.h>
 
 #if !defined (ENABLE_MOZILLA_EMBED) && !defined (ENABLE_GTKHTML_EMBED)
 #error "At least one embed needs to be enabled"
@@ -56,6 +57,7 @@ enum
 	ZOOM_CHANGE,
 	FAVICON,
 	POPUP_BLOCKED,
+	CONTEXTMENU,
 	LAST_SIGNAL
 };
 
@@ -271,6 +273,16 @@ galeon_embed_base_init (gpointer g_class)
                               G_TYPE_NONE,
                               1,
 			      G_TYPE_STRING);
+	galeon_embed_signals[CONTEXTMENU] =
+                g_signal_new ("ge_contextmenu",
+                              GALEON_EMBED_TYPE,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GaleonEmbedClass, contextmenu),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__OBJECT,
+                              G_TYPE_NONE,
+                              1,
+			      GALEON_EMBED_EVENT_TYPE);
 	initialized = TRUE;
 	}
 }
@@ -570,6 +582,13 @@ galeon_embed_shistory_copy (GaleonEmbed *source,
         return klass->shistory_copy (source, dest, back_history, forward_history, set_current);
 }
 
+gresult
+galeon_embed_shistory_clear (GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+	return klass->shistory_clear (embed);
+}
+
 gresult
 galeon_embed_scroll (GaleonEmbed *embed, 
                      EmbedScrollDirection direction)
@@ -642,3 +661,94 @@ galeon_embed_print_preview_navigate (GaleonEmbed *embed,
 	return klass->print_preview_navigate (embed, navType, pageNum);
 }
 
+gresult
+galeon_embed_get_stylesheets(GaleonEmbed *embed, GList **retSheets)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->get_stylesheets(embed, retSheets);
+}
+gresult
+galeon_embed_get_selected_stylesheet(GaleonEmbed *embed,
+				     EmbedStyleSheet **retSheet)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->get_selected_stylesheet(embed, retSheet);
+}
+
+gresult
+galeon_embed_set_stylesheet(GaleonEmbed *embed, EmbedStyleSheet *sheet)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->set_stylesheet(embed, sheet);
+}
+
+gresult
+galeon_embed_apply_user_stylesheet(GaleonEmbed *embed, const gchar *sheetfile,
+				   EmbedStyleSheet **retSheet)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->apply_user_stylesheet(embed, sheetfile, retSheet);
+}
+
+gresult
+galeon_embed_remove_user_stylesheet(GaleonEmbed *embed, EmbedStyleSheet *sheet)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->remove_user_stylesheet(embed, sheet);
+}
+
+void
+galeon_embed_stylesheet_free (EmbedStyleSheet *ess)
+{
+	if (ess)
+	{
+		g_free (ess->name);
+		g_free (ess);
+	}
+}
+
+void
+galeon_embed_stylesheet_list_free (GList *l)
+{
+	GList *li;
+	for (li = l; li; li = li->next)
+	{
+		galeon_embed_stylesheet_free (li->data);
+	}
+	g_list_free (l);
+}
+
+gboolean
+galeon_embed_stylesheet_compare	(EmbedStyleSheet *ess1, EmbedStyleSheet *ess2)
+{
+	if (ess2 == ess1)
+	{
+		return TRUE;
+	}
+	else if (ess1 && ess2)
+	{
+		if (ess1->name && ess2->name && !strcmp (ess1->name, ess2->name))
+		{
+			return TRUE;
+		}
+		if (ess2->type != ess1->type
+		    || ess2->sheet != ess1->sheet)
+		{
+			return FALSE;
+		}
+		else if ((ess1->name == ess2->name) || !ess1->name || !ess2->name)
+			 
+		{
+			return TRUE;
+		}
+		else
+		{
+			return TRUE;
+		}
+	}
+	else
+	{
+		return FALSE;
+	}
+}
+
