@@ -60,7 +60,7 @@ struct HistoryItemPrivate
 {
 	HistoryItem *host;
 	GHashTable *properties;
-	GList *children;
+	GSList *children;
 	gboolean is_host;
 };
 
@@ -164,8 +164,6 @@ global_history_class_init (GlobalHistoryClass *klass)
 			      G_TYPE_POINTER);
 }
 
-GlobalHistory *hack_gh;
-
 /**
  * xml_decode_entity: translate &amp; et al and return a new
  * expanded string 
@@ -361,18 +359,19 @@ static void
 history_start_element (void *ctx, const xmlChar *fullname, 
 		       const xmlChar **attrs)
 {
-	extern GlobalHistory *hack_gh;
+	xmlParserCtxtPtr  ctxt = ctx;
+	GlobalHistory    *gh   = ctxt->_private;
 
 	/* parse element */
 	if (xmlStrcmp (fullname, "host") == 0 && attrs != NULL)
 	{
 		/* <host> element */
-		history_parse_host_element (hack_gh, attrs);
+		history_parse_host_element (gh, attrs);
 	}
 	else if (xmlStrcmp (fullname, "item") == 0 && attrs != NULL)
 	{
 		/* <item> element */
-		history_parse_item_element (hack_gh, attrs);
+		history_parse_item_element (gh, attrs);
 	}
 	else if (xmlStrcmp (fullname, "history") == 0)
 	{
@@ -422,8 +421,6 @@ history_load (GlobalHistory *gh)
         gchar *confdir;
         gchar *histfile;
 
-	hack_gh = gh;
-
 	/* build the filename */
         confdir = g_build_filename (g_get_home_dir (), GALEON_DIR, NULL);
 	histfile = g_build_filename (confdir, "history.xml", NULL);
@@ -442,7 +439,7 @@ history_load (GlobalHistory *gh)
 	/* load the file if we can access it */
 	if (access (histfile, F_OK) != -1)
 	{
-		xmlSAXParseFile (&parser, histfile, TRUE);
+		xmlSAXParseFileWithData (&parser, histfile, TRUE, gh);
 	}
 	
 	/* free allocated strings */
@@ -581,23 +578,17 @@ history_periodic_save_cb (GlobalHistoryPrivate *ghpriv)
 /**
  * history_item_free_cb: free one history item
  */
-static gboolean
+static void
 history_item_free_cb (gpointer key, gpointer value, GlobalHistory *gh)
 {
 	HistoryItem *hi = (HistoryItem *)value;
-	HistoryItem *host = NULL;
 
 	if (!hi->priv->is_host)
 	{
-		host = history_get_host (gh, hi->url);
-	}
-
-	if (host)
-	{
-		if (g_list_find (host->priv->children, hi))
+		HistoryItem *host = hi->priv->host;
+		if (host) /* I think host can't be NULL, but I'm not sure */
 		{
-			host->priv->children = g_list_remove
-				(host->priv->children, hi);
+			host->priv->children = g_slist_remove (host->priv->children, hi);
 		}
 	}
 	
@@ -612,8 +603,6 @@ history_item_free_cb (gpointer key, gpointer value, GlobalHistory *gh)
 	g_free (hi->title);
 
 	g_hash_table_destroy (hi->priv->properties);
-
-	return TRUE;
 }
 
 static void
@@ -810,7 +799,7 @@ history_add_item (GlobalHistory *gh,
 	/* first item of host */
 	host->first = first;
 	host->last = last;
-	host->priv->children = g_list_append (host->priv->children, hi);
+	host->priv->children = g_slist_append (host->priv->children, hi);
 
 	/* add to the table */
         g_hash_table_insert (gh->priv->history_sites, 
@@ -979,24 +968,34 @@ global_history_remove_url (GlobalHistory *gh,
 void
 global_history_clear (GlobalHistory *gh)
 {
-	/* Destroy history hash table */
-	g_hash_table_foreach_remove (gh->priv->history_sites, 
-				     (GHRFunc)history_item_free_cb, 
-				     gh);
-
-	/* destroy hosts hash table */
-	g_hash_table_foreach_remove (gh->priv->history_hosts, 
-				     (GHRFunc)history_item_free_cb, 
-				     gh);
+	GlobalHistoryPrivate *p = gh->priv;
+	GHashTable *old_history_sites = p->history_sites;
+	GHashTable *old_history_hosts = p->history_hosts;
 
 	/* no last_visited_site */
-	gh->priv->last_visited_site = NULL;
+	p->last_visited_site = NULL;
+
+	p->history_hosts = g_hash_table_new (g_str_hash, g_str_equal);
+	p->history_sites = g_hash_table_new (g_str_hash, g_str_equal);
 
 	global_history_emit_changed_data_changed (gh);
 
 	/* clear the file */
 	gh->priv->history_dirty = TRUE;
 	history_save (gh->priv);
+
+	/* Destroy old history hash table */
+	g_hash_table_foreach (old_history_sites, 
+			      (GHFunc)history_item_free_cb, 
+			      gh);
+	g_hash_table_destroy (old_history_sites);
+
+	/* destroy old hosts hash table */
+	g_hash_table_foreach (old_history_hosts, 
+			      (GHFunc)history_item_free_cb, 
+			      gh);
+	g_hash_table_destroy (old_history_hosts);
+
 }
 
 /**
@@ -1122,7 +1121,7 @@ global_history_get_urls_list (GlobalHistory *gh,
 			      HistoryFilter *filter)
 {
 	GList *list = NULL;
-	GList *l;
+	GSList *l;
 	
 	g_return_val_if_fail (item != NULL, NULL);
 	g_return_val_if_fail (item->priv != NULL, NULL);
