@@ -168,6 +168,12 @@ extern "C" {
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkprogress.h>
 #include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkliststore.h>
+#include <gtk/gtkwindow.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtkcellrenderertext.h>
 #include <libgnome/gnome-exec.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-config.h>
@@ -289,11 +295,14 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 }
 
 /* nsILocalFile promptForSaveToFile (in nsISupports aWindowContext, in wstring aDefaultFile, in wstring aSuggestedFileExtension); */
-NS_IMETHODIMP GContentHandler::
-		PromptForSaveToFile(nsISupports *aWindowContext,
-				    const PRUnichar *aDefaultFile,
-				    const PRUnichar *aSuggestedFileExtension,
-				    nsILocalFile **_retval)
+NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
+#if MOZILLA_SNAPSHOT > 10
+						   nsIHelperAppLauncher *aLauncher,
+#endif
+						   nsISupports *aWindowContext,
+						   const PRUnichar *aDefaultFile,
+						   const PRUnichar *aSuggestedFileExtension,
+						   nsILocalFile **_retval)
 {
 	nsresult rv;
 
