@@ -39,10 +39,6 @@
 #define G_FILEPICKER_CONTRACTID "@mozilla.org/filepicker;1"
 #define G_FILEPICKER_CLASSNAME "Galeon's File Picker Implementation"
 
-class nsIFactory;
-
-extern nsresult NS_NewFilePickerFactory(nsIFactory** aFactory);
-
 /* Header file */
 class GFilePicker : public nsIFilePicker
 {
