@@ -68,12 +68,8 @@
 #include "nsCWebBrowser.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
-#if MOZILLA_SNAPSHOT > 7
 #include "nsIDOMPopupBlockedEvent.h"
 #include "nsIDOMNSDocument.h"
-#else
-#include "nsIDOMNSHTMLDocument.h"
-#endif
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDOMHTMLCollection.h"
 #include "nsIDOMHTMLElement.h"
@@ -91,10 +87,7 @@
 
 GaleonWrapper::GaleonWrapper () :
 			mFaviconEventListener(new GDOMEventListener())
-#if MOZILLA_SNAPSHOT > 7
 			,mPopupEventListener(new GDOMPopupEventListener())
-#endif
-
 {
 }
 
@@ -141,13 +134,11 @@ nsresult GaleonWrapper::Init (GtkMozEmbed *mozembed)
 			                        mFaviconEventListener, PR_FALSE);
 	if (NS_FAILED(result)) g_warning ("AddFaviconEventListener failed!");
 
-#if MOZILLA_SNAPSHOT > 7
 	// set up popup block listener
 	mPopupEventListener->Init(GALEON_EMBED(mozembed));
 	result = mEventTarget->AddEventListener(NS_LITERAL_STRING("DOMPopupBlocked"),
 						mPopupEventListener, PR_FALSE);
 	if (NS_FAILED(result)) g_warning ("AddPopupEventListener failed!");
-#endif
 
 	// set up global history
 	nsCOMPtr<nsIDocShell> DocShell;
@@ -287,11 +278,8 @@ nsresult GaleonWrapper::Destroy ()
 {
 	mEventTarget->RemoveEventListener(NS_LITERAL_STRING("DOMLinkAdded"),
 					  mFaviconEventListener, PR_FALSE);
-
-#if MOZILLA_SNAPSHOT > 7
 	mEventTarget->RemoveEventListener(NS_LITERAL_STRING("DOMPopupBlocked"),
 					  mPopupEventListener, PR_FALSE);
-#endif
 
 	mEventTarget = nsnull;
       	mWebBrowser = nsnull;
@@ -677,16 +665,9 @@ nsresult GaleonWrapper::GetDocumentTitle (char **title)
 	nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
 
-#if MOZILLA_SNAPSHOT > 6
 	nsAutoString t;
 	doc->GetDocumentTitle(t);
 	*title = g_strdup (NS_ConvertUCS2toUTF8(t).get());
-#else
-	const nsString* t;
-	t = doc->GetDocumentTitle();
-
-	*title = g_strdup (NS_ConvertUCS2toUTF8(*t).get());
-#endif
 
 	return NS_OK;
 }
@@ -1171,13 +1152,8 @@ nsresult GaleonWrapper::GetLastModified (gchar **ret)
 	result = GetDOMDocument (getter_AddRefs(DOMDocument));
 	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
-#if MOZILLA_SNAPSHOT > 7
 	nsCOMPtr<nsIDOMNSDocument> doc = do_QueryInterface(DOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
-#else
-	nsCOMPtr<nsIDOMNSHTMLDocument> doc = do_QueryInterface(DOMDocument);
-	if(!doc) return NS_ERROR_FAILURE;
-#endif
 
 	nsAutoString value;
 	doc->GetLastModified(value);
@@ -1616,7 +1592,6 @@ GDOMEventListener::HandleEvent (nsIDOMEvent* aDOMEvent)
 }
 
 //------------------------------------------------------------------------------
-#if MOZILLA_SNAPSHOT > 7
 NS_IMPL_ISUPPORTS1(GDOMPopupEventListener, nsIDOMEventListener)
 
 NS_IMETHODIMP
@@ -1640,5 +1615,3 @@ GDOMPopupEventListener::HandleEvent (nsIDOMEvent* aDOMEvent)
 
 	return rv;
 }
-#endif
-
