@@ -32,20 +32,11 @@
 #include "PrintingPromptService.h"
 #include "ProgressListener.h"
 #include "SideBarProxy.h"
-#if MOZILLA_SNAPSHOT < 7
-#include "PromptService.h"
-#endif
 
 #include "ExternalProtocolService.h"
 #include "gUrlCID.h"
 #include "nsAboutRedirector.h"
 
-#include "GnomeHelpProtocolHandler.h"
-#include "GHelpProtocolHandler.h"
-#include "InfoProtocolHandler.h"
-#include "ManProtocolHandler.h"
-#include "TOCProtocolHandler.h"
-
 #include <nsIGenericFactory.h>
 #include <nsIComponentRegistrar.h>
 #include <nsCOMPtr.h>
@@ -53,9 +44,6 @@
 
 #include <glib.h>
 
-#if MOZILLA_SNAPSHOT < 7
-NS_GENERIC_FACTORY_CONSTRUCTOR(CPromptService)
-#endif
 NS_GENERIC_FACTORY_CONSTRUCTOR(GProgressListener)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GFilePicker)
 NS_GENERIC_FACTORY_CONSTRUCTOR(JSConsoleService)
@@ -69,12 +57,6 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(GIRCProtocolHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GFtpProtocolHandler)
 
 static const nsModuleComponentInfo sAppComps[] = {
-#if MOZILLA_SNAPSHOT < 7
-	{	G_PROMPTSERVICE_CLASSNAME,
-		G_PROMPTSERVICE_CID,
-		G_PROMPTSERVICE_CONTRACTID,
-		CPromptServiceConstructor	},
-#endif
 	{
 		G_PROGRESSDIALOG_CLASSNAME,
 		G_PROGRESSDIALOG_CID,
@@ -197,11 +179,6 @@ static const nsModuleComponentInfo sModuleComps[] = {
 		 G_EXTERNALPROTOCOLSERVICE_CID,
 		 NS_EXTERNALPROTOCOLSERVICE_CONTRACTID
 	},
-	{
-		 G_GNOMEHELPURL_CLASSNAME,
-		 G_GNOMEHELPURL_CID,
-		 G_GNOMEHELPURL_CONTRACTID
-	},
 	{
 		 G_ABOUT_TOPHER_MODULE_CLASSNAME,
 		 G_ABOUT_REDIRECTOR_MODULE_CID,
@@ -211,35 +188,6 @@ static const nsModuleComponentInfo sModuleComps[] = {
 
 static const int sNumModuleComps = sizeof(sModuleComps) / sizeof(nsModuleComponentInfo);
 
-static const nsModuleComponentInfo sHelpComps[] = {
-	{
-		 G_TOC_HANDLER_CLASSNAME,
-		 G_TOC_PROTOCOL_CID,
-		 G_TOC_HANDLER_CONTRACTID
-	},
-	{
-		G_MAN_HANDLER_CLASSNAME,
-		G_MAN_PROTOCOL_CID,
-		G_MAN_HANDLER_CONTRACTID
-	},
-	{
-		G_INFO_HANDLER_CLASSNAME,
-		G_INFO_PROTOCOL_CID,
-		G_INFO_HANDLER_CONTRACTID
-	},
-	{
-		G_GNOMEHELP_HANDLER_CLASSNAME,
-		G_GNOMEHELP_PROTOCOL_CID,
-		G_GNOMEHELP_HANDLER_CONTRACTID
-	},
-	{
-		G_GHELP_HANDLER_CLASSNAME,
-		G_GHELP_PROTOCOL_CID,
-		G_GHELP_HANDLER_CONTRACTID
-	}
-};
-
-static const int sNumHelpComps = sizeof(sHelpComps) / sizeof(nsModuleComponentInfo);
 
 static NS_DEFINE_CID(kMailtoUrlCID, NS_MAILTOURL_CID);
 
@@ -400,60 +348,3 @@ mozilla_register_MailtoProtocolHandler (void)
 	}
 	return retVal;
 }
-
-/**
- * mozilla_register_gHelpProtocolHandlers: Register gHelp Protocol Handlers
- * eg: man, info, etc.
- */
-extern "C" gboolean
-mozilla_register_gHelpProtocolHandlers (void)
-{
-	gboolean ret = TRUE;
-	nsresult rv;
-
-	nsCOMPtr<nsILocalFile> dllFile;
-	rv = NS_NewLocalFile(NS_ConvertUTF8toUCS2(sModulePath), PR_TRUE,
-			     getter_AddRefs (dllFile));
-	if (NS_FAILED(rv) || !dllFile) return FALSE;
-
-	nsCOMPtr<nsIComponentRegistrar> cr;
-	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
-	if (NS_FAILED(rv) || !cr) return FALSE;
-
-	for (int i = 0 ; i < sNumHelpComps ; i++)
-	{
-		rv = cr->RegisterFactoryLocation(sHelpComps[i].mCID,
-						 sHelpComps[i].mDescription,
-						 sHelpComps[i].mContractID,
-						 dllFile, nsnull, nsnull);
-		if (NS_FAILED(rv)) ret = FALSE;
-	}
-	return ret;
-}
-
-/**
- * mozilla_unregister_gHelpProtocolHandlers: Unregister gHelp Protocol Handlers
- */
-extern "C" gboolean 
-mozilla_unregister_gHelpProtocolHandlers (void)
-{
-	gboolean ret = TRUE;
-	nsresult rv;
-
-	nsCOMPtr<nsILocalFile> dllFile;
-	rv = NS_NewLocalFile(NS_ConvertUTF8toUCS2(sModulePath), PR_TRUE,
-			     getter_AddRefs (dllFile));
-	if (NS_FAILED(rv) || !dllFile) return FALSE;
-
-	nsCOMPtr<nsIComponentRegistrar> cr;
-	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
-	if (NS_FAILED(rv) || !cr) return FALSE;
-
-	for (int i = 0 ; i < sNumHelpComps ; i++)
-	{
-		rv = cr->UnregisterFactoryLocation(sHelpComps[i].mCID,
-						   dllFile);
-		if (NS_FAILED(rv)) ret = FALSE;
-	}
-	return ret;
-}
