@@ -134,6 +134,36 @@ NS_IMETHODIMP GMyportalProtocolHandler::NewURI(const nsACString &aSpec,
 	return rv;
 }
 
+static NS_IMETHODIMP
+OpenMyportalStylesheet (nsIURI *aURI, nsIChannel **aChannel)
+{
+	nsresult rv;
+
+	gchar *cssPath = gul_general_user_file ("myportal.css", FALSE);
+	g_return_val_if_fail (cssPath != NULL, NS_ERROR_FAILURE);
+
+	// open the path as file since it could be relative path (thanks to
+	// gul_general_user_file) and simply prepending 'file://' would make 
+	// an invalid URI
+	nsCOMPtr<nsILocalFile> cssFile;
+	rv = NS_NewLocalFile(NS_ConvertUTF8toUCS2(cssPath), PR_TRUE,
+			     getter_AddRefs(cssFile));
+	g_free (cssPath);
+
+	nsCOMPtr<nsIURI> cssURI;
+	rv = NS_NewFileURI(getter_AddRefs(cssURI), cssFile);
+	if (NS_FAILED(rv)) return rv;
+
+	nsCOMPtr<nsIInputStream> stream;
+	rv = NS_OpenURI(getter_AddRefs(stream), cssURI);
+	if (NS_FAILED(rv)) return rv;
+
+	rv = NS_NewInputStreamChannel(aChannel, aURI,
+				      stream, NS_LITERAL_CSTRING("text/css"),
+				      NS_LITERAL_CSTRING("utf-8"));
+	return rv;
+}
+
 /* nsIChannel newChannel (in nsIURI aURI); */
 NS_IMETHODIMP GMyportalProtocolHandler::NewChannel(nsIURI *aURI,
 					      nsIChannel **_retval)
@@ -149,7 +179,10 @@ NS_IMETHODIMP GMyportalProtocolHandler::NewChannel(nsIURI *aURI,
 		nsCAutoString path;
 		rv = aURI->GetPath (path);
 		if (NS_FAILED(rv)) return rv;
-		rv = CreateMyportalPage (path, aURI, _retval);
+		if (path.Equals("myportal.css"))
+			rv = OpenMyportalStylesheet (aURI, _retval);
+		else
+			rv = CreateMyportalPage (path, aURI, _retval);
 	}
 	return rv;
 }
@@ -169,10 +202,6 @@ NS_METHOD GMyportalProtocolHandler::CreateMyportalPage (const nsACString &path,
 	/* check bookmarks are loaded */
 	//	g_return_val_if_fail (bookmarks_root != NULL,NS_ERROR_FAILURE);
 	
-	/* get the stylesheet filename */
-		gchar *stylesheet_filename =
-			gul_general_user_file ("myportal.css", FALSE);
-	
 	/* open the rendering stream */
 #if USE_INTERMEDIATE_FILE
 	gchar *filename = g_strconcat (g_get_home_dir (),
@@ -183,7 +212,6 @@ NS_METHOD GMyportalProtocolHandler::CreateMyportalPage (const nsACString &path,
 	if (stream == NULL)
 	{
 		g_warning ("unable to create `%s'", filename);
-		g_free (stylesheet_filename);
 		g_free (filename);
 		return NS_ERROR_FAILURE;
 	}
@@ -200,12 +228,9 @@ NS_METHOD GMyportalProtocolHandler::CreateMyportalPage (const nsACString &path,
 #endif
 
 	/* render the complete portal */
-	gchar full_path[PATH_MAX];
 	gchar* myportal_utfstr = _("My Portal");
 	RENDER ("<html><head>\n");
-	RENDER ("<link rel=\"stylesheet\" href=\"file://");
-	RENDER (realpath (stylesheet_filename, full_path));
-	RENDER ("\" type=\"text/css\">\n");
+	RENDER ("<link rel=\"stylesheet\" href=\"myportal:myportal.css\" type=\"text/css\">\n");
 	RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=utf-8\">\n");
 	RENDER ("<title>");
 	RENDER (myportal_utfstr);
@@ -247,7 +272,6 @@ NS_METHOD GMyportalProtocolHandler::CreateMyportalPage (const nsACString &path,
 				      NS_LITERAL_CSTRING("utf-8"));
 	if (NS_FAILED(rv)) return rv;
 #endif
-	g_free (stylesheet_filename);
 	return rv;
 }
 
