@@ -494,10 +494,8 @@ mozilla_register_external_protocols (void)
 #endif
 
 	mozilla_register_MailtoProtocolHandler ();
-	mozilla_register_gHelpProtocolHandlers ();
 }
 
-#if MOZILLA_SNAPSHOT > 6
 static void
 mozilla_init_cookie_observer(void)
 {
@@ -518,7 +516,6 @@ mozilla_init_cookie_observer(void)
 			     PERM_CHANGE_NOTIFICATION,
 			     PR_FALSE);
 }
-#endif
 
 static nsresult
 getUILang(nsAString& aUILang)
@@ -593,9 +590,7 @@ mozilla_embed_shell_init (MozillaEmbedShell *mes)
 
 	mozilla_init_sidebar ();
 
-#if MOZILLA_SNAPSHOT > 6
 	mozilla_init_cookie_observer();
-#endif
 		
 	/* FIXME alert if fails */
 }
@@ -1024,7 +1019,6 @@ impl_set_permission(GaleonEmbedShell *shell,
 	nsCOMPtr<nsIPermissionManager> permissionManager =
                         do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
 
-#if MOZILLA_SNAPSHOT > 6
 	nsCOMPtr<nsIURI> uri;
 	rv = NS_NewURI(getter_AddRefs(uri), url);
 	if (NS_FAILED(rv) || !uri) return G_FAILED;
@@ -1036,10 +1030,7 @@ impl_set_permission(GaleonEmbedShell *shell,
 	#endif
 				    allow ? (PRUint32)nsIPermissionManager::ALLOW_ACTION
 				    	  : (PRUint32)nsIPermissionManager::DENY_ACTION);
-#else
-        rv = permissionManager->Add(nsDependentCString(url),
-        			    allow ? PR_TRUE : PR_FALSE, type);
-#endif
+
 	if (NS_FAILED(rv)) return G_FAILED;
 
 	return G_OK;
@@ -1074,17 +1065,12 @@ impl_list_permissions (GaleonEmbedShell *shell,
 		nsPermission->GetType(cType);
 		if (cType.Equals(gPermType[type]))
 #else
-	#if MOZILLA_SNAPSHOT > 6
 		PRUint32 cType;
-	#else
-                PRInt32 cType;
-	#endif
                 nsPermission->GetType(&cType);
                 if ((PRUint32)cType == (PRUint32)type)
 #endif
                 {
                         PermissionInfo *b = g_new0(PermissionInfo, 1);
-#if MOZILLA_SNAPSHOT > 6
 			nsCString host;
 			nsPermission->GetHost(host);
 			b->domain = g_strdup(host.get());
@@ -1102,15 +1088,6 @@ impl_list_permissions (GaleonEmbedShell *shell,
 			  default :
 			  	b->status = FALSE;
 			}
-#else
-			nsXPIDLCString host;
-                        nsPermission->GetHost(getter_Copies(host));
-                        b->domain = g_strdup(host.get());
-
-                        PRBool cap;
-                        nsPermission->GetCapability(&cap);
-			b->status = cap;
-#endif
                         *permissions = g_list_prepend(*permissions, b);
                 }
         }
@@ -1154,7 +1131,6 @@ impl_test_permission (GaleonEmbedShell *shell,
 	nsCOMPtr<nsIPermissionManager> permissionManager =
                         do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
 
-#if MOZILLA_SNAPSHOT > 6
 	nsCOMPtr<nsIURI> uri;
 	rv = NS_NewURI(getter_AddRefs(uri), url);
 	if (NS_FAILED(rv) || !uri) return G_FAILED;
@@ -1178,13 +1154,6 @@ impl_test_permission (GaleonEmbedShell *shell,
 	  	*ret_action = DENY_ACTION;
 	  	break;
 	}
-#else
-	PRBool blocked = PR_FALSE;
-        rv = permissionManager->TestForBlocking(nsDependentCString(url),
-					        type, &blocked);
-
-	*ret_action = blocked ?  DENY_ACTION : ALLOW_ACTION;
-#endif
 	if (NS_FAILED(rv)) return G_FAILED;
 
 	return G_OK;
