@@ -16,6 +16,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "galeon-config.h"
 #include "gtkmozembed.h"
 #include "gtkmozembed_internal.h"
@@ -27,6 +31,8 @@
 #include "GaleonWrapper.h"
 #include "EventContext.h"
 
+#include <libgnome/gnome-i18n.h>
+
 #include <nsIURI.h>
 #include <nsIURL.h>
 #include <nsNetUtil.h>
@@ -37,13 +43,16 @@
 #include <nsITransportSecurityInfo.h>
 #include <nsIPrintOptions.h>
 #include <nsGfxCIID.h>
+#include <nsIDOMStyleSheetList.h>
+#include <nsIDOMStyleSheet.h>
+#include <nsIDOMHTMLLinkElement.h>
 
 #include <math.h>
 
 #define WINDOWWATCHER_CONTRACTID "@mozilla.org/embedcomp/window-watcher;1"
 
-//#define DEBUG_MSG(x)
-#define DEBUG_MSG(x) g_print x
+#define DEBUG_MSG(x)
+//#define DEBUG_MSG(x) g_print x
 
 static void
 mozilla_embed_class_init (MozillaEmbedClass *klass);
@@ -157,6 +166,10 @@ impl_shistory_copy (GaleonEmbed *source,
 		    gboolean back_history,
 		    gboolean forward_history,
 		    gboolean set_current);
+
+static gresult
+impl_shistory_clear (GaleonEmbed *embed);
+
 static gresult
 impl_scroll (GaleonEmbed *embed, 
              EmbedScrollDirection direction);
@@ -190,6 +203,25 @@ impl_print_preview_navigate (GaleonEmbed *embed,
 			     EmbedPrintPreviewNavType navType,
 			     gint pageNum);
 
+static gresult
+impl_get_stylesheets(GaleonEmbed *embed,
+		     GList **retSheets);
+static gresult
+impl_get_selected_stylesheet(GaleonEmbed *embed,
+			     EmbedStyleSheet **retSheet);
+static gresult
+impl_set_stylesheet(GaleonEmbed *embed,
+		    EmbedStyleSheet *sheet);
+static gresult
+impl_apply_user_stylesheet(GaleonEmbed *embed,
+			   const gchar *sheetfile,
+			   EmbedStyleSheet **retSheet);
+static gresult
+impl_remove_user_stylesheet(GaleonEmbed *embed,
+			    EmbedStyleSheet *sheet);
+static gboolean
+stylesheet_is_alternate(nsIDOMStyleSheet *item);
+
 static void 
 mozilla_embed_connect_signals (MozillaEmbed *membed);
 static char *
@@ -217,9 +249,6 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event, Mozill
 static gint
 mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event, 
 				 MozillaEmbed *membed);
-static gint
-mozilla_embed_dom_key_press_cb (GtkMozEmbed *embed, gpointer dom_event,
-				MozillaEmbed *membed);
 static void
 mozilla_embed_size_to_cb (GtkMozEmbed *embed, gint width, gint height, MozillaEmbed *membed);
 static void
@@ -232,6 +261,10 @@ mozilla_embed_security_change_cb (GtkMozEmbed *embed,
 				  gpointer request,
                                   guint state, MozillaEmbed *membed);
 
+static gint
+mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
+		               MozillaEmbed *membed);
+
 static EmbedSecurityLevel
 mozilla_embed_security_level (MozillaEmbed *membed);
 
@@ -253,10 +286,10 @@ signal_connections[] =
         { "destroy_browser", (void *) mozilla_embed_destroy_brsr_cb      },
         { "dom_mouse_click", (void *) mozilla_embed_dom_mouse_click_cb   },
 	{ "dom_mouse_down",  (void *) mozilla_embed_dom_mouse_down_cb    },
-	{ "dom_key_press",   (void *) mozilla_embed_dom_key_press_cb	 },
         { "size_to",         (void *) mozilla_embed_size_to_cb           },
         { "new_window",      (void *) mozilla_embed_new_window_cb        },
         { "security_change", (void *) mozilla_embed_security_change_cb   },
+	{ "dom_key_down",    (void *) mozilla_embed_dom_key_down_cb      },
 
         /* terminator -- must be last in the list! */
         { NULL, NULL } 
@@ -319,6 +352,7 @@ galeon_embed_init (GaleonEmbedClass *embed_class)
 	embed_class->shistory_get_pos = impl_shistory_get_pos;
 	embed_class->shistory_go_nth = impl_shistory_go_nth;
 	embed_class->shistory_copy = impl_shistory_copy;
+	embed_class->shistory_clear = impl_shistory_clear;
 	embed_class->scroll = impl_scroll;
 	embed_class->fine_scroll = impl_fine_scroll;
 	embed_class->get_security_level = impl_get_security_level;
@@ -329,15 +363,118 @@ galeon_embed_init (GaleonEmbedClass *embed_class)
 	embed_class->print_preview_close = impl_print_preview_close;
 	embed_class->print_preview_num_pages = impl_print_preview_num_pages;
 	embed_class->print_preview_navigate = impl_print_preview_navigate;
+	embed_class->get_stylesheets = impl_get_stylesheets;
+	embed_class->get_selected_stylesheet = impl_get_selected_stylesheet;
+	embed_class->set_stylesheet = impl_set_stylesheet;
+	embed_class->apply_user_stylesheet = impl_apply_user_stylesheet;
+	embed_class->remove_user_stylesheet = impl_remove_user_stylesheet;
 }
 
+/****************************************************************
+ * Start Dirty Mouse Wheel hack
+ *
+ * This hack is needed as the mozilla embed object doesn't emit a
+ * zoom changed signal, and you can't seem to be able to capture
+ * the mouse wheel movements in the parent widgets. This is also
+ * abusing the fact that we know that gtkmozembed object is also
+ * a GtkContainer containing the real widget.
+ ****************************************************************/
+
+static gint before_zoom_val = 0;
+
+/**
+ *  This function will get called before the actual scroll-event
+ *  has been delivered to the widget, this allows us to get the current
+ *  zoom value
+ */
+static gboolean
+mozilla_embed_child_scroll_event_cb (GtkWidget    *widget, 
+				     GdkEvent     *event,
+				     MozillaEmbed *embed)
+{
+	galeon_embed_zoom_get (GALEON_EMBED (embed), &before_zoom_val);
+	return FALSE;
+}
+
+
+/**
+ *  This gets called after the scroll event has been delivered to the
+ *  widget. We can get the new zoom value, and emit the appropriate
+ *  signal if the zoom factor changed 
+ */
+static gboolean
+mozilla_embed_child_event_after_cb (GtkWidget    *widget, 
+				    GdkEvent     *event,
+				    MozillaEmbed *embed)
+{
+	int zoom;
+	if (event->type != GDK_SCROLL) return FALSE;
+
+	if (galeon_embed_zoom_get (GALEON_EMBED (embed), &zoom) == G_OK &&
+	    zoom != before_zoom_val)
+	    
+	{
+		g_signal_emit_by_name (embed, "ge_zoom_change", zoom);
+	}
+	return FALSE;
+}
+
+/**
+ *  Hack: We know that the parent class is really GtkBin object, so
+ *  hook onto the "add" signal and connect appropriate signals for the
+ *  child
+ */
+static void
+mozilla_embed_add (GtkContainer *container, GtkWidget *widget)
+{
+	MozillaEmbed *embed = MOZILLA_EMBED (container);
+
+	GTK_CONTAINER_CLASS(parent_class)->add (container, widget);
+
+	g_signal_connect (widget, "scroll-event",
+			  G_CALLBACK (mozilla_embed_child_scroll_event_cb), embed);
+
+	g_signal_connect (widget, "event-after",
+			  G_CALLBACK (mozilla_embed_child_event_after_cb), embed);
+}
+
+
+/**
+ *  If the child is removed, remove the signals that we attached 
+ */
+static void
+mozilla_embed_remove (GtkContainer *container, GtkWidget *widget)
+{
+	MozillaEmbed *embed = MOZILLA_EMBED (container);
+
+	g_signal_handlers_disconnect_by_func 
+		(widget,
+		 (gpointer)mozilla_embed_child_scroll_event_cb,
+		 embed);
+
+	g_signal_handlers_disconnect_by_func 
+		(widget,
+		 (gpointer)mozilla_embed_child_event_after_cb,
+		 embed);
+
+	GTK_CONTAINER_CLASS(parent_class)->remove (container, widget);
+}
+
+/*************************************************************
+ * End dirty mouse wheel zoom hack
+ *************************************************************/
+
 static void
 mozilla_embed_class_init (MozillaEmbedClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
      	GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass); 
+	GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
+
 	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
 	
+	container_class->add    = mozilla_embed_add;
+	container_class->remove = mozilla_embed_remove;
 	gtk_object_class->destroy = mozilla_embed_destroy;
         object_class->finalize = mozilla_embed_finalize;
 }
@@ -731,9 +868,16 @@ static gresult
 impl_reload (GaleonEmbed *embed, 
              EmbedReloadFlags flags)
 {
-	guint32 mflags;
+	if ((flags & EMBED_RELOAD_FRAME))
+	{
+		GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+		g_return_val_if_fail(wrapper != NULL, G_FAILED);
 
-	mflags = GTK_MOZ_EMBED_FLAG_RELOADNORMAL;
+		nsresult rv = wrapper->ReloadDocument();
+		return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
+	}
+
+	guint32 mflags = GTK_MOZ_EMBED_FLAG_RELOADNORMAL;
 	
 	if ((flags & EMBED_RELOAD_BYPASSCACHE) &&
 	    (flags & EMBED_RELOAD_BYPASSPROXY))
@@ -1067,6 +1211,21 @@ impl_shistory_copy (GaleonEmbed *source,
 	return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
 }
 
+static gresult
+impl_shistory_clear (GaleonEmbed *embed)
+{
+	nsresult rv;
+	GaleonWrapper *wrapper;
+	
+	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	
+	rv = wrapper->ClearHistory();
+
+	return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
+}
+
+
 static gresult
 impl_scroll (GaleonEmbed *embed, 
              EmbedScrollDirection direction)
@@ -1399,7 +1558,6 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event,
 				  MozillaEmbed *membed)
 {
 	GaleonEmbedEvent *info;
-	EventContext event_context;
 	gint return_value = 0;
 	GaleonWrapper *wrapper;
 	nsresult result;
@@ -1409,12 +1567,13 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event,
 	
 	info = galeon_embed_event_new();
 	
-	event_context.Init((nsIDOMEvent*)dom_event, wrapper);
-        result = event_context.GetMouseEventInfo(info);
+	EventContext ctx;
+	ctx.Init(wrapper);
+        result = ctx.GetMouseEventInfo(static_cast<nsIDOMMouseEvent*>(dom_event), info);
 	if (NS_SUCCEEDED(result))
 	{
 		nsCOMPtr<nsIDOMDocument> domDoc;
-		result = event_context.GetTargetDocument(getter_AddRefs(domDoc));
+		result = ctx.GetTargetDocument(getter_AddRefs(domDoc));
 		if (NS_SUCCEEDED(result))
 		{
 			result = wrapper->PushTargetDocument(domDoc);
@@ -1438,7 +1597,6 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 				 MozillaEmbed *membed)
 {
 	GaleonEmbedEvent *info;
-	EventContext event_context;
 	gint return_value = 0;
 	GaleonWrapper *wrapper;
 	nsresult result;
@@ -1448,12 +1606,13 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	
 	info = galeon_embed_event_new ();
 	
-	event_context.Init ((nsIDOMEvent*)dom_event, wrapper);
-        result = event_context.GetMouseEventInfo (info);
+	EventContext ctx;
+	ctx.Init (wrapper);
+        result = ctx.GetMouseEventInfo (static_cast<nsIDOMMouseEvent*>(dom_event), info);
 	if (NS_SUCCEEDED(result))
 	{
 		nsCOMPtr<nsIDOMDocument> domDoc;
-		result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
+		result = ctx.GetTargetDocument (getter_AddRefs(domDoc));
 		if (NS_SUCCEEDED(result))
 		{
 			result = wrapper->PushTargetDocument (domDoc);
@@ -1473,11 +1632,63 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 }
 
 static gint
-mozilla_embed_dom_key_press_cb (GtkMozEmbed *embed, gpointer dom_event,
-				MozillaEmbed *membed)
+mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
+		               MozillaEmbed *membed)
 {
-	g_print("Hello keypress\n");
-	return false;
+	nsCOMPtr<nsIDOMKeyEvent> ev = static_cast<nsIDOMKeyEvent*>(dom_event);
+	if (!ev)
+	{
+		return FALSE;
+	}
+
+	GaleonWrapper *wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
+	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+
+	GaleonEmbedEvent *info;
+	info = galeon_embed_event_new ();
+
+	gboolean ret = FALSE;
+
+	// Just check for Shift-F10 so that we know to popup the context menu.
+	//
+	// The DOM_VK_* keycodes are not compatible with the keycodes defined
+	// in GDK, so making a generic dom_key_down signal is probably not
+	// worth the trouble.
+
+	nsresult rv;
+	EventContext ctx;
+	ctx.Init (wrapper);
+	rv = ctx.GetKeyEventInfo (ev, info);
+	if (NS_SUCCEEDED(rv) &&
+	    (info->keycode == nsIDOMKeyEvent::DOM_VK_F10 &&
+	     info->modifier == GDK_SHIFT_MASK))
+	{
+		// Translate relative coordinates to absolute values, and try
+		// to avoid covering links by adding a little offset.
+
+		int x, y;
+		gdk_window_get_origin (GTK_WIDGET(membed)->window, &x, &y);
+		info->x += x + 6;	
+		info->y += y + 6;
+
+		nsCOMPtr<nsIDOMDocument> doc;
+		rv = ctx.GetTargetDocument (getter_AddRefs(doc));
+		if (NS_SUCCEEDED(rv))
+		{
+			rv = wrapper->PushTargetDocument (doc);
+			if (NS_SUCCEEDED(rv))
+			{
+				g_signal_emit_by_name (membed, "ge_contextmenu", info);
+				wrapper->PopTargetDocument ();
+
+				// return TRUE to stop further event handling
+				ret = TRUE;
+			}
+		}
+	}
+
+	g_object_unref (info);
+	return ret;
 }
 
 static void
@@ -1579,3 +1790,287 @@ mozilla_embed_security_level (MozillaEmbed *membed)
 	return level;
 }
 
+static gint
+stylesheet_find_func (gconstpointer a, gconstpointer b)
+{
+	const EmbedStyleSheet *sheet = (const EmbedStyleSheet*)a;
+	const char * name = (const char*)b;
+
+	return strcmp (sheet->name, name);;
+}
+
+
+static gresult
+impl_get_stylesheets(GaleonEmbed *embed,
+		     GList **retSheets)
+{
+	nsresult result;
+
+        GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+        g_return_val_if_fail(wrapper != NULL, G_FAILED);
+
+	nsCOMPtr<nsIDOMStyleSheetList> list;
+	result = wrapper->GetStyleSheets(getter_AddRefs(list));
+	if (NS_FAILED(result) || !list) return G_FAILED;
+
+	PRUint32 count(0); 
+	result = list->GetLength(&count);
+	if (NS_FAILED(result)) return G_FAILED;
+
+	//Sometimes Mozilla wants to return -1 as an unsigned value...
+	if ((count+1) == 0) count = 0;
+
+	gint found(0);
+	gint found_named(0);
+	GList *csslist(NULL);
+	for (PRUint32 i = 0; i < count; i++)
+	{
+		nsCOMPtr<nsIDOMStyleSheet> item;
+		result = list->Item(i, getter_AddRefs(item));
+		if (NS_FAILED(result) || !item) continue;
+
+		found++;
+
+		nsAutoString string;
+		result = item->GetTitle(string);
+		if (NS_FAILED(result)) continue;
+
+		if (string.IsEmpty()) continue;
+
+		const char * name = NS_ConvertUCS2toUTF8(string).get();
+
+		if (g_list_find_custom (csslist, name, stylesheet_find_func))
+		{
+			continue;
+		}
+
+		if (!stylesheet_is_alternate (item))
+		{
+			found_named++;
+		}
+
+		EmbedStyleSheet *sheet = g_new0(EmbedStyleSheet, 1);
+		sheet->name  = g_strdup(name);
+		sheet->sheet = item;
+		sheet->type  = STYLESHEET_NAMED;
+
+		csslist = g_list_append(csslist, sheet);
+	}
+
+	if (found > 0 && found_named == 0)
+	{
+		/* Add in the "Default" style if we found stylesheets but
+		 * we didn't find any (non-alternate) named ones) */
+		EmbedStyleSheet *sheet = g_new0(EmbedStyleSheet, 1);
+		sheet->name  = g_strdup(_("Default"));
+		sheet->sheet = NULL;
+		sheet->type  = STYLESHEET_BASIC;
+
+		csslist = g_list_append(csslist, sheet);
+	}
+
+	if (found > 0)
+	{
+		/* prepend None item if any sheets were found */
+		EmbedStyleSheet *sheet = g_new0(EmbedStyleSheet, 1);
+		sheet->name = g_strdup(_("None"));
+		sheet->sheet = NULL;
+		sheet->type = STYLESHEET_NONE;
+
+		csslist = g_list_prepend(csslist, sheet);
+	}
+
+	(*retSheets) = csslist;
+	return G_OK;
+}
+
+static gresult
+impl_get_selected_stylesheet(GaleonEmbed *embed,
+			     EmbedStyleSheet **retSheet)
+{
+	gboolean found = FALSE;
+	nsresult result;
+
+        GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+        g_return_val_if_fail(wrapper != NULL, G_FAILED);
+	
+	nsCOMPtr<nsIDOMStyleSheetList> list;
+	result = wrapper->GetStyleSheets(getter_AddRefs(list));
+	if (NS_FAILED(result) || !list) return G_FAILED;
+	
+	PRUint32 count, i; 
+	result = list->GetLength(&count);
+	if (NS_FAILED(result)) return G_FAILED;
+
+        //Sometimes Mozilla wants to return -1 as an unsigned value...
+        if ((count+1) == 0) count = 0;
+
+	for (i = 0; i < count; i++)
+	{
+		nsCOMPtr<nsIDOMStyleSheet> item;
+		result = list->Item(i, getter_AddRefs(item));
+		if (NS_FAILED(result) || !item) continue;
+
+		nsAutoString string;
+		result = item->GetTitle(string);
+		if (NS_FAILED(result)) continue;
+
+		PRBool disabled;
+		item->GetDisabled(&disabled);
+		
+		if (string.IsEmpty())
+		{
+			/* fine, but let's try to get something more sensible */
+			if (disabled == PR_FALSE) found = TRUE;
+			continue;
+		}
+		
+		if (disabled == PR_FALSE)
+		{
+			EmbedStyleSheet *sheet =
+				g_new0(EmbedStyleSheet, 1);
+			sheet->name =
+				g_strdup(NS_ConvertUCS2toUTF8(string).get());
+			sheet->sheet = item;
+			sheet->type = STYLESHEET_NAMED;
+
+			(*retSheet) = sheet;
+			return G_OK;
+		}
+	}
+
+	if (found)
+	{
+		EmbedStyleSheet *sheet = g_new0(EmbedStyleSheet, 1);
+		sheet->type = STYLESHEET_BASIC;
+		(*retSheet) = sheet;
+		return G_OK;
+	}
+	else
+	{
+		(*retSheet) =  NULL;
+		return G_FAILED;
+	}
+}
+
+static gresult
+impl_set_stylesheet(GaleonEmbed *embed,
+		    EmbedStyleSheet *sheet)
+{
+	nsresult result;
+
+        GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+        g_return_val_if_fail(wrapper != NULL, G_FAILED);
+	
+	nsCOMPtr<nsIDOMStyleSheetList> list;
+	result = wrapper->GetStyleSheets(getter_AddRefs(list));
+	if (NS_FAILED(result) || !list) return G_FAILED;
+	
+	PRUint32 count, i; 
+	result = list->GetLength(&count);
+	if (NS_FAILED(result)) return G_FAILED;
+
+        //Sometimes Mozilla wants to return -1 as an unsigned value...
+        if ((count+1) == 0) count = 0;
+
+	for (i = 0; i < count; i++)
+	{
+		nsCOMPtr<nsIDOMStyleSheet> item;
+		result = list->Item(i, getter_AddRefs(item));
+		if (NS_FAILED(result) || !item) continue;
+		
+		nsAutoString string;
+		result = item->GetTitle(string);
+		if (NS_FAILED(result)) continue;
+
+		const char* title = NS_ConvertUCS2toUTF8(string).get();
+
+		/*
+		 * if STYLESHEET_NONE is requtested, disable all.
+		 * if STYLESHEET_BASIC is requested, enable only unnamed sheets
+		 * if STYLESHEET_NAMED is requested load all sheets that have
+		 *    that name, and all unnamed ones
+		 */
+		if      (item == sheet->sheet || strcmp (title, sheet->name) == 0)
+		{
+			item->SetDisabled(FALSE);
+		}
+		else if (sheet->type != STYLESHEET_NONE && string.IsEmpty())
+		{
+			item->SetDisabled(FALSE);
+		}
+		else
+		{
+			item->SetDisabled(TRUE);
+		}
+	}
+
+	return G_OK;
+}
+
+static gresult
+impl_apply_user_stylesheet(GaleonEmbed *embed,
+			   const gchar *sheetfile,
+			   EmbedStyleSheet **retSheet)
+{
+        GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+        g_return_val_if_fail(wrapper != NULL, G_FAILED);
+
+	nsIStyleSheet *item = nsnull;
+
+	gchar *tmp = g_strconcat("file://", sheetfile, NULL);
+	wrapper->LoadOverrideStyleSheet(tmp, &item);
+	g_free (tmp);
+
+	EmbedStyleSheet *sheet = g_new0(EmbedStyleSheet, 1);
+	sheet->name = g_strdup(sheetfile);
+	sheet->sheet = item;
+	sheet->type = STYLESHEET_NAMED;
+
+	(*retSheet) = sheet;
+	return G_OK;
+}
+
+static gresult
+impl_remove_user_stylesheet(GaleonEmbed *embed,
+			    EmbedStyleSheet *sheet)
+{
+        GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+        g_return_val_if_fail(wrapper != NULL, G_FAILED);
+
+	nsIStyleSheet *item = NS_STATIC_CAST(nsIStyleSheet*, sheet->sheet);
+	wrapper->RemoveOverrideStyleSheet(item);
+
+	g_free(sheet->name);
+	g_free(sheet);
+
+	return G_OK;
+}
+
+static gboolean
+stylesheet_is_alternate(nsIDOMStyleSheet *item)
+{
+	nsresult result;
+	gboolean ret = FALSE;
+
+	nsCOMPtr<nsIDOMNode> node;
+	result = item->GetOwnerNode(getter_AddRefs(node));
+	if (NS_FAILED(result)) return FALSE;
+
+	nsCOMPtr<nsIDOMHTMLLinkElement> link = do_QueryInterface(node);
+	
+	if (link)
+	{
+		nsAutoString str;
+		link->GetRel(str);
+		
+		gchar *tmp = g_ascii_strdown(NS_ConvertUCS2toUTF8(str).get(), -1);
+		
+		if (strstr (tmp, "alternate") != NULL) ret = TRUE;
+
+		g_free(tmp);
+	}
+
+	return ret;
+}
+
