@@ -55,7 +55,7 @@ static void		gb_galeon_icon_provider_favicon_cache_changed_cb (GaleonFaviconCach
 									  const char *url,
 									  GbGaleonIconProvider *gip);
 static void		gb_galeon_icon_provider_add_cache_url_to_bookmark (GbGaleonIconProvider *gip, 
-									   char *curl, GbBookmark *b);
+									   const char *curl, GbBookmark *b);
 static void		gb_galeon_icon_provider_weak_notify		(gpointer datap,
 									 GObject *where_the_object_was);
 static void		gb_galeon_icon_provider_bset_weak_notify	(gpointer datap,
@@ -170,10 +170,7 @@ gb_galeon_icon_provider_icon_provider_get_icon_impl (GbIconProvider *ip, GbBookm
 				gb_galeon_icon_provider_add_cache_url_to_bookmark 
 					(GB_GALEON_ICON_PROVIDER (ip), curl, rb);
 			}
-			else
-			{
-				g_free (curl);
-			}
+			g_free (curl);
 		}
 	}
 
@@ -233,7 +230,7 @@ gb_galeon_icon_provider_favicon_cache_changed_cb (GaleonFaviconCache *cache, con
 }
 
 static void
-gb_galeon_icon_provider_add_cache_url_to_bookmark (GbGaleonIconProvider *gip, char *curl, GbBookmark *b)
+gb_galeon_icon_provider_add_cache_url_to_bookmark (GbGaleonIconProvider *gip, const char *curl, GbBookmark *b)
 {
 	GbGaleonIconProviderPrivate *p = gip->priv;
 	gpointer *data;
@@ -243,7 +240,7 @@ gb_galeon_icon_provider_add_cache_url_to_bookmark (GbGaleonIconProvider *gip, ch
 		return;
 	}
 
-	g_hash_table_insert (p->cache_url_to_bookmark, curl, b);
+	g_hash_table_insert (p->cache_url_to_bookmark, g_strdup(curl), b);
 	
 	data = g_new (gpointer, 2);
 	data[0] = g_object_ref (gip);
