@@ -35,8 +35,8 @@
 #include "eel-gconf-extensions.h"
 
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-#define DEBUG_MSG(x) g_print x
-//#define DEBUG_MSG(x)
+//#define DEBUG_MSG(x) g_print x
+#define DEBUG_MSG(x)
 
 static void		gnv_embed_location_cb 			(GaleonEmbed *embed, 
 								 GaleonNautilusView *view);
@@ -382,7 +382,7 @@ gnv_embed_dom_mouse_down_cb (GaleonEmbed *embed,
 		{
 			GaleonEmbedAutoscroller *as = galeon_embed_autoscroller_new ();
 			guint x, y;
-			galeon_embed_event_get_mouse_coords (event, &x, &y);
+			galeon_embed_event_get_coords (event, &x, &y);
 			galeon_embed_autoscroller_set_embed (as, embed);
 			galeon_embed_autoscroller_start_scroll (as, GTK_WIDGET (embed), x, y);
 			g_object_unref (as);
@@ -392,7 +392,7 @@ gnv_embed_dom_mouse_down_cb (GaleonEmbed *embed,
 		{
 			GaleonEmbedManualScroller *ms = galeon_embed_manual_scroller_new ();
 			guint x, y;
-			galeon_embed_event_get_mouse_coords (event, &x, &y);
+			galeon_embed_event_get_coords (event, &x, &y);
 			galeon_embed_manual_scroller_set_embed (ms, embed);
 			galeon_embed_manual_scroller_start_scroll (ms, GTK_WIDGET (embed), x, y);
 			g_object_unref (ms);
