@@ -29,11 +29,21 @@
 #include "galeon-navigation-button.h"
 #include "galeon-shell.h"
 #include <gtk/gtkstock.h>
+#include <gtk/gtkimagemenuitem.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkimage.h>
 #include <string.h>
 
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-#define DEBUG_MSG(x) g_print x
-//#define DEBUG_MSG(x)
+//#define DEBUG_MSG(x) g_print x
+#define DEBUG_MSG(x)
+
+/* these should be kept in sync:
+ *   bookmarks/bookmarks-gtk-menu-item.h   - GB_GTK_MENU_ITEM_NAME_MAX_LENGTH
+ *   src/galeon-navigation-button.c        - MENU_ITEM_MAX_LENGTH
+ *   src/window-recent-history-menu.c      - MENU_ITEM_MAX_LENGTH
+ */
+#define MENU_ITEM_MAX_LENGTH 40
 
 /**
  * Private data
@@ -438,14 +448,21 @@ galeon_navigation_button_clicked_cb (GtkWidget *w, GaleonNavigationButton *b)
 /* TODO: clean all this, it came from toolbar.c and is messy */
 
 static GtkWidget *
-new_num_accel_menu_item (gint num, gchar *origtext, gboolean lettersok, const GdkPixbuf *ico)
+new_num_accel_menu_item (gint num, const gchar *origtext, gboolean lettersok, const GdkPixbuf *ico)
 {
-        gchar *text;
+        gchar *shortened, *escaped, *text;
         GtkWidget *item;
         GtkWidget *hb;
         GtkWidget *label;
 
-	text = gul_string_new_num_accel (num, origtext, lettersok);
+	/* shorten, escape, add accel */
+	shortened = gul_string_shorten (origtext, MENU_ITEM_MAX_LENGTH);
+	escaped = gul_string_escape_uline_accel (shortened);
+
+	text = gul_string_new_num_accel (num, escaped, lettersok);
+	g_free (shortened);
+	g_free (escaped);
+
 	if (text == NULL) return NULL;
 
 	item = gtk_image_menu_item_new ();
@@ -460,6 +477,8 @@ new_num_accel_menu_item (gint num, gchar *origtext, gboolean lettersok, const Gd
 
         gtk_widget_show_all (item);
 
+	g_free (text);
+
         return item;
 }
 
@@ -603,16 +622,15 @@ setup_back_or_forward_menu (GaleonWindow *window, GtkMenuShell *ms, GaleonNaviga
 
 	while (start != end)
 	{
-		char *title, *url, *tmp;
+		char *title, *url;
 		GtkWidget *item;
 
 		galeon_embed_shistory_get_nth (embed, start, FALSE, &url, &title);
 		g_free (url);
 
-		tmp = gul_string_escape_uline_accel (title);
+		item = new_num_accel_menu_item (accell_count, title, TRUE, NULL);
 		g_free (title);
-		item = new_num_accel_menu_item (accell_count, tmp, TRUE, NULL);
-		g_free (tmp);
+
 		if (item == NULL) break;
 
         	gtk_menu_shell_append (ms, item);
@@ -651,12 +669,9 @@ setup_up_menu (GaleonWindow *window, GtkMenuShell *ms)
 	for (li = l; li; li = li->next)
 	{
 		char *url = li->data;
-		char *tmp;
 		GtkWidget *item;
 
-		tmp = gul_string_escape_uline_accel (url);
-		item = new_num_accel_menu_item (accell_count, tmp, TRUE, NULL);
-		g_free (tmp);
+		item = new_num_accel_menu_item (accell_count, url, TRUE, NULL);
 		if (item == NULL) break;
 
         	gtk_menu_shell_append (ms, item);
