@@ -28,7 +28,10 @@
 #include "gul-string.h"
 #include "galeon-new-button.h"
 #include "galeon-shell.h"
+#include "galeon-dnd.h"
 #include <gtk/gtkstock.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkclipboard.h>
 #include <string.h>
 
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
@@ -57,6 +60,12 @@ static gchar *		galeon_new_button_to_string_impl		(GulTbItem *i);
 static GulTbItem *	galeon_new_button_clone_impl			(GulTbItem *i);
 static void		galeon_new_button_parse_properties_impl	(GulTbItem *i, const gchar *props);
 static void		galeon_new_button_clicked_cb			(GtkWidget *w, GaleonNewButton *b);
+static void		galeon_new_button_drag_data_received_cb (GtkWidget* widget, 
+								 GdkDragContext *context,
+								 gint x, gint y, 
+								 GtkSelectionData *selection_data,
+								 guint info, guint time, 
+								 GaleonNewButton *b);
 
 
 static gpointer gul_tb_item_class;
@@ -225,6 +234,11 @@ galeon_new_button_get_widget_impl (GulTbItem *i)
 		g_signal_connect (gul_tb_button_get_button (p->widget), "button-press-event",
 				  G_CALLBACK (galeon_new_button_button_press_event_cb), i);
 
+		galeon_dnd_url_drag_dest_set (GTK_WIDGET(gul_tb_button_get_button (p->widget)));
+		g_signal_connect (gul_tb_button_get_button (p->widget), "drag_data_received",
+				  G_CALLBACK(galeon_new_button_drag_data_received_cb),
+				  i);
+
 		ms = gul_tb_button_get_menu (p->widget);
 
 		gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("New _Window"), 
@@ -314,3 +328,43 @@ galeon_new_button_clicked_cb (GtkWidget *w, GaleonNewButton *b)
 			      GALEON_NEW_TAB_HOMEPAGE
 			      | GALEON_NEW_TAB_JUMP);
 }
+
+
+static void
+each_url_receive_data_binder (const char * url, const char * title, 
+			      gpointer data )
+{
+	char ** urlp = data;
+	if (*urlp == NULL)
+	{
+		*urlp = g_strdup (url);
+	}
+}	
+
+static void
+galeon_new_button_drag_data_received_cb (GtkWidget* widget, 
+					 GdkDragContext *dc,
+					 gint x, gint y, 
+					 GtkSelectionData *selection_data,
+					 guint info, guint time, 
+					 GaleonNewButton *b)
+{
+	char * url = NULL;
+	GaleonWindow *window = galeon_tbi_get_window (GALEON_TBI (b));
+
+	g_return_if_fail (IS_GALEON_WINDOW (window));
+
+	galeon_dnd_drag_data_receive (widget, dc, x, y, selection_data,
+			 info, time, &url, each_url_receive_data_binder);
+	
+	if (url)
+	{
+		GaleonTab *tab;
+
+		tab = galeon_window_get_active_tab (window);
+	
+		galeon_shell_new_tab (galeon_shell, window, tab, url,
+				      GALEON_NEW_TAB_JUMP);
+		g_free (url);
+	}
+}
