@@ -664,29 +664,40 @@ fonts_load_font (GaleonPrefsDialog *self, const char *type, GtkWidget *combo)
 	GaleonPrefsDialogPrivate *p = self->priv;
 	char     gconf_key[256];
 	char    *font;
-	GList   *fonts = NULL;
+	GList   *fonts = NULL, *rest = NULL, *l;
 	gresult  rv;
 
 	g_return_if_fail (GTK_IS_COMBO(combo));
 
+	/* list all available fonts, show first those that are "good" for the
+	 * given encoding, after those show the remaining fonts */
 	rv = galeon_embed_shell_get_font_list (embed_shell, p->fonts_encoding,
 			                       type, &fonts, NULL);
-	if (G_OK == rv && fonts == NULL)
+	g_return_if_fail (G_OK == rv);
+
+	rv = galeon_embed_shell_get_font_list (embed_shell, "", type,
+			                       &rest, NULL);
+	g_return_if_fail (G_OK == rv);
+
+	for (l = rest; l != NULL; /* nothing */)
 	{
-		/* no fonts available for the given encoding, try again but
-		 * this time accept fonts for any encoding so that you can
-		 * still pick a font even if it's not "good" for the encoding
-		 */
-		rv = galeon_embed_shell_get_font_list (embed_shell, "",
-				                       type, &fonts, NULL);
+		GList *next = l->next;
+
+		if (g_list_find_custom (fonts, l->data, (GCompareFunc)strcmp))
+		{
+			g_free (l->data);
+			rest = g_list_delete_link (rest, l);
+		}
+		l = next;
 	}
 
-	g_return_if_fail (G_OK == rv);
+	fonts = g_list_concat (fonts, rest);
 
 	fonts_make_gconf_key (gconf_key, sizeof(gconf_key),
 			      type, p->fonts_encoding);
 	font = eel_gconf_get_string (gconf_key);
-	if (font == NULL || !g_list_find_custom (fonts, font, g_str_equal))
+	if (font == NULL || !g_list_find_custom (fonts, font,
+				                 (GCompareFunc)strcmp))
 	{
 		g_free (font);
 		font = g_strdup (fonts != NULL ? fonts->data : "");
@@ -1021,6 +1032,8 @@ galeon_prefs_mouse_construct (GaleonPrefsDialog *self, GladeXML *gxml)
 		NULL);
 
 	widget = glade_xml_get_widget (gxml, "mouse_wheel_override");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget),
+		!eel_gconf_get_boolean (CONF_MOUSE_WHEEL_NOKEY_DEF_STEP));
 	g_signal_connect (G_OBJECT(widget), "toggled",
 			  G_CALLBACK(on_toggle_button_toggled_invert),
 			  CONF_MOUSE_WHEEL_NOKEY_DEF_STEP);
