@@ -47,6 +47,9 @@
 
 #endif
 
+/* #define DEBUG_MSG(x) g_print x */
+#define DEBUG_MSG(x)
+
 struct GaleonShellPrivate
 {
 	GbBookmarkSet *bookmark_set;
@@ -54,6 +57,7 @@ struct GaleonShellPrivate
 	Session *session;
 	GaleonFaviconCache *favicon_cache;
 	GaleonAutocompletion *autocompletion;
+	gboolean server_mode;
 };
 
 enum
@@ -169,8 +173,9 @@ galeon_shell_init (GaleonShell *gs)
 	gul_pixbuf_cache_register_stocks ();
 	
         gs->priv = g_new0 (GaleonShellPrivate, 1);
-	gs->priv->session = NULL;
-	
+	gs->priv->session     = NULL;
+	gs->priv->server_mode = FALSE;
+
 #ifdef ENABLE_GTKHTML_EMBED
 	gs->priv->embed_shell = galeon_embed_shell_new ("gtkhtml");	
 #elif ENABLE_MOZILLA_EMBED
@@ -307,8 +312,11 @@ migrate_proxy_prefs ()
 		for (i=0; keys[i][0] != NULL; i++)
 		{		  
 			value = eel_gconf_get_value (keys[i][0]);
-			eel_gconf_set_value (keys[i][1], value);
-			eel_gconf_value_free (value);
+			if (value)
+			{
+				eel_gconf_set_value (keys[i][1], value);
+				eel_gconf_value_free (value);
+			}
 		}
 
 		proxy_mode = eel_gconf_get_integer (OLD_NETWORK_PROXY_MODE);
@@ -350,7 +358,6 @@ load_homepage (GaleonShell *gs,
 	       GaleonEmbed *previous_embed)
 {
         const gchar *last_page_url;
-        gchar *home_page_url;
         gint page_type;
 	char *result = NULL;
 	
@@ -361,8 +368,11 @@ load_homepage (GaleonShell *gs,
         if (page_type == STARTPAGE_HOME)
         {
                 /* get location of home page */
-                home_page_url = eel_gconf_get_string(CONF_GENERAL_HOMEPAGE);
-		result = home_page_url;
+                result = eel_gconf_get_string(CONF_GENERAL_HOMEPAGE);
+		if (result != NULL)
+		{
+			result = g_strstrip (result);
+		}
         }
 	else if (page_type == STARTPAGE_LAST)
 	{
@@ -382,6 +392,13 @@ load_homepage (GaleonShell *gs,
 		result = g_strdup (last_page_url);
 	}
 
+	/* empty URL equals to about:blank */
+	if (result != NULL && *result == '\0')
+	{
+		g_free (result);
+		result = NULL;
+	}
+
 	if (result == NULL)
 	{
 		/* even in case of error, it's a good default */
@@ -436,13 +453,43 @@ galeon_shell_new_tab (GaleonShell *shell,
 		      GaleonTab *previous_tab,
 		      const char *url,
 		      GaleonNewTabFlags flags)
+{
+	GaleonEmbed * previous_embed = NULL;
+
+	if (previous_tab)
+	{
+		previous_embed = galeon_tab_get_embed (previous_tab);
+	}
+	
+	return galeon_shell_new_tab_from_embed
+		(shell, parent_window, previous_embed, url, flags);
+}
+
+
+/**
+ * galeon_shell_new_tab_from_embed:
+ * @shell: a #GaleonShell
+ * @parent_window: the target #GaleonWindow or %NULL
+ * @previous_embed: the referrer embed or %NULL
+ * @url: an url to load or %NULL
+ *
+ * Create a new tab and the parent window when necessary.
+ * Ever use this function to open urls in new window/tabs.
+ *
+ * ReturnValue: the created #GaleonTab
+ **/
+GaleonTab *
+galeon_shell_new_tab_from_embed (GaleonShell *shell,
+				 GaleonWindow *parent_window,
+				 GaleonEmbed *previous_embed,
+				 const char *url,
+				 GaleonNewTabFlags flags)
 {
 	GaleonWindow *window;
 	GaleonTab *tab;
 	GaleonEmbed *embed;
 	gboolean in_new_window;
 	gboolean jump_to;
-	GaleonEmbed *previous_embed = NULL;
 	
 	in_new_window = !eel_gconf_get_boolean (CONF_TABS_TABBED);
 
@@ -463,11 +510,6 @@ galeon_shell_new_tab (GaleonShell *shell,
         	window = galeon_window_new ();
 	}
 
-	if (previous_tab)
-	{
-		previous_embed = galeon_tab_get_embed (previous_tab);
-	}
-	
 	tab = galeon_tab_new ();
 	embed = galeon_tab_get_embed (tab);
 	gtk_widget_show (GTK_WIDGET(embed));
@@ -498,6 +540,16 @@ galeon_shell_new_tab (GaleonShell *shell,
 		galeon_embed_shistory_copy (previous_embed, embed, TRUE, FALSE, FALSE);
 	}
 
+	if (flags & GALEON_NEW_TAB_RAISE_WINDOW)
+	{
+		gtk_window_present (GTK_WINDOW (window));
+	}
+
+	if (flags & GALEON_NEW_TAB_FULLSCREEN)
+	{
+		gtk_window_fullscreen (GTK_WINDOW (window));
+	}
+
         return tab;
 }
 
@@ -539,15 +591,15 @@ bookmarks_init (GaleonShell *gs)
 	gb_galeon_icon_provider_set_default_provider (gip, def);
 	gb_system_set_icon_provider (GB_ICON_PROVIDER (gip));
 
-	g_print ("trying to load bookmarks from %s\n", filename);
+	DEBUG_MSG (("trying to load bookmarks from %s\n", filename));
 	gs->priv->bookmark_set = gb_io_load_from_file (GB_IO (io), filename);
 
 	if (gs->priv->bookmark_set == NULL)
 	{
 		gchar *filename2 = gul_general_user_file ("default-bookmarks.xbel", FALSE);
 		g_warning ("I could not load the bookmarks file, "
-			   "will load the default bookmarks.");
-		if (filename)
+			   "will load the default bookmarks from %s.", filename2);
+		if (filename2)
 		{
 			gs->priv->bookmark_set = gb_io_load_from_file (GB_IO (io), filename2);
 		}
@@ -559,17 +611,18 @@ bookmarks_init (GaleonShell *gs)
 			gs->priv->bookmark_set = gb_bookmark_set_new ();
 			root = gb_folder_new (gs->priv->bookmark_set, _("Bookmarks"));
 			gb_bookmark_set_set_root (gs->priv->bookmark_set, root);
+			gb_bookmark_set_set_io (gs->priv->bookmark_set, GB_IO (io));
 		}		
 		g_free (filename2);
 		gb_bookmark_set_set_filename (gs->priv->bookmark_set, filename);
 	}
 
-	g_print ("Detected version of bookmarks file: %s\n", 
+	DEBUG_MSG (("Detected version of bookmarks file: %s\n", 
 		 gs->priv->bookmark_set->file_format_version == GB_FILE_FORMAT_VERSION_GALEON_1 
 		 ? "galeon1" 
 		 : (gs->priv->bookmark_set->file_format_version == GB_FILE_FORMAT_VERSION_GALEON_2 
 		    ? "galeon2" 
-		    : "unknown (galeon1)"));
+		    : "unknown (galeon1)")));
 
 	if (gs->priv->bookmark_set->file_format_version == GB_FILE_FORMAT_VERSION_UNKNOWN)
 	{
@@ -637,6 +690,46 @@ galeon_shell_get_session (GaleonShell *gs)
 	return gs->priv->session;
 }
 
+
+/**
+ *  galeon_shell_get_server_mode
+ * 
+ *  @gs: a #GaleonShell
+ * 
+ *  Returns whether galeon is running in server mode
+ *
+ *  Return value: TRUE if we are in server mode, FALSE otherwise
+ */
+gboolean
+galeon_shell_get_server_mode (GaleonShell *gs)
+{
+	return gs->priv->server_mode;
+}
+
+/**
+ *  galeon_shell_set_server_mode
+ * 
+ *  @gs: a #GaleonShell
+ *  mode: Whether to run in server mode.
+ * 
+ *  Sets galeon server mode to be true or false
+ *
+ */
+void
+galeon_shell_set_server_mode (GaleonShell *gs, gboolean mode)
+{
+	if (mode && !gs->priv->server_mode)
+	{
+		g_object_ref (G_OBJECT(gs));
+	}
+	else if (!mode && gs->priv->server_mode)
+	{
+		g_object_unref (G_OBJECT(gs));
+	}
+
+	gs->priv->server_mode = mode;
+}
+
 /**
  * galeon_shell_get_favicon_cache:
  * @gs: a #GaleonShell
