@@ -235,6 +235,7 @@ select_spinner_image (GaleonSpinner *spinner)
 	}
 
 	element = g_list_nth (spinner->details->image_list, spinner->details->current_frame);
+	g_return_val_if_fail (element, NULL);
 	
 	return g_object_ref (element->data);
 }
@@ -366,6 +367,7 @@ galeon_spinner_expose (GtkWidget *widget, GdkEventExpose *event)
 {
 	GaleonSpinner *spinner;
 	GdkPixbuf *pixbuf, *massaged_pixbuf;
+	GdkGC *gc;
 	int x_offset, y_offset, width, height;
 	GdkRectangle pix_area, dest;
 
@@ -408,14 +410,13 @@ galeon_spinner_expose (GtkWidget *widget, GdkEventExpose *event)
 		return FALSE;
 	}
 	
-	gdk_pixbuf_render_to_drawable_alpha (
-		pixbuf, widget->window, 
-		dest.x - x_offset, dest.y - y_offset,
-		dest.x, dest.y,
-		dest.width, dest.height,
-		GDK_PIXBUF_ALPHA_BILEVEL, 128,
-		GDK_RGB_DITHER_MAX,
-		0, 0);
+	gc = gdk_gc_new (widget->window);
+	gdk_draw_pixbuf (widget->window, gc, pixbuf,
+			 dest.x - x_offset, dest.y - y_offset,
+			 dest.x, dest.y,
+			 dest.width, dest.height,
+			 GDK_RGB_DITHER_MAX, 0, 0);
+	g_object_unref (gc);
 
 	g_object_unref (pixbuf);
 
@@ -576,6 +577,13 @@ galeon_spinner_load_images (GaleonSpinner *spinner)
 	image_theme = eel_gconf_get_string (CONF_TOOLBAR_SPINNER_THEME);
 	
 	path = galeon_spinner_get_theme_path (image_theme);
+	if (path == NULL)
+	{
+		g_warning ("spinner theme `%s' not found, using default `%s'",
+			   image_theme, "galeon");
+		path = galeon_spinner_get_theme_path ("galeon");
+	}
+	g_free (image_theme);
 	g_return_if_fail (path != NULL);
 
 	qpixbuf = load_themed_image (path, "rest.png", 
@@ -599,8 +607,9 @@ galeon_spinner_load_images (GaleonSpinner *spinner)
 		image_list = g_list_prepend (image_list, pixbuf);
 	}
 	spinner->details->image_list = g_list_reverse (image_list);
+	spinner->details->current_frame = 0;
 
-	g_free (image_theme);
+	g_free (path);
 }
 
 static gboolean
