@@ -37,6 +37,7 @@
 #include "galeon-embed-manual-scroller.h"
 #include "gul-download.h"
 #include "gul-gestures.h"
+#include "galeon-embed-favicon.h"
 
 #include <bonobo/bonobo-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -53,10 +54,12 @@
 
 struct GaleonTabPrivate
 {
+	GList *notifiers;
 	GaleonEmbed *embed;
 	GaleonWindow *window;
 	gboolean is_active;
 	gboolean closing;
+	GtkWidget *tab_icon;
 	TabLoadStatus load_status;
 	char *link_message;
 	char *js_message;
@@ -136,6 +139,8 @@ galeon_tab_security_change_cb (GaleonEmbed *embed, EmbedSecurityLevel level,
 static void
 galeon_tab_zoom_changed_cb (GaleonEmbed *embed, gint zoom, 
 			    GaleonTab *tab);
+static void
+galeon_tab_destroy_brsr_cb (GaleonEmbed *embed, GaleonTab *tab);
 
 static void
 galeon_tab_popup_blocked_cb(GaleonEmbed *embed, const char *uri,
@@ -240,6 +245,51 @@ out:
 	g_free (url2);
 }
 
+
+static void
+tab_icon_set_visibility (GaleonFavicon *favicon)
+{
+	int mode = eel_gconf_get_integer (CONF_TABS_FAVICON);
+
+	switch (mode)
+	{
+	case 0:
+		gtk_widget_hide (GTK_WIDGET(favicon));
+		break;
+
+	case 1:
+		gtk_widget_show (GTK_WIDGET(favicon));
+		break;
+
+	default:
+		if (galeon_favicon_get_is_default (favicon))
+		{
+			gtk_widget_hide (GTK_WIDGET(favicon));
+		}
+		else
+		{
+			gtk_widget_show (GTK_WIDGET(favicon));
+		}
+		break;
+	}
+}
+
+
+static void
+tab_icon_changed_cb (GaleonFavicon *favicon, gpointer dummy)
+{
+	tab_icon_set_visibility (favicon);
+}
+
+
+static void
+tab_favicon_mode_gconf_changed_cb (GConfClient *client, guint cnxn_id,
+				   GConfEntry *entry, GaleonTab *tab)
+{
+	tab_icon_set_visibility (GALEON_FAVICON (tab->priv->tab_icon));
+}
+
+
 static void
 galeon_tab_init (GaleonTab *tab)
 {
@@ -252,6 +302,20 @@ galeon_tab_init (GaleonTab *tab)
 	
 	tab->priv->embed = galeon_embed_new (G_OBJECT(shell));
 
+	tab->priv->tab_icon = galeon_embed_favicon_new (tab->priv->embed);
+	g_object_ref (G_OBJECT (tab->priv->tab_icon));
+	gtk_object_sink (GTK_OBJECT (tab->priv->tab_icon));
+
+	tab_icon_set_visibility (GALEON_FAVICON (tab->priv->tab_icon));
+
+	galeon_notification_add(CONF_TABS_FAVICON,
+				(GConfClientNotifyFunc)tab_favicon_mode_gconf_changed_cb,
+				tab, &tab->priv->notifiers);
+
+	g_signal_connect (tab->priv->tab_icon, "changed",
+			  G_CALLBACK (tab_icon_changed_cb), 
+			  tab);
+
 	tab->priv->window = NULL;
 	tab->priv->is_active = FALSE;
 	tab->priv->link_message = g_strdup ("");
@@ -306,8 +370,8 @@ galeon_tab_init (GaleonTab *tab)
 	g_signal_connect (embed, "ge_visibility",
 			  GTK_SIGNAL_FUNC (galeon_tab_visibility_cb), 
 			  tab);
-	g_signal_connect_swapped (embed, "ge_destroy_brsr",
-			  GTK_SIGNAL_FUNC (galeon_tab_destroy), 
+	g_signal_connect (embed, "ge_destroy_brsr",
+			  GTK_SIGNAL_FUNC (galeon_tab_destroy_brsr_cb), 
 			  tab);
 	g_signal_connect (embed, "ge_open_uri",
 			  GTK_SIGNAL_FUNC (galeon_tab_open_uri_cb), 
@@ -348,10 +412,28 @@ galeon_tab_finalize (GObject *object)
 
         g_return_if_fail (tab->priv != NULL);
 
+	galeon_notification_remove (&tab->priv->notifiers);
+
+	/* Ensure that we disconnect our embed signals, we are
+	 * being deleted, but the embed may still emit
+	 * events, such as a popup blocked event if there was
+	 * some 'onunload' javascript. I think net_state signals
+	 * may also be emitted, and various other ones
+	 */
+	g_signal_handlers_disconnect_matched (tab->priv->embed,
+					      G_SIGNAL_MATCH_DATA,
+					      0, 0, NULL, NULL,
+					      tab);
+
 	g_idle_remove_by_data (tab->priv->embed);
 	
+	g_idle_remove_by_data (tab);
+
+	g_object_unref (G_OBJECT(tab->priv->tab_icon));
+
 	g_free (tab->priv->link_message);
 	g_free (tab->priv->js_message);
+	g_free (tab->priv->blocked_popup_uri);
 
         g_free (tab->priv->title);
         g_free (tab->priv->location);
@@ -468,6 +550,12 @@ galeon_tab_get_size (GaleonTab *tab, int *width, int *height)
 	*height = tab->priv->height;
 }
 
+GtkWidget *
+galeon_tab_get_icon (GaleonTab *tab)
+{
+	return tab->priv->tab_icon;
+}
+
 static void 
 galeon_tab_set_visibility (GaleonTab *tab,
                            gboolean visible)
@@ -557,7 +645,14 @@ galeon_tab_title_cb (GaleonEmbed *embed, GaleonTab *tab)
 	if (*(tab->priv->title) == '\0')
 	{
 		g_free (tab->priv->title);
-		tab->priv->title = g_strdup (_("Untitled"));
+		if (!strcmp (tab->priv->location, "about:blank"))
+		{
+			tab->priv->title = g_strdup(_("Untitled"));
+		}
+		else
+		{
+			tab->priv->title = g_strdup (tab->priv->location);
+		}
 	}
 	
 	galeon_window_site_visited (tab->priv->window, tab->priv->location, tab->priv->title);
@@ -861,6 +956,41 @@ galeon_tab_visibility_cb (GaleonEmbed *embed, gboolean visibility,
 #endif
 }
 
+
+static gboolean
+galeon_tab_destroy_brsr_idle_cb (GaleonTab *tab)
+{
+	DEBUG_MSG (("galeon_tab_destroy_brsr_idle_cb\n"));
+
+	galeon_tab_destroy (tab);
+	return FALSE;
+}
+
+
+static void
+galeon_tab_destroy_brsr_cb (GaleonEmbed *embed, GaleonTab *tab)
+{
+	/* This is a dirty to hack to work around mozillas problems
+	 * with closing windows via javascript that dont have focus.
+	 * 
+	 * The problem is that if you have a tab loaded in the background
+	 * in the notebook, then mozilla won't emit the ge_destroy_brsr 
+	 * signal until it gets focus by someone clicking on the tab label.
+	 * At that point, the signal is emitted, we close the tab, remove
+	 * the label, and then the notebook crashes because widget for the
+	 * label has been destroyed.
+	 *
+	 * So lets work around the problem by adding an g_idle function
+	 * that destroys the tab when we know nothing is referencing it
+	 * in the stack.
+	 */
+	DEBUG_MSG (("galeon_tab_destroy_brsr_cb\n"));
+
+	g_idle_add_full (G_PRIORITY_HIGH_IDLE, 
+			 (GSourceFunc)galeon_tab_destroy_brsr_idle_cb, tab, 0);
+}
+
+
 void
 galeon_tab_destroy (GaleonTab *tab)
 {
