@@ -286,6 +286,9 @@ galeon_window_fullscreen (GaleonWindow *window);
 static void
 galeon_window_unfullscreen (GaleonWindow *window);
 
+static gboolean 
+galeon_window_key_press_event(GtkWidget *widget,
+			      GdkEventKey *event);
 
 /* static class variables */
 static GdkColor _galeon_window_loading_tab_color;
@@ -310,6 +313,7 @@ galeon_window_class_init (GaleonWindowClass *klass)
         object_class->finalize = galeon_window_finalize;
 	
 	widget_class->show = galeon_window_show;
+	widget_class->key_press_event = galeon_window_key_press_event;
 
 	/* initialize colors */
 	tmp = eel_gconf_get_string (CONF_TABS_TABBED_LOADING_COLOR);
@@ -335,31 +339,46 @@ galeon_window_gb_location_source_init (GbLocationSourceIface *iface)
 
 static
 gboolean 
-galeon_window_key_press_event_cb (GtkWidget *widget,
-				  GdkEventKey *event,
-				  GaleonWindow *window)
+galeon_window_key_press_event(GtkWidget *widget,
+			      GdkEventKey *event)
 {
-        int page;
+        if ((event->state & GDK_CONTROL_MASK) || (event->state & GDK_MOD1_MASK))
+	{
+		GtkWidget *entry = gtk_window_get_focus(GTK_WINDOW(widget));
+		if (GTK_IS_EDITABLE(entry) && gtk_widget_event(entry, (GdkEvent*)event))
+			return TRUE;
+	}
 
-	if (event->state & GDK_SHIFT_MASK)
-                return FALSE;
+	if (event->state & GDK_MOD1_MASK)
+	{
+		int page = event->keyval - GDK_0 -1;
 
-        if (event->state & GDK_MOD1_MASK)
-        {
-                page = event->keyval - GDK_0 -1;
+		if (page == -1) page = 9;
 
-                if (page == -1) page = 9;
+		if (page>=-1 && page<=9)
+		{
+			gtk_notebook_set_current_page 
+				(GTK_NOTEBOOK(GALEON_WINDOW(widget)->priv->notebook),
+				page == -1 ? -1 : page);
+			return TRUE;
+		}
+	}
 
-                if (page>=-1 && page<=9)
-                {
-                        gtk_notebook_set_current_page 
-				(GTK_NOTEBOOK (window->priv->notebook),
-                                 page == -1 ? -1 : page);
-                        return TRUE;
-                }
-        }
-	
-        return FALSE;
+#if YOU_KNOW_WHAT_THE_HELL_MOZILLA_DOES_WITH_KEY_EVENTS
+	if (event->keyval == GDK_Escape)
+	{
+		GtkWidget *focus = gtk_window_get_focus(GTK_WINDOW(widget));
+		g_print("Got an escape\n");
+		g_print("My name is: %s\n", G_OBJECT_TYPE_NAME(focus));
+		if (focus)
+		{
+			g_print("passing escape on\n");
+			gtk_widget_event(focus, (GdkEvent*)event);
+		}
+	}
+#endif
+
+	return GTK_WIDGET_CLASS(parent_class)->key_press_event(widget, event);
 }
 
 static void
@@ -440,12 +459,6 @@ setup_bonobo_window (GaleonWindow *window,
 			       "galeon-ui.xml",
 			       "galeon", NULL);
 
-	/* Key handling */
-	g_signal_connect(G_OBJECT(win), 
-			 "key-press-event",
-                         G_CALLBACK(galeon_window_key_press_event_cb),
-                         window);
-
 	g_signal_connect (G_OBJECT(win), "window_state_event",
 			  G_CALLBACK (galeon_window_state_event_cb), NULL);
 	g_signal_connect (G_OBJECT(win), "delete-event",
@@ -1562,6 +1575,7 @@ galeon_window_load_url (GaleonWindow *window,
 	real_url = gb_bookmark_set_get_url_by_nick_and_args (set, url);
 
         galeon_embed_load_url (embed, real_url != NULL ? real_url : url);
+	galeon_embed_grab_focus (embed);
 
 	g_free (real_url);
 }
