@@ -20,15 +20,19 @@
 #endif
 
 #include "history-dialog.h"
-#include "galeon-shell.h"
+
 #include "galeon-embed-shell.h"
-#include "gul-string.h"
-#include "gul-gui.h"
 #include "galeon-dnd.h"
+#include "galeon-shell.h"
+#include "galeon-window.h"
+#include "gul-gui.h"
+#include "gul-string.h"
+#include "gul-state.h"
 #include "location-entry.h"
 
 #include <gtk/gtktreeview.h>
 #include <gtk/gtktreestore.h>
+#include <gtk/gtktreeselection.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <bonobo/bonobo-i18n.h>
 
@@ -42,6 +46,11 @@ static void history_dialog_finalize (GObject *object);
 static void history_dialog_set_embedded (HistoryDialog *d,
 			     	         gboolean embedded);
 
+//#define DEBUG_TIME
+
+#ifdef DEBUG_TIME
+#include <glib/gtimer.h>
+#endif
 
 /* Glade callbacks */
 void
@@ -71,8 +80,11 @@ struct HistoryDialogPrivate
 	HistoryFilter filter;
 	gboolean group;
 	gboolean embedded;
+	guint filter_entry_timeout;
 };
 
+#define FILTER_ENTRY_TIMEOUT 700
+
 enum
 {
 	PROP_0,
@@ -81,6 +93,7 @@ enum
 
 enum
 {
+	PROP_DIALOG,
 	PROP_TREEVIEW,
 	PROP_WORD,
 	PROP_TIME,
@@ -97,6 +110,7 @@ static GtkTargetEntry url_drag_types [] =
 static const
 GaleonDialogProperty properties [] =
 {
+	{ PROP_DIALOG, "history_dialog", NULL, PT_NORMAL, NULL },
 	{ PROP_TREEVIEW, "history_treeview", NULL, PT_NORMAL, NULL },
 	{ PROP_WORD, "history_entry", CONF_HISTORY_SEARCH_TEXT, PT_NORMAL, NULL },
 	{ PROP_TIME, "history_time_optionmenu", CONF_HISTORY_SEARCH_TIME, PT_NORMAL, NULL },
@@ -218,7 +232,7 @@ each_url_get_data_binder (GaleonDragEachSelectedItemDataGet iteratee,
 	for (l = refs; l != NULL; l = l->next)
 	{
 		GValue val = {0, };
-        	const char *url;
+        	const char *url, *title;
 		GtkTreeIter iter;
                 GtkTreePath *path;
 
@@ -229,7 +243,10 @@ each_url_get_data_binder (GaleonDragEachSelectedItemDataGet iteratee,
 
 		gtk_tree_model_get_value (model, &iter, 1, &val);
 	        url = g_value_get_string (&val);
-		iteratee (url, -1, -1, -1, -1, data);
+
+		gtk_tree_model_get_value (model, &iter, 0, &val);
+	        title = g_value_get_string (&val);
+		iteratee (url, title, -1, -1, -1, -1, data);
 	}
 
 	gul_gui_treeview_free_selection_refs (refs);
@@ -298,7 +315,7 @@ history_dialog_setup_view (HistoryDialog *dialog)
         column = gtk_tree_view_get_column (treeview, 0);
         gtk_tree_view_column_set_resizable (column, TRUE);
         gtk_tree_view_column_set_reorderable (column, TRUE);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_min_width (column, 20);
 	gtk_tree_view_column_set_max_width (column, 400);
         gtk_tree_view_column_set_sort_column_id (column, 0);
 
@@ -310,7 +327,7 @@ history_dialog_setup_view (HistoryDialog *dialog)
         column = gtk_tree_view_get_column (treeview, 1);
         gtk_tree_view_column_set_resizable (column, TRUE);
         gtk_tree_view_column_set_reorderable (column, TRUE);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_min_width (column, 20);
 	gtk_tree_view_column_set_max_width (column, 500);
         gtk_tree_view_column_set_sort_column_id (column, 1);
 
@@ -509,20 +526,19 @@ history_dialog_find_host_iter (HistoryDialog *dialog,
 {
 	GtkTreeIter iter;
 	
-	gtk_tree_model_get_iter (dialog->priv->model,
-				 &iter,
-				 gtk_tree_path_new_first());
-	
-	do
+	if (gtk_tree_model_get_iter (dialog->priv->model, &iter,
+				     gtk_tree_path_new_first()) == TRUE)
 	{
-		GValue val = { 0, }; 
-		HistoryItem *i;
-		gtk_tree_model_get_value (dialog->priv->model, 
-					  &iter, 5, &val);
-		i = (HistoryItem *) g_value_get_pointer (&val);
-		if (i == item) return gtk_tree_iter_copy (&iter); 
+		do {
+			GValue val = { 0, }; 
+			HistoryItem *i;
+			gtk_tree_model_get_value (dialog->priv->model, 
+						  &iter, 5, &val);
+			i = (HistoryItem *) g_value_get_pointer (&val);
+			if (i == item) return gtk_tree_iter_copy (&iter); 
+		}
+		while (gtk_tree_model_iter_next (dialog->priv->model, &iter));
 	}
-	while (gtk_tree_model_iter_next (dialog->priv->model, &iter));
 
 	return NULL;
 }
@@ -690,6 +706,14 @@ history_dialog_set_embedded (HistoryDialog *dialog,
 				 embedded ? 
 				 "history_dock_box" :
 				 "history_dialog");
+
+	if (!embedded)
+	{
+		gul_state_monitor_window (
+			galeon_dialog_get_control (GALEON_DIALOG(dialog),
+				                   PROP_DIALOG),
+			"History", -1, -1);
+	}
 	
 	dialog->priv->treeview = GTK_TREE_VIEW (
 				 galeon_dialog_get_control (GALEON_DIALOG(dialog), 
@@ -703,28 +727,35 @@ static void
 history_dialog_finalize (GObject *object)
 {
 	HistoryDialog *dialog;
+	HistoryDialogPrivate *p;
 
         g_return_if_fail (object != NULL);
         g_return_if_fail (IS_HISTORY_DIALOG (object));
 
 	dialog = HISTORY_DIALOG (object);
+	p = dialog->priv;
 
         g_return_if_fail (dialog->priv != NULL);
 
+	if (p->filter_entry_timeout)
+	{
+		g_source_remove (p->filter_entry_timeout);
+	}
+
 	g_signal_handlers_disconnect_by_func 
-		(G_OBJECT(dialog->priv->gh),
+		(G_OBJECT(p->gh),
 		 G_CALLBACK (history_item_add_cb),
 		 dialog);
 	g_signal_handlers_disconnect_by_func 
-		(G_OBJECT(dialog->priv->gh),
+		(G_OBJECT(p->gh),
 		 G_CALLBACK (history_item_update_cb),
 		 dialog);
 	g_signal_handlers_disconnect_by_func 
-		(G_OBJECT(dialog->priv->gh),
+		(G_OBJECT(p->gh),
 		 G_CALLBACK (history_item_remove_cb),
 		 dialog);
 	
-        g_free (dialog->priv);
+        g_free (p);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -769,14 +800,55 @@ history_host_checkbutton_toggled_cb (GtkWidget *widget,
 	history_dialog_fill_view (dialog);
 }
 
+static gboolean
+history_filter_entry_timeout_cb (gpointer data)
+{
+	HistoryDialog *dialog = data;
+	HistoryDialogPrivate *p = dialog->priv;
+#ifdef DEBUG_TIME
+	GTimer *timer = g_timer_new ();
+	g_timer_start (timer);
+#endif
+
+	p->filter_entry_timeout = 0;
+
+	history_dialog_clear_view (dialog);
+#ifdef DEBUG_TIME
+	g_print ("HD: %f elapsed clearing liststore\n", g_timer_elapsed (timer, NULL));
+	g_timer_start (timer);
+#endif	
+	history_dialog_setup_filter (dialog);
+#ifdef DEBUG_TIME
+	g_print ("HD: %f elapsed setting filter\n", g_timer_elapsed (timer, NULL));
+	g_timer_start (timer);
+#endif	
+	history_dialog_fill_view (dialog);
+#ifdef DEBUG_TIME
+	g_print ("HD: %f elapsed filling view\n", g_timer_elapsed (timer, NULL));
+#endif	
+
+#ifdef DEBUG_TIME
+	g_timer_destroy (timer);
+#endif
+	return FALSE;
+}
+
+
 void
 history_entry_changed_cb (GtkWidget *widget, 
 			  HistoryDialog *dialog)
 {
-	if (dialog->priv->treeview == NULL) return;
-	history_dialog_clear_view (dialog);
-	history_dialog_setup_filter (dialog);
-	history_dialog_fill_view (dialog);
+	HistoryDialogPrivate *p = dialog->priv;
+
+	if (p->treeview == NULL) return;
+
+	if (p->filter_entry_timeout)
+	{
+		g_source_remove (p->filter_entry_timeout);
+	}
+	
+	p->filter_entry_timeout = g_timeout_add (FILTER_ENTRY_TIMEOUT, 
+						 history_filter_entry_timeout_cb, dialog);
 }
 
 void
@@ -800,8 +872,20 @@ void
 history_clear_button_clicked_cb (GtkWidget *button,
 			         HistoryDialog *dialog)
 {
+	Session * session;
+	const GList *windows;
+	const GList *win;
+
 	history_dialog_clear_view (dialog);
 	global_history_clear (dialog->priv->gh);
 	galeon_location_entry_clear_history ();
+
+	session = galeon_shell_get_session (galeon_shell);
+	windows = session_get_windows (session);
+
+	for (win = windows; win ; win = win->next)
+	{
+		galeon_window_clear_history (win->data);
+	} 
 }
 
