@@ -22,12 +22,17 @@
 #include "pdm-dialog.h"
 #include "galeon-shell.h"
 #include "galeon-embed-shell.h"
-#include "gul-gui.h"
 #include "gul-ellipsizing-label.h"
+#include "gul-gui.h"
+#include "gul-state.h"
 
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkliststore.h>
 #include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtksizegroup.h>
+#include <gtk/gtkdialog.h>
+#include <gtk/gtkstock.h>
 #include <bonobo/bonobo-i18n.h>
 
 typedef struct PdmActionInfo PdmActionInfo;
@@ -70,7 +75,7 @@ struct PdmActionInfo
 
 struct PdmDialogPrivate
 {
-	GtkTreeModel *model;
+	GtkWidget     *notebook;
 	PdmActionInfo *cookies;
 	PdmActionInfo *passwords;
 	PdmActionInfo *password_sites;
@@ -82,6 +87,7 @@ struct PdmDialogPrivate
 enum
 {
 	PROP_DIALOG,
+	PROP_NOTEBOOK,
 
 	PROP_COOKIES_TREEVIEW,
 	PROP_COOKIES_REMOVE,
@@ -149,6 +155,7 @@ static const
 GaleonDialogProperty properties [] =
 {
 	{ PROP_DIALOG, "pdm_dialog", NULL, PT_NORMAL, NULL },
+	{ PROP_NOTEBOOK, "pdm_notebook", NULL, PT_NORMAL, NULL },
 
 	{ PROP_COOKIES_TREEVIEW, "cookies_treeview", NULL, PT_NORMAL, NULL },
 	{ PROP_COOKIES_REMOVE, "cookies_remove_button", NULL, PT_NORMAL, NULL },
@@ -920,6 +927,14 @@ pdm_dialog_init (PdmDialog *dialog)
                                  "galeon.glade",
                                  "pdm_dialog");
 
+	gul_state_monitor_window 
+		(galeon_dialog_get_control (GALEON_DIALOG(dialog), PROP_DIALOG),
+		 "PDM", -1, -1);
+
+	dialog->priv->notebook = 
+		galeon_dialog_get_control (GALEON_DIALOG(dialog),
+				           PROP_NOTEBOOK);
+
 	group_button_allocations (GALEON_DIALOG (dialog));
 
 	cookies_tv = setup_cookies_treeview (dialog);
@@ -1044,6 +1059,25 @@ pdm_dialog_new (GtkWidget *window)
 	return GALEON_DIALOG(dialog);
 }
 
+void
+pdm_dialog_set_current_page (PdmDialog *dialog, PdmDialogPage page)
+{
+	GtkNotebook *notebook;
+	guint        n_pages;
+	
+	notebook = GTK_NOTEBOOK(dialog->priv->notebook);
+	n_pages  = gtk_notebook_get_n_pages (notebook);
+
+	if (page < n_pages)
+	{
+		gtk_notebook_set_current_page (notebook, page);
+	}
+	else
+	{
+		g_warning ("%s: invalid page `%d'", G_STRLOC, page);
+	}
+}
+
 static void
 show_cookies_properties (PdmDialog *dialog,
 			 const CookieInfo *info)
