@@ -151,19 +151,31 @@ void popup_cmd_add_bookmark (BonoboUIComponent *uic,
 	GaleonEmbedEvent *info = galeon_embed_popup_get_event (popup);
 	const GValue *link_title;
 	const GValue *link;
+	const GValue *link_is_smart;
+	const GValue *link_rel;
 	const char *title;
 	const char *location;
+	const char *smart_loc;
+	gboolean smart_site = FALSE;
 	GbBookmarkSet *set;
 	GbSite *b;
 
 	galeon_embed_event_get_property (info, "link", &link);
 	galeon_embed_event_get_property (info, "link_title", &link_title);
+	galeon_embed_event_get_property (info, "link_is_smart", &link_is_smart);
+	galeon_embed_event_get_property (info, "link_rel", &link_rel);
 	
 	title = g_value_get_string (link_title);
 	location = g_value_get_string (link);
+	smart_loc = g_value_get_string (link_rel);
 
 	g_return_if_fail (location);
 	
+	if (link_is_smart && g_value_get_int (link_is_smart))
+	{
+		smart_site = TRUE;
+	}
+
 	if (!title || !title[0])
 	{
 		title = location;
@@ -172,7 +184,16 @@ void popup_cmd_add_bookmark (BonoboUIComponent *uic,
 	set = galeon_shell_get_bookmark_set (galeon_shell);
 	g_return_if_fail (set);
 
-	b = gb_site_new (set, title, location);
+	if (!smart_site)
+	{
+		b = gb_site_new (set, title, location);
+	}
+	else
+	{
+		b = GB_SITE (gb_smart_site_new (set, title, 
+						location, smart_loc));
+		/* TODO add image saving support */
+	}
 	gb_bookmark_set_add_default (set, GB_BOOKMARK (b));
 	
 }
