@@ -27,6 +27,9 @@
 #include "eel-gconf-extensions.h"
 
 #include <string.h>
+#include <time.h>
+
+#include <bonobo/bonobo-i18n.h>
 #include <gtk/gtkdialog.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
@@ -59,6 +62,7 @@ static void session_class_init (SessionClass *klass);
 static void session_init (Session *t);
 static void session_finalize (GObject *object);
 static void session_dispose (GObject *object);
+static void session_create_bookmarks (Session *session, const char *filename);
 
 static GObjectClass *parent_class = NULL;
 
@@ -197,19 +201,20 @@ static void
 do_session_resume (Session *session, int type)
 {
 	char *crashed_session;
-	
+
+	crashed_session = get_session_filename (SESSION_CRASHED);	
 	switch (type)
 	{
 	case RESTORE_SESSION:
-		crashed_session = get_session_filename (SESSION_CRASHED);
 		session_load (session, crashed_session);
 		break;
 	case RESTORE_AS_BOOKMARKS:
-		g_assert (FALSE);
+		session_create_bookmarks (session, crashed_session);
 		break;
 	case DISCARD_SESSION:
 		break;
 	}
+	g_free (crashed_session);
 }
 
 static gboolean
@@ -250,9 +255,11 @@ crashed_resume_dialog (Session *session)
  *
  * Resume a crashed session when necessary (interactive)
  * 
- * Return value: return false if no window  has been opened
+ * Return value: returns SESSION_ABORT   if the user wants to abort starting galeon
+ *                       SESSION_RESUMED if a session has been resumed
+ *                       SESSION_OK      if no session was resumed
  **/
-gboolean
+SessionResumeType
 session_autoresume (Session *session)
 {
 	char *saved_session;
@@ -262,13 +269,13 @@ session_autoresume (Session *session)
 
 	if (g_file_test (saved_session, G_FILE_TEST_EXISTS))
 	{
-		/* we want to exit now, so dont open windows
-		 * but return like if we did . Also we need
-		 * to not remove the crashed session. */
+		/* we want to exit now, so tell our caller that the
+		 * user wants to abort, but dont remove the crashed
+		 * session. */
 		if (!crashed_resume_dialog (session))
 		{
 			session->priv->dont_remove_crashed = TRUE;
-			return TRUE;
+			return SESSION_ABORT;
 		}
 			
 		loaded = TRUE;
@@ -287,8 +294,8 @@ session_autoresume (Session *session)
 	
 	g_free (saved_session);
 
-	/* return false if no window has been opened */
-	return (session->priv->windows != NULL);
+	/* return SESSION_RESUMED, if a window was opened, else SESSION_OK */
+	return (session->priv->windows != NULL) ? SESSION_RESUMED : SESSION_OK;
 }
 
 static void
@@ -342,7 +349,14 @@ static void
 session_die_cb (GnomeClient* client, 
 		Session *session)
 {
+	g_object_ref (galeon_shell);
+
 	session_close (session);
+
+	/* Turn off server mode, so that we exit nicely */
+	galeon_shell_set_server_mode (galeon_shell, FALSE);
+
+	g_object_unref (galeon_shell);
 }
 
 static void
@@ -385,19 +399,20 @@ session_init (Session *session)
 void
 session_close (Session *session)
 {
-	GList *l;
+	GList *windows, *l;
 
 	/* save the session */
 	session_save (session, SESSION_SAVED);
 	session->priv->session_saved = TRUE;
 	
 	/* close all windows */
-	l = g_list_copy (session->priv->windows);
-	for (; l != NULL; l = l->next)
+	windows = g_list_copy (session->priv->windows);
+	for (l = windows; l != NULL; l = l->next)
 	{	
 		GaleonWindow *window = GALEON_WINDOW(l->data);
 		gtk_widget_destroy (GTK_WIDGET(window));
 	}
+	g_list_free(windows);
 }
 
 static void 
@@ -511,13 +526,15 @@ void
 session_save (Session *session,
 	      const char *filename)
 {
-	GList *w;
+	const GList *w;
         xmlNodePtr root_node;
         xmlNodePtr window_node;
         xmlDocPtr doc;
         gchar buffer[32];
 	char *save_to;
 	
+	g_return_if_fail (IS_SESSION (session));
+
 	save_to = get_session_filename (filename);
 		
         doc = xmlNewDoc ("1.0");
@@ -526,12 +543,10 @@ session_save (Session *session,
         root_node = xmlNewDocNode (doc, NULL, "session", NULL);
         xmlDocSetRootElement (doc, root_node);
 
-	w = session_get_windows (session);
-	
         /* iterate through all the windows */
-        for (; w != NULL; w = w->next)
+        for (w = session->priv->windows; w != NULL; w = w->next)
         {
-		const GList *tabs;
+		GList *tabs, *l;
         	int x = 0, y = 0, width = 0, height = 0;
 		GaleonWindow *window = GALEON_WINDOW(w->data);
 		GtkWidget *wmain;
@@ -558,11 +573,12 @@ session_save (Session *session,
                 snprintf(buffer, 32, "%d", height);
                 xmlSetProp (window_node, "height", buffer);
 
-		for (; tabs != NULL; tabs = tabs->next)
+		for (l = tabs; l != NULL; l = l->next)
 	        {
-			GaleonTab *tab = GALEON_TAB(tabs->data);
+			GaleonTab *tab = GALEON_TAB(l->data);
 			save_tab (window, tab, doc, window_node);
 		}
+		g_list_free (tabs);
 
 		xmlAddChild (root_node, window_node);
         }
@@ -574,12 +590,17 @@ session_save (Session *session,
 	g_free (save_to);
 }
 
+typedef void (*WindowCb) (xmlNodePtr child, 
+			  const int x, const int y, 
+			  const int width, const int height, 
+			  gpointer data);
+typedef void (*TabCb) (const char *url, const char *title, 
+		       gpointer data);
+
+
 static void
-parse_embed (xmlNodePtr child, GaleonWindow *window)
-{
-	GaleonTab *tab;
-	GaleonEmbed *embed;
-	
+parse_embed (xmlNodePtr child, TabCb tab_cb, gpointer data)
+{	
 	while (child != NULL)
 	{
 		if (strcmp (child->name, "embed") == 0)
@@ -587,20 +608,9 @@ parse_embed (xmlNodePtr child, GaleonWindow *window)
 			char *url;
 			char *title;
 			
-			g_return_if_fail (window != NULL);
-			
 			url = xmlGetProp (child, "url");
 			title = xmlGetProp (child, "title");
-
-			tab = galeon_tab_new ();	
-		        embed = galeon_tab_get_embed (tab);
-
-			gtk_widget_show (GTK_WIDGET(embed));
-		
-			galeon_window_add_tab (window, tab, FALSE);
-			
-			galeon_embed_load_url (embed, url);
-			
+			tab_cb (url, title, data);
 			xmlFree (url);
 			xmlFree (title);
 		}
@@ -609,21 +619,13 @@ parse_embed (xmlNodePtr child, GaleonWindow *window)
 	}
 }
 
-/*
- * session_load:
- * @session: a #Session
- * @filename: the path of the source file
- *
- * Load a session from disk, restoring the windows and their state
- **/
-void 
-session_load (Session *session,
-	      const char *filename)
+static void 
+parse_xml (Session *session,
+	   const char *filename, 
+	   WindowCb window_cb, gpointer data)
 {
 	xmlDocPtr doc;
         xmlNodePtr child;
-	GtkWidget *wmain;
-	GaleonWindow *window;
 	char *save_to;
 	
 	save_to = get_session_filename (filename);
@@ -645,16 +647,7 @@ session_load (Session *session,
                         y = xmlGetIntProp (child, "y");
                         width = xmlGetIntProp (child, "width");
                         height = xmlGetIntProp (child, "height");
-
-			window = galeon_window_new ();
-			wmain = GTK_WIDGET (window);
-			gtk_widget_show (GTK_WIDGET(window));
-			
-			gtk_window_move (GTK_WINDOW(wmain), x, y);
-                	gtk_window_set_default_size (GTK_WINDOW (wmain),
-                        	                     width, height);
-			
-			parse_embed (child->children, window);
+			window_cb (child, x, y, width, height, data);
 		}
 
 		child = child->next;
@@ -665,7 +658,85 @@ session_load (Session *session,
 	g_free (save_to);
 }
 
-GList *
+
+static void 
+create_new_tab (const char *url, const char *title, GaleonWindow *window)
+{
+	GaleonTab *tab;
+	GaleonEmbed *embed;
+
+	tab = galeon_tab_new ();	
+	embed = galeon_tab_get_embed (tab);
+	
+	gtk_widget_show (GTK_WIDGET(embed));
+	
+	galeon_window_add_tab (window, tab, FALSE);
+	
+	galeon_embed_load_url (embed, url);
+}
+
+static void
+create_new_window (xmlNodePtr child,
+		   const int x, const int y, 
+		   const int width, const int height)
+{
+	GtkWidget *wmain;
+	GaleonWindow *window;
+
+	window = galeon_window_new ();
+	wmain = GTK_WIDGET (window);
+	gtk_widget_show (GTK_WIDGET(window));
+	
+	gtk_window_move (GTK_WINDOW(wmain), x, y);
+	gtk_window_set_default_size (GTK_WINDOW (wmain),
+				     width, height);
+
+	parse_embed (child->children, (TabCb)create_new_tab, window);
+}
+
+/*
+ * The following 2 functions are used to create bookmarks from a session
+ * files (used when galeon crashed)
+ */
+
+static void 
+bookmark_from_tab (const char *url, const char *title, GbFolder *folder)
+{
+	GbSite *new_site;
+	GbBookmarkSet *set;
+
+	g_return_if_fail (GB_IS_FOLDER (folder));
+
+	set = galeon_shell_get_bookmark_set (galeon_shell);
+	new_site = gb_site_new (set, title, url);
+	gb_folder_add_child (folder, GB_BOOKMARK(new_site), -1);
+}
+
+static void
+bookmarks_from_window (xmlNodePtr child,
+		       const int x, const int y, 
+		       const int width, const int height,
+		       GbFolder *folder)
+{
+	g_return_if_fail (GB_IS_FOLDER (folder));
+	parse_embed (child->children, (TabCb)bookmark_from_tab, folder);
+}
+
+/*
+ * session_load:
+ * @session: a #Session
+ * @filename: the path of the source file
+ *
+ * Load a session from disk, restoring the windows and their state
+ **/
+
+void 
+session_load (Session *session, const char *filename)
+{
+	parse_xml (session, filename, (WindowCb)create_new_window, NULL);
+}
+
+const GList *
 session_get_windows (Session *session)
 {
 	g_return_val_if_fail (IS_SESSION (session), NULL);
@@ -673,6 +744,53 @@ session_get_windows (Session *session)
 	return session->priv->windows;
 }
 
+/*
+ * Create bookmarks from a crashed session. Store the bookmarks 
+ * in a subfolder of the root folder called "Sites opened before
+ * crash on <date> at <time>"
+ */
+#define DATE_LENGTH 30
+static
+void session_create_bookmarks (Session *session, const char *filename)
+{
+	GbFolder *root_folder;
+	GbFolder *new_folder;
+	GbBookmarkSet *set;
+	char date[DATE_LENGTH];
+	const char *date_hack="%c"; /* This is necessary to avoid a warning */
+	struct tm *tm;
+	time_t timep;
+	gchar *folder_name;
+
+	g_return_if_fail (IS_SESSION (session));
+
+	/* Create a new folder under the root bookmarks which will be used
+	 * to add the recovered bookmarks
+	 */
+	set = galeon_shell_get_bookmark_set (galeon_shell);
+	root_folder = set->root;
+
+	time (&timep);
+	tm = localtime (&timep);
+	if (strftime (date, DATE_LENGTH, date_hack, tm) != 0) {
+		gchar *date_utf8 = g_locale_to_utf8 (date, -1,
+						     NULL, NULL, NULL);
+		
+		folder_name = g_strdup_printf (_("Sites opened before crash on %s"), date_utf8);
+		g_free (date_utf8);
+	} else {
+		folder_name = g_strdup_printf ( _("Sites opened before crash"));
+	}
+	new_folder =  gb_folder_new (set, folder_name);
+	g_free (folder_name);
+	gb_folder_add_child (root_folder, GB_BOOKMARK(new_folder), -1);
+	
+	/* Add bookmarks corresponding to the previous session to this folder
+	 */
+	parse_xml (session, filename, (WindowCb)bookmarks_from_window, 
+		   new_folder);
+}
+
 /**
  * session_add_window:
  * @session: a #Session
@@ -712,7 +830,8 @@ session_remove_window (Session *session,
 
 	/* autodestroy of the session, necessay to avoid
 	 * conflicts with the nautilus view */
-	if (session->priv->windows == NULL)
+	if (session->priv->windows == NULL && 
+	    galeon_shell_get_server_mode (galeon_shell) == FALSE)
 	{
 		g_object_unref (session);
 	}
