@@ -42,6 +42,7 @@
 #include "galeon-home-button.h"
 #include "galeon-new-button.h"
 #include "galeon-stop-button.h"
+#include "galeon-fullscreen-button.h"
 
 #include <string.h>
 #include <bonobo/bonobo-i18n.h>
@@ -54,11 +55,13 @@
 
 #define DEFAULT_TOOLBAR_SETUP \
 	"back_menu=navigation_button(direction=back,arrow=TRUE);" \
-	"up_menu=navigation_button(direction=up,arrow=TRUE);" \
 	"forward_menu=navigation_button(direction=forward,arrow=TRUE);" \
 	"stop=stop_button;" \
 	"reload=reload_button;" \
+	"home_separator=separator;" \
 	"home=home_button;" \
+	"fullscreen=fullscreen_button;" \
+	"favicons_separator=separator;" \
 	"favicon=favicon;" \
 	"location=location;" \
 	"zoom=zoom;" \
@@ -70,6 +73,7 @@
 
 static void toolbar_class_init (ToolbarClass *klass);
 static void toolbar_init (Toolbar *t);
+static void toolbar_dispose (GObject *object);
 static void toolbar_finalize (GObject *object);
 static void toolbar_set_window (Toolbar *t, GaleonWindow *window);
 static void toolbar_get_widgets (Toolbar *t);
@@ -110,6 +114,8 @@ struct ToolbarPrivate
 	GtkWidget *spinner;
 	gboolean visibility;
 
+	guint style_notification_id;
+
 	GSList *navigation_buttons;
 	GaleonReloadButton *reload_button;
 	GaleonGoButton *go_button;
@@ -123,6 +129,7 @@ struct ToolbarPrivate
 	GtkWidget *zoom_spinbutton;
 	guint zoom_timeout_id;
 	gboolean zoom_lock;
+	GaleonFullscreenButton *fullscreen_button;
 };
 
 GType 
@@ -161,6 +168,7 @@ toolbar_class_init (ToolbarClass *klass)
 
         parent_class = g_type_class_peek_parent (klass);
 
+        object_class->dispose = toolbar_dispose;
         object_class->finalize = toolbar_finalize;
 
 	object_class->set_property = toolbar_set_property;
@@ -180,6 +188,7 @@ toolbar_class_init (ToolbarClass *klass)
 	gul_toolbar_item_register_type ("home_button", (GulTbItemConstructor) galeon_home_button_new);
 	gul_toolbar_item_register_type ("stop_button", (GulTbItemConstructor) galeon_stop_button_new);
 	gul_toolbar_item_register_type ("new_button", (GulTbItemConstructor) galeon_new_button_new);
+	gul_toolbar_item_register_type ("fullscreen_button", (GulTbItemConstructor) galeon_fullscreen_button_new);
 }
 
 static void
@@ -215,23 +224,47 @@ toolbar_get_property (GObject *object,
 }
 
 static void
-toolbar_location_url_activate_cb (GaleonLocationEntry *entry, const gchar *url, GaleonWindow *window)
+toolbar_location_url_activate_cb (GaleonLocationEntry *entry, const gchar *url,
+                                  gboolean new_tab_or_window, GaleonWindow *window)
 {
-	galeon_window_load_url (window, url);
+	GbBookmarkSet *set;
+	gchar *real_url;
+	GaleonTab *tab;
+
+	if (!new_tab_or_window)
+	{
+		/* galeon_window_load_url handles bookmark nicks */
+		galeon_window_load_url (window, url);
+		return;
+	}
+
+	set = galeon_shell_get_bookmark_set (galeon_shell);
+	real_url = gb_bookmark_set_get_url_by_nick_and_args (set, url);
+
+	/* Open the link in a new tab (or window) */
+	tab = galeon_window_get_active_tab (window);
+	galeon_shell_new_tab (galeon_shell, window, tab, 
+			      real_url != NULL ? real_url : url, 0);
+	
+	g_free (real_url);
 }
 
 static void
 each_url_get_data_binder (GaleonDragEachSelectedItemDataGet iteratee, 
         		  gpointer iterator_context, gpointer data)
 {
-	const char *location;
+	const char *location, *title;
 	GaleonTab *tab;
 	GaleonWindow *window = GALEON_WINDOW(iterator_context);
 
 	tab = galeon_window_get_active_tab (window);
 	location = galeon_tab_get_location (tab);
-	
-	iteratee (location, -1, -1, -1, -1, data);
+	title    = galeon_tab_get_title (tab);
+
+	if (strcmp (location, "about:blank"))
+	{
+		iteratee (location, title, -1, -1, -1, -1, data);
+	}
 }
 
 static void
@@ -261,7 +294,7 @@ toolbar_setup_favicon_ebox (Toolbar *t, GtkWidget *w)
 	gtk_container_add (GTK_CONTAINER (p->favicon_ebox), p->favicon);
 	gtk_container_set_border_width (GTK_CONTAINER (p->favicon_ebox), 2);
 
-	galeon_dnd_url_drag_source_set (p->favicon_ebox);
+	galeon_dnd_single_url_drag_source_set (p->favicon_ebox);
 
 	g_signal_connect (G_OBJECT (p->favicon_ebox),
 			  "drag_data_get",
@@ -358,7 +391,7 @@ toolbar_setup_spinner (Toolbar *t, GtkWidget *w)
 static void
 toolbar_setup_style (Toolbar *t)
 {
-	static const char *styles[] = { "both_horiz", "both", "text", "icons" };
+	static const char *styles[] = { "both_horiz", "both", "text", "icon" };
 	const char *style;
 	GConfValue *value;
 
@@ -374,8 +407,8 @@ toolbar_setup_style (Toolbar *t)
 			style = styles[i];
 		}
 		gconf_value_free (value);
-		gul_bonobo_set_look (t->priv->ui_component, "/Toolbar", style);
 	}
+	gul_bonobo_set_look (t->priv->ui_component, "/Toolbar", style);
 }
 
 static void
@@ -397,8 +430,9 @@ toolbar_set_window (Toolbar *t, GaleonWindow *window)
 
 	gul_tb_bonobo_view_set_path (t->priv->bview, t->priv->ui_component, "/Toolbar");
 
-	eel_gconf_notification_add (CONF_TOOLBAR_STYLE,
-			            toolbar_style_changed_cb, t);
+	t->priv->style_notification_id = 
+		eel_gconf_notification_add (CONF_TOOLBAR_STYLE,
+				            toolbar_style_changed_cb, t);
 
 	toolbar_setup_style (t);
 	toolbar_get_widgets (t);
@@ -442,6 +476,12 @@ toolbar_get_widgets (Toolbar *t)
 	}
 	p->go_button = NULL;
 
+	if (p->fullscreen_button)
+	{
+		g_object_unref (p->fullscreen_button);
+	}
+	p->fullscreen_button = NULL;
+
 	if (p->home_button)
 	{
 		g_object_unref (p->home_button);
@@ -609,6 +649,25 @@ toolbar_get_widgets (Toolbar *t)
 		}
 	}
 
+	it = gul_toolbar_get_item_by_id (gt, "fullscreen");
+	if (it)
+	{
+		if (GALEON_IS_FULLSCREEN_BUTTON (it))
+		{	
+			DEBUG_MSG (("    got a fullscreen button\n"));
+			p->fullscreen_button = g_object_ref (it);
+			if (p->window)
+			{
+				galeon_tbi_set_window (GALEON_TBI (it), p->window);
+			}
+		}
+		else
+		{
+			g_warning ("An unexpected button has been found in your toolbar. "
+				   "Maybe your setup is too old.");
+		}
+	}
+
 	it = gul_toolbar_get_item_by_id (gt, "location");
 	if (it)
 	{
@@ -693,6 +752,22 @@ toolbar_changed_cb (GulToolbar *gt, Toolbar *t)
 	}
 }
 
+static void
+toolbar_dispose (GObject *object)
+{
+	Toolbar *t = (Toolbar *) object;
+
+        g_return_if_fail (IS_TOOLBAR (object));
+
+	if (t->priv->bview)
+	{
+		g_object_unref (t->priv->bview);
+		g_object_unref (t->priv->ui_component);
+		t->priv->bview = NULL;
+		t->priv->ui_component = NULL;
+	}
+}
+
 static void
 toolbar_finalize (GObject *object)
 {
@@ -708,6 +783,8 @@ toolbar_finalize (GObject *object)
 
         g_return_if_fail (p != NULL);
 
+	eel_gconf_notification_remove (p->style_notification_id);
+
 	if (p->location_entry) g_object_unref (p->location_entry);
 	if (p->favicon_ebox) g_object_unref (p->favicon_ebox);
 	if (p->favicon) g_object_unref (p->favicon);
@@ -719,8 +796,6 @@ toolbar_finalize (GObject *object)
 		g_source_remove (p->zoom_timeout_id);
 	}
 
-	g_object_unref (t->priv->bview);
-	
 	for (li = t->priv->navigation_buttons; li; li = li->next)
 	{
 		g_object_unref (li->data);
@@ -752,6 +827,11 @@ toolbar_finalize (GObject *object)
 		g_object_unref (p->new_button);
 	}
 
+	if (t->priv->fullscreen_button)
+	{
+		g_object_unref (p->fullscreen_button);
+	}
+
         g_free (t->priv);
 
 	DEBUG_MSG (("Toolbar finalized\n"));
@@ -868,6 +948,17 @@ toolbar_set_location (Toolbar *t,
 	}
 }
 
+void
+toolbar_set_fullscreen (Toolbar *t, gboolean s)
+{
+	if (t->priv->fullscreen_button)
+	{
+		galeon_fullscreen_button_set_fullscreen 
+			( GALEON_FULLSCREEN_BUTTON(t->priv->fullscreen_button), s);
+	}
+}
+
+
 void
 toolbar_update_favicon (Toolbar *t)
 {
