@@ -33,6 +33,7 @@
 #include "galeon-window-bookmark-activated-proxy.h"
 #include "gul-toolbar-editor.h"
 #include "gul-bonobo-extensions.h"
+#include "galeon-prefs-dialog.h"
 
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -738,16 +739,99 @@ window_cmd_tools_history (BonoboUIComponent *uic,
 	galeon_window_show_history (window);
 }
 
+static void
+open_pdm_dialog (GtkWidget *window, int page_num)
+{
+	static GaleonDialog *dialog = NULL;
+
+	/* FIXME: we want have only one PDM dialog shown, but since it's a
+	 * GaleonDialog we can't re-use set_transient_for with it like we can
+	 * with sane dialogs.  So we'll just destroy and recreate the dialog
+	 * instead..
+	 */
+
+	if (dialog != NULL)
+	{
+		galeon_dialog_destruct (dialog);
+	}
+
+	dialog = pdm_dialog_new (window);
+	g_object_add_weak_pointer (G_OBJECT(dialog), (gpointer)&dialog);
+
+	if (page_num != -1)
+	{
+		pdm_dialog_set_current_page (PDM_DIALOG(dialog), page_num);
+	}
+	galeon_dialog_show (dialog);
+}
+
 void 
 window_cmd_tools_pdm (BonoboUIComponent *uic, 
 		      GaleonWindow *window, 
 		      const char* verbname)
 {
-	GaleonDialog *dialog;
+	open_pdm_dialog (GTK_WIDGET(window), -1);
+}
 
-	dialog = pdm_dialog_new (GTK_WIDGET(window));
+static void
+on_prefs_dialog_request (GaleonPrefsDialog  *dialog,
+		         GaleonPrefsRequest  request,
+			 GaleonWindow       *window)
+{
+	switch (request)
+	{
+	case GALEON_PREFS_CLEAR_CACHE:
+		galeon_embed_shell_clear_cache (embed_shell, MEMORY_CACHE);
+		galeon_embed_shell_clear_cache (embed_shell, DISK_CACHE);
+		break;
+	case GALEON_PREFS_MANAGE_CERTIFICATES:
+	{
+		GaleonWindow *new_win;
+		GaleonTab    *tab;
 
-	galeon_dialog_show (dialog);
+		new_win = galeon_window_new ();
+		gtk_window_resize (GTK_WINDOW(new_win), 550, 400);
+		galeon_window_set_chrome (new_win, EMBED_CHROME_CENTERSCREEN
+				                 | EMBED_CHROME_WINDOWRAISED
+						 | EMBED_CHROME_OPENASCHROME
+						 | EMBED_CHROME_OPENASPOPUP);
+		tab = galeon_tab_new ();
+		galeon_window_add_tab (new_win, tab, TRUE);
+		
+		gtk_widget_show_all (GTK_WIDGET(galeon_tab_get_embed (tab)));
+
+		galeon_window_load_url (new_win, "chrome://pippki/content/certManager.xul");
+
+		gtk_widget_show (GTK_WIDGET(new_win));
+		gtk_window_set_title (GTK_WINDOW(new_win), _("Certificate Manager"));
+
+		break;
+	}
+	case GALEON_PREFS_MANAGE_COOKIES:
+		open_pdm_dialog (GTK_WIDGET(dialog), PDM_PAGE_COOKIES);
+		break;
+	case GALEON_PREFS_MANAGE_IMAGES:
+		open_pdm_dialog (GTK_WIDGET(dialog), PDM_PAGE_IMAGE_SITES);
+		break;
+	case GALEON_PREFS_MANAGE_PASSWORDS:
+		open_pdm_dialog (GTK_WIDGET(dialog), PDM_PAGE_PASSWORDS);
+		break;
+	case GALEON_PREFS_MANAGE_POPUPS:
+		open_pdm_dialog (GTK_WIDGET(dialog), PDM_PAGE_POPUP_SITES);
+		break;
+	default:
+		g_warning ("%s: unhandled request `%d'", G_STRLOC, request);
+	}
+}
+
+static char *
+on_prefs_dialog_location (GaleonPrefsDialog *dialog,
+		          GaleonWindow      *window)
+{
+	GaleonTab *tab;
+
+	tab = galeon_window_get_active_tab (window);
+	return g_strdup (galeon_tab_get_location (tab));
 }
 
 void 
@@ -755,14 +839,37 @@ window_cmd_edit_prefs (BonoboUIComponent *uic,
 		       GaleonWindow *window, 
 		       const char* verbname)
 {
-	GtkDialog *dialog;
+	static GtkWidget *dialog = NULL;
+
+	/* We want to have only one prefs dialog shown, so reparent the one we
+	 * have. */
+
+	if (dialog == NULL)
+	{
+		dialog = galeon_prefs_dialog_new ();
+		g_object_add_weak_pointer (G_OBJECT (dialog),
+				           (gpointer)&dialog);
+	}
+	else
+	{
+		g_signal_handlers_disconnect_matched
+			(G_OBJECT(dialog), G_SIGNAL_MATCH_FUNC, 0, 0, NULL, 
+			 G_CALLBACK(on_prefs_dialog_request), NULL);
+		g_signal_handlers_disconnect_matched
+			(G_OBJECT(dialog), G_SIGNAL_MATCH_FUNC, 0, 0, NULL, 
+			 G_CALLBACK(on_prefs_dialog_location), NULL);
+	}
+
+	g_signal_connect (G_OBJECT(dialog), "request",
+			  G_CALLBACK(on_prefs_dialog_request),
+			  window);
+	g_signal_connect (G_OBJECT(dialog), "location",
+			  G_CALLBACK(on_prefs_dialog_location),
+			  window);
 
-	dialog = prefs_dialog_new ();
-	prefs_dialog_show_page (PREFS_DIALOG(dialog),
-				PREFS_PAGE_GENERAL);
 	gtk_window_set_transient_for (GTK_WINDOW (dialog),
 				      GTK_WINDOW (window));
-	gtk_widget_show (GTK_WIDGET(dialog));
+	gtk_window_present (GTK_WINDOW(dialog));
 }
 
 static void
@@ -866,19 +973,25 @@ window_cmd_help_about (BonoboUIComponent *uic,
 	gchar      	 *file;
 
 	const gchar *authors[] = {
+		"Philip Langdale <philipl@mail.utexas.edu>",
+		"Tommi Komulainen <tommi.komulainen@iki.fi>",
+		"Ricardo Fern\xc3\xa1ndez Pascual <ric@users.sourceforge.net>",
+		/* this one is supposed to be in cyrillic */
+		_("Yanko Kaneti <yaneti@declera.com>"),
+		"Crispin Flowerday <gnome@flowerday.cx>",
+		/* don't delete the space */
+		" ",
+		_("Past developers:"),
 		"Marco Pesenti Gritti <mpeseng@tin.it>",
 		"Remco de Jong <rdj@rdj.cg.nu>",
-		"Ricardo Fern\xc3\xa1ndez Pascual <ric@users.sourceforge.net>",
 		"Nate Case <nd@kracked.com>",
 		"Bob Phan <ex-freek@yahoo.com>",
 		"Matt Aubury <matt@ookypooky.com>",
 		"Joshua Horvath <jmhorvat@home.com>",
-		/* this one is supposed to be in cyrillic */
-		_("Yanko Kaneti <yaneti@declera.com>"),
-		"Philip Langdale <philipl@mail.utexas.edu>",
 		"Matthew Mueller <donut@azstarnet.com>",
 		"Daniel Erat <dan_erat@pobox.com>",
 		"Jorn Baayen <jorn@nl.linux.org>",
+		"Christophe Fergeau <teuf@users.sourceforge.net>",
 		/* don't delete the space */
 		" ",
 		_("Graphic arts:"),
@@ -911,7 +1024,7 @@ window_cmd_help_about (BonoboUIComponent *uic,
 	
 	about = gnome_about_new(
 		       _("Galeon"), VERSION,
-		       "Copyright \xc2\xa9 2000-2002 Marco Pesenti Gritti",
+		       "Copyright \xc2\xa9 2003 The Galeon Developers\nCopyright \xc2\xa9 2000-2002 Marco Pesenti Gritti",
 		       _("A GNOME browser based on Mozilla"),
 		       (const char **)authors,
 		       (const char **)documenters,
@@ -923,7 +1036,7 @@ window_cmd_help_about (BonoboUIComponent *uic,
 				
 	gtk_window_set_transient_for (GTK_WINDOW (about),
 				      GTK_WINDOW (window));
-	g_object_add_weak_pointer (G_OBJECT (about), (gpointer *)&about);
+	g_object_add_weak_pointer (G_OBJECT (about), (gpointer)&about);
 	gtk_widget_show (about);
 }
 
