@@ -98,7 +98,7 @@ static gboolean	galeon_autocompletion_window_key_press_cb	(GtkWidget *widget,
 								 GdkEventKey *event,
 								 GaleonAutocompletionWindow *aw);
 static gboolean	galeon_autocompletion_window_key_press_hack	(GaleonAutocompletionWindow *aw, 
-								 guint keyval);
+								 GdkEventKey *event);
 static void 	hack_tree_view_move_selection 			(GtkTreeView *tv, int dir);
 static void	galeon_autocompletion_window_event_after_cb	(GtkWidget *wid, GdkEvent *event,
 								 GaleonAutocompletionWindow *aw);
@@ -137,8 +137,8 @@ galeon_autocompletion_window_class_init (GaleonAutocompletionWindowClass *klass)
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP,
                 G_STRUCT_OFFSET (GaleonAutocompletionWindowClass, url_activated), 
 		NULL, NULL, 
-		galeon_marshal_VOID__STRING,
-		G_TYPE_NONE, 1, G_TYPE_STRING);
+		galeon_marshal_VOID__STRING_INT,
+		G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
 	GaleonAutocompletionWindowSignals[GALEON_AUTOCOMPLETION_WINDOW_HIDDEN] = g_signal_new (
 		"hidden", G_OBJECT_CLASS_TYPE (klass),  
@@ -165,6 +165,16 @@ galeon_autocompletion_window_init (GaleonAutocompletionWindow *aw)
 	gtk_tree_selection_set_mode (s, GTK_SELECTION_MULTIPLE);
 	
 	g_signal_connect (s, "changed", G_CALLBACK (galeon_autocompletion_window_selection_changed_cb), aw);
+
+	g_signal_connect (p->window, "button-press-event", 
+			  G_CALLBACK (galeon_autocompletion_window_button_press_event_cb),
+			  aw);
+	g_signal_connect (p->window, "key-press-event",
+			  G_CALLBACK (galeon_autocompletion_window_key_press_cb),
+			  aw);
+	g_signal_connect (p->tree_view, "event-after", 
+			  G_CALLBACK (galeon_autocompletion_window_event_after_cb),
+			  aw);
 }
 
 static void
@@ -580,21 +590,8 @@ galeon_autocompletion_window_show (GaleonAutocompletionWindow *aw)
 				  GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK |
 				  GDK_BUTTON_RELEASE_MASK,
 				  NULL, NULL, GDK_CURRENT_TIME);
-		gdk_keyboard_grab (p->parent->window, TRUE, GDK_CURRENT_TIME);\
+		gdk_keyboard_grab (p->parent->window, TRUE, GDK_CURRENT_TIME);
 		gtk_grab_add (p->window);
-		
-		g_signal_connect (p->window, "button-press-event", 
-				  G_CALLBACK (galeon_autocompletion_window_button_press_event_cb),
-				  aw);
-		g_signal_connect (p->window, "key-press-event",
-				  G_CALLBACK (galeon_autocompletion_window_key_press_cb),
-				  aw);
-		g_signal_connect (p->parent, "key-press-event",
-				  G_CALLBACK (galeon_autocompletion_window_key_press_cb),
-				  aw);
-		g_signal_connect (p->tree_view, "event-after", 
-				  G_CALLBACK (galeon_autocompletion_window_event_after_cb),
-				  aw);
 		p->shown = TRUE;
 	}
 
@@ -647,7 +644,7 @@ galeon_autocompletion_window_key_press_cb (GtkWidget *widget,
 	    || ((event->keyval == GDK_space || event->keyval == GDK_Return)
 		&& p->selected))
 	{
-		return galeon_autocompletion_window_key_press_hack (aw, event->keyval);
+		return galeon_autocompletion_window_key_press_hack (aw, event);
 	}
 	else
 	{
@@ -676,10 +673,11 @@ galeon_autocompletion_window_key_press_cb (GtkWidget *widget,
 }
 
 static gboolean
-galeon_autocompletion_window_key_press_hack (GaleonAutocompletionWindow *aw, guint keyval)
+galeon_autocompletion_window_key_press_hack (GaleonAutocompletionWindow *aw, 
+					     GdkEventKey *event)
 {
 	GaleonAutocompletionWindowPrivate *p = aw->priv;
-	switch (keyval)
+	switch (event->keyval)
 	{
 	case GDK_Up:
 		hack_tree_view_move_selection (p->tree_view, -1);
@@ -697,8 +695,11 @@ galeon_autocompletion_window_key_press_hack (GaleonAutocompletionWindow *aw, gui
 	case GDK_space:
 		if (p->selected)
 		{
+			gboolean new_tab_or_window = event->state & GDK_CONTROL_MASK;
+
 			g_signal_emit (aw, GaleonAutocompletionWindowSignals
-				       [GALEON_AUTOCOMPLETION_WINDOW_URL_ACTIVATED], 0, p->selected->data);
+				       [GALEON_AUTOCOMPLETION_WINDOW_URL_ACTIVATED], 0, 
+				       p->selected->data, new_tab_or_window);
 		}
 		break;
 	default:
@@ -797,7 +798,8 @@ galeon_autocompletion_window_event_after_cb (GtkWidget *wid, GdkEvent *event,
 		if (p->selected)
 		{
 			g_signal_emit (aw, GaleonAutocompletionWindowSignals
-				       [GALEON_AUTOCOMPLETION_WINDOW_URL_ACTIVATED], 0, p->selected->data);
+				       [GALEON_AUTOCOMPLETION_WINDOW_URL_ACTIVATED], 0, 
+				       p->selected->data, FALSE);
 		}
 	}
 }
