@@ -111,6 +111,30 @@ extern void galeon_marshal_INT__OBJECT (GClosure     *closure,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
 
+/* STRING:VOID (./galeon-marshal.list:16) */
+extern void galeon_marshal_STRING__VOID (GClosure     *closure,
+                                         GValue       *return_value,
+                                         guint         n_param_values,
+                                         const GValue *param_values,
+                                         gpointer      invocation_hint,
+                                         gpointer      marshal_data);
+
+/* STRING:STRING (./galeon-marshal.list:17) */
+extern void galeon_marshal_STRING__STRING (GClosure     *closure,
+                                           GValue       *return_value,
+                                           guint         n_param_values,
+                                           const GValue *param_values,
+                                           gpointer      invocation_hint,
+                                           gpointer      marshal_data);
+
+/* BOOLEAN:OBJECT (./galeon-marshal.list:18) */
+extern void galeon_marshal_BOOLEAN__OBJECT (GClosure     *closure,
+                                            GValue       *return_value,
+                                            guint         n_param_values,
+                                            const GValue *param_values,
+                                            gpointer      invocation_hint,
+                                            gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* __galeon_marshal_MARSHAL_H__ */
