@@ -613,6 +613,7 @@ gul_bonobo_add_dockitem (BonoboUIComponent *uic,
 	return path;
 }
 
+/* deprecated, use gul_bonobo_add_numbered_widget instead */
 BonoboControl *
 gul_bonobo_add_numbered_control (BonoboUIComponent *uic, GtkWidget *w, 
 				 guint index, const char *container_path)
@@ -642,6 +643,22 @@ gul_bonobo_add_numbered_control (BonoboUIComponent *uic, GtkWidget *w,
 	return control;
 }
 
+void
+gul_bonobo_add_numbered_widget (BonoboUIComponent *uic, GtkWidget *w, 
+				 guint index, const char *container_path)
+{
+	gchar *path = gul_bonobo_get_numbered_menu_item_path (uic, container_path, index);
+	gchar *item_name = get_numbered_menu_item_name (index);
+	gchar *xml_string = g_strdup_printf ("<control name=\"%s\"/>", item_name);
+	
+	bonobo_ui_component_set (uic, container_path, xml_string, NULL);
+	bonobo_ui_component_widget_set (uic, path, w, NULL);
+
+	g_free (path);
+	g_free (item_name);
+	g_free (xml_string);
+}
+
 void 
 gul_bonobo_replace_path (BonoboUIComponent *uic, const gchar *path_src, 
 			 const char *path_dst)
@@ -677,3 +694,11 @@ gul_bonobo_clear_path (BonoboUIComponent *uic,
 		g_free (remove_wildcard);
 	}
 }
+
+void
+gul_bonobo_set_look (BonoboUIComponent *uic,
+		     const gchar *path,
+		     const gchar *look)
+{
+	bonobo_ui_component_set_prop (uic, path, "look", look, NULL);
+}
