@@ -30,6 +30,14 @@
 #include <libgnome/gnome-i18n.h>
 #include <gtk/gtktreemodel.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkwindow.h>
+#include <gtk/gtkdialog.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkimagemenuitem.h>
+#include <gtk/gtkcheckmenuitem.h>
+#include <gtk/gtkmessagedialog.h>
 
 static void
 make_selection_list (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
