@@ -20,10 +20,11 @@
 #define GUL_GUI_H
 
 /* system includes */
-#include <gtk/gtk.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <gnome.h>
+#include <glib.h>
+#include <gtk/gtkmenu.h>
+#include <gtk/gtktreeview.h>
+#include <gtk/gtkradiobutton.h>
+
 
 G_BEGIN_DECLS
 	
