@@ -59,8 +59,6 @@ struct GulNotebookPrivate
 	gulong motion_notify_handler_id;
 	gint x_start, y_start;
 	gboolean drag_in_progress;
-	GulNotebook *src_notebook;
-	gint src_page;
 };
 
 /* GObject boilerplate code */
@@ -74,9 +72,7 @@ static GList *notebooks  = NULL;
 
 
 /* Local functions */
-static void drag_start (GulNotebook *notebook, 
-			GulNotebook *src_notebook,
-			gint src_page);
+static void drag_start (GulNotebook *notebook);
 static void drag_stop  (GulNotebook *notebook);
 
 static gboolean motion_notify_cb (GulNotebook *notebook, 
@@ -432,7 +428,7 @@ move_tab_to_another_notebook(GulNotebook *src,
 	gul_notebook_move_page (src, dest, child, dest_page);
 
 	/* "Give" drag handling to the new notebook */
-	drag_start (dest, src->priv->src_notebook, src->priv->src_page);
+	drag_start (dest);
 	drag_stop (src);		
 	gtk_grab_remove (GTK_WIDGET (src));
 	
@@ -466,13 +462,9 @@ move_tab (GulNotebook *notebook, gint dest_page_num)
 }
 
 static void 
-drag_start (GulNotebook *notebook, 
-	    GulNotebook *src_notebook, 
-	    gint src_page)
+drag_start (GulNotebook *notebook)
 {
 	notebook->priv->drag_in_progress = TRUE;
-	notebook->priv->src_notebook = src_notebook;
-	notebook->priv->src_page = src_page;
 
 	/* get a new cursor, if necessary */
 	if (!cursor) cursor = gdk_cursor_new (GDK_FLEUR);
@@ -491,8 +483,7 @@ static void
 drag_stop (GulNotebook *notebook)
 {
 	notebook->priv->drag_in_progress = FALSE;
-	notebook->priv->src_notebook = NULL;
-	notebook->priv->src_page = -1;
+
 	if (notebook->priv->motion_notify_handler_id != 0) 
 	{
 		g_signal_handler_disconnect (G_OBJECT (notebook), 
@@ -553,16 +544,19 @@ motion_notify_cb (GulNotebook *notebook, GdkEventMotion *event,
 		return FALSE;
 	}
 	
-	if ((notebook->priv->drag_in_progress == FALSE) 
-	    && (gtk_drag_check_threshold (GTK_WIDGET (notebook),
-					  notebook->priv->x_start, 
-					  notebook->priv->y_start, 
-					  event->x_root, event->y_root)))
+	if (notebook->priv->drag_in_progress == FALSE)
 	{
-		gint cur_page;
-		
-		cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-		drag_start (notebook, notebook, cur_page); 
+		if (gtk_drag_check_threshold (GTK_WIDGET (notebook),
+					      notebook->priv->x_start, 
+					      notebook->priv->y_start, 
+					      event->x_root, event->y_root))
+		{
+			drag_start (notebook);
+		}
+		else
+		{
+			return FALSE;
+		}
 	}
 
 	result = find_notebook_and_tab_at_pos ((gint)event->x_root, 
@@ -666,8 +660,6 @@ gul_notebook_init (GulNotebook *notebook)
 
 	notebook->priv->drag_in_progress = FALSE;
 	notebook->priv->motion_notify_handler_id = 0;
-	notebook->priv->src_notebook = NULL;
-	notebook->priv->src_page = -1;
 	notebook->priv->focused_pages = NULL;
 	notebook->priv->automatic_tab_switch = FALSE;
 	notebook->priv->opened_tabs = NULL;
