@@ -23,12 +23,12 @@
 #include "gul-string.h"
 
 /* system includes */
-#include <string.h>
-#include <glib.h>
-#include <libgnome/libgnome.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libxml/parser.h>
+#include <time.h>
+#include <string.h>
+#include <errno.h>
 
 /* precomputed g_unichar_to_utf8 (8230, ellipsis); */
 #define ELLIPSIS "\xe2\x80\xa6"
