@@ -31,6 +31,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkalignment.h>
 #include <gtk/gtktoolbar.h>
+#include <gdk/gdkkeysyms.h>
 #include <bonobo/bonobo-ui-toolbar.h>
 #include <string.h>
 
@@ -59,6 +60,7 @@ struct _GulTbButtonPrivate
 	gboolean priority;
 	gboolean always_image;
 	gboolean show_button;
+	gboolean toggle_button;
 
 	gboolean in_bonobo_toobar;
 
@@ -95,7 +97,9 @@ static gboolean		gul_tb_button_button_button_press_event_cb (GtkWidget *widget,
 								    GulTbButton *b);
 static void 		gul_tb_button_button_popup_menu_cb	(GtkWidget *w, GulTbButton *b);
 static void 		gul_tb_button_menu_deactivated_cb	(GtkMenuShell *ms, GulTbButton *b);
-
+static void             gul_tb_button_state_changed_cb          (GtkWidget *widget, 
+								 GtkStateType previous_state,
+								 GulTbButton *b);
 
 static gpointer gtk_hbox_class;
 
@@ -185,6 +189,23 @@ gul_tb_button_finalize_impl (GObject *o)
 		g_object_unref (p->arrow_widget);
 	}
 
+	if (p->button_box)
+	{
+		g_object_unref (p->button_box);
+	}
+
+	if (p->button)
+	{
+		g_object_unref (p->button);
+	}
+
+	if (p->label_wid)
+	{
+		g_object_unref (p->label_wid);
+	}
+
+	g_free (p->label);
+	g_free (p->tooltip_text);
 	g_free (p);
 	
 	DEBUG_MSG (("GulTbButton finalized\n"));
@@ -269,11 +290,27 @@ maybe_reparent (GtkWidget *w, GtkWidget *p)
 	}
 }
 
+/* This is used to empty the container before it is destroyed, other wise
+ * for some reasons images go blank, and widgets dont get their signals if
+ * they are re-added to a different container */
+static void
+empty_container (GtkContainer *c)
+{
+	GList *list, *item;
+
+	list = gtk_container_get_children (GTK_CONTAINER(c));
+	for (item = list; item; item = item->next)
+	{
+		gtk_container_remove (GTK_CONTAINER (c), GTK_WIDGET (item->data));
+
+	}
+	g_list_free (list);
+}
+
 static void
 gul_tb_button_build (GulTbButton *b)
 {
 	GulTbButtonPrivate *p = b->priv;
-	GtkWidget *image;
 	GtkStockItem stock_item;
 	gboolean really_use_stock = p->use_stock && gtk_stock_lookup (p->label, &stock_item);
 	gboolean show_image = p->always_image || p->label[0] == '\0'
@@ -292,8 +329,9 @@ gul_tb_button_build (GulTbButton *b)
 	
 	if (!p->button && p->show_button)
 	{
-		p->button = gtk_button_new ();
+		p->button = (p->toggle_button) ? gtk_toggle_button_new() : gtk_button_new ();
 		g_object_ref (p->button);
+		gtk_object_sink (GTK_OBJECT (p->button));
 		gtk_box_pack_start_defaults (GTK_BOX (b), p->button);
 	}
 
@@ -320,6 +358,9 @@ gul_tb_button_build (GulTbButton *b)
 				  G_CALLBACK (gul_tb_button_button_button_press_event_cb), b);
 		g_signal_connect (p->button, "popup_menu", 
 				  G_CALLBACK (gul_tb_button_button_popup_menu_cb), b);
+		g_signal_connect (p->button, "state_changed",
+				  G_CALLBACK (gul_tb_button_state_changed_cb), b);
+
 		if (p->tooltips)
 		{
 			gtk_tooltips_set_tip (p->tooltips, p->button,
@@ -351,28 +392,35 @@ gul_tb_button_build (GulTbButton *b)
 	{
 		if (p->button_box && !GTK_IS_VBOX (p->button_box))
 		{
+			empty_container (GTK_CONTAINER (p->button_box));
 			gtk_container_remove (GTK_CONTAINER (p->align), GTK_WIDGET (p->button_box));
+			g_object_unref (p->button_box);
 			p->button_box = NULL;
 		}
 		if (!p->button_box)
 		{
 			p->button_box = GTK_BOX (gtk_vbox_new (FALSE, 2));
+			g_object_ref (p->button_box);
+			gtk_object_sink (GTK_OBJECT (p->button_box));
 		}
 	}
 	else
 	{
 		if (p->button_box && !GTK_IS_HBOX (p->button_box))
 		{
-			
+			empty_container (GTK_CONTAINER (p->button_box));
 			gtk_container_remove (GTK_CONTAINER (p->align), GTK_WIDGET (p->button_box));
+			g_object_unref (p->button_box);
 			p->button_box = NULL;
 		}
 		if (!p->button_box)
 		{
 			p->button_box = GTK_BOX (gtk_hbox_new (FALSE, 2));
+			g_object_ref (p->button_box);
+			gtk_object_sink (GTK_OBJECT (p->button_box));
 		}
 	}
-	g_object_ref (p->button_box);
+
 	gtk_widget_show (GTK_WIDGET (p->button_box));
 	maybe_reparent (GTK_WIDGET (p->button_box), p->align);
 	
@@ -384,24 +432,24 @@ gul_tb_button_build (GulTbButton *b)
 
 	if (!p->image && really_use_stock && show_image)
 	{
-		image = gtk_image_new_from_stock (p->label, p->icon_size);
+		GtkWidget *image = gtk_image_new_from_stock (p->label, p->icon_size);
 		if (image)
 		{
 			p->image = g_object_ref (image);
-			gtk_object_sink (GTK_OBJECT (image));
+			gtk_object_sink (GTK_OBJECT (p->image));
 		}
 	}
-	else
-	{
-		image = p->image;
-	}
 
-	if (image)
+	if (p->image)
 	{
 		if (show_image)
 		{
-			maybe_reparent (image, GTK_WIDGET (p->button_box));
-			gtk_widget_show (image);
+			gtk_widget_show (p->image);
+			maybe_reparent (p->image, GTK_WIDGET (p->button_box));
+		}
+		else
+		{
+			gtk_widget_hide (p->image);
 		}
 	}
 	else if (!p->widget)
@@ -412,7 +460,8 @@ gul_tb_button_build (GulTbButton *b)
 	if (p->label_wid && GTK_WIDGET (p->label_wid)->parent == GTK_WIDGET (p->button_box))
 	{
 		gtk_container_remove (GTK_CONTAINER (p->button_box), GTK_WIDGET (p->label_wid));
-		}
+	}
+
 	if (p->label_wid) 
 	{
 		g_object_unref (p->label_wid);
@@ -421,16 +470,9 @@ gul_tb_button_build (GulTbButton *b)
 
 	if (show_label)
 	{
-		if (p->label_wid && GTK_WIDGET (p->label_wid)->parent == GTK_WIDGET (p->button_box))
-		{
-			gtk_container_remove (GTK_CONTAINER (p->button_box), GTK_WIDGET (p->label_wid));
-		}
-		if (p->label_wid) 
-		{
-			g_object_unref (p->label_wid);
-		}
 		p->label_wid = GTK_LABEL (gtk_label_new (p->label));
 		g_object_ref (p->label_wid);
+		gtk_object_sink (GTK_OBJECT (p->label_wid));
 		
 		if (really_use_stock)
 		{
@@ -523,6 +565,9 @@ gul_tb_button_set_show_arrow (GulTbButton *b, gboolean value)
 		g_signal_connect  (p->arrow_widget, "button_press_event",
 				   G_CALLBACK (gul_tb_button_arrow_button_press_event_cb),
 				   b);
+		g_signal_connect (p->arrow_widget, "state_changed",
+				  G_CALLBACK (gul_tb_button_state_changed_cb),
+				  b);
 
 		gtk_widget_show_all (p->arrow_widget);
 		gtk_box_pack_end_defaults (GTK_BOX (b), p->arrow_widget);
@@ -568,6 +613,24 @@ gul_tb_button_get_button (GulTbButton *b)
 	return GTK_BUTTON (p->button);
 }
 
+void
+gul_tb_button_set_use_toggle_button (GulTbButton *b)
+{
+	GulTbButtonPrivate *p = b->priv;
+	p->toggle_button = TRUE;
+
+	if (p->button && ! GTK_IS_TOGGLE_BUTTON (p->button))
+	{
+		g_object_unref (p->button);
+		p->button = 0;
+	}
+	if ( !p->button )
+	{
+		gul_tb_button_build (b);
+	}
+}
+
+
 void
 gul_tb_button_set_use_stock (GulTbButton *b, gboolean value)
 {
@@ -831,6 +894,47 @@ gul_tb_button_button_popup_menu_cb (GtkWidget *w, GulTbButton *b)
 	}
 }
 
+
+static void
+gul_tb_button_state_changed_cb (GtkWidget *widget,
+				GtkStateType previous_state,
+				GulTbButton *b)
+{
+	GulTbButtonPrivate *p = b->priv;
+	GtkWidget *button;
+	GtkStateType state = GTK_WIDGET_STATE (widget);
+
+	if (!p->arrow_widget) return;
+
+	button = (widget == p->arrow_widget) ? p->button : p->arrow_widget;
+
+	g_signal_handlers_block_by_func
+		         (G_OBJECT (button),
+		          G_CALLBACK (gul_tb_button_state_changed_cb),
+			  b);
+	
+
+	if (state == GTK_STATE_PRELIGHT &&
+	    previous_state != GTK_STATE_ACTIVE)
+	{
+		gtk_widget_set_state (button, state);
+	}
+	else if (state == GTK_STATE_NORMAL)
+	{
+		 gtk_widget_set_state (button, state);
+	}
+	else if (state == GTK_STATE_ACTIVE)
+	{
+		gtk_widget_set_state (button, GTK_STATE_NORMAL);
+	}
+
+	g_signal_handlers_unblock_by_func
+		         (G_OBJECT (button),
+		          G_CALLBACK (gul_tb_button_state_changed_cb),
+			  b);
+}
+
+
 void
 gul_tb_button_set_sensitivity (GulTbButton *b, gboolean value)
 {
