@@ -20,14 +20,20 @@
 #include "config.h"
 #endif
 
+#include <gtk/gtktreeview.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtkwindow.h>
+#include <gtk/gtkcellrendererpixbuf.h>
+#include <gtk/gtkcellrenderertext.h>
+
 #include <libgnome/gnome-i18n.h>
+
 #include "gul-gobject-misc.h"
 #include "galeon-marshal.h"
 #include "gul-toolbar-editor.h"
 #include "gul-toolbar-tree-model.h"
 #include "gul-glade.h"
 
-#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
 //#define DEBUG_MSG(x) g_print x
 #define DEBUG_MSG(x)
 
@@ -245,6 +251,50 @@ gul_tb_editor_right_clicked_cb (GtkWidget *b, GulTbEditor *tbe)
 	}
 }
 
+static void
+gul_tb_editor_update_arrows_sensitivity (GulTbEditor *tbe)
+{
+	GtkTreeSelection *selection;
+	gboolean current_sel;
+	gboolean avail_sel;	
+	gboolean first = FALSE;
+	gboolean last = FALSE;
+	GtkTreeModel *tm;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+
+	selection = gtk_tree_view_get_selection 
+		(GTK_TREE_VIEW (tbe->priv->current_view));
+	current_sel = gtk_tree_selection_get_selected (selection, &tm, &iter);	
+	if (current_sel)
+	{
+		path = gtk_tree_model_get_path (tm, &iter);
+		first = !gtk_tree_path_prev (path);
+		last = !gtk_tree_model_iter_next (tm, &iter);	
+	}
+
+	selection = gtk_tree_view_get_selection 
+		(GTK_TREE_VIEW (tbe->priv->available_view));
+	avail_sel = gtk_tree_selection_get_selected (selection, &tm, &iter);
+	
+	gtk_widget_set_sensitive (tbe->priv->right_button, 
+				  avail_sel);
+	gtk_widget_set_sensitive (tbe->priv->left_button, 
+				  current_sel);
+	gtk_widget_set_sensitive (tbe->priv->up_button, 
+				  current_sel && !first);
+	gtk_widget_set_sensitive (tbe->priv->down_button,
+				  current_sel && !last);
+}
+
+static void
+gul_tb_editor_treeview_selection_changed_cb (GtkTreeSelection *selection,
+					     GulTbEditor *tbe)
+{
+	gul_tb_editor_update_arrows_sensitivity (tbe);
+}
+
+
 static GulTbItem *
 gul_tb_editor_get_selected (GulTbEditor *tbe, GtkTreeView *tv)
 {
@@ -361,15 +411,16 @@ gul_tb_editor_set_toolbar (GulTbEditor *tbe, GulToolbar *tb)
 	g_free (p->tb_undo_string);
 	p->tb_undo_string = gul_toolbar_to_string (p->tb);
 
+	gul_tb_editor_set_treeview_toolbar (tbe, GTK_TREE_VIEW (p->current_view), p->tb);
+
 	if (p->available)
 	{
 		gul_tb_editor_remove_used_items (tbe);
+		gul_tb_editor_update_arrows_sensitivity (tbe);
 	}
 
 	g_signal_connect (p->tb, "changed", G_CALLBACK (gul_tb_editor_toolbar_changed_cb), tbe);
 
-	gul_tb_editor_set_treeview_toolbar (tbe, GTK_TREE_VIEW (p->current_view), p->tb);
-
 	gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (p->current_view),
 						GDK_BUTTON1_MASK,
 						tree_view_row_targets,
@@ -399,13 +450,14 @@ gul_tb_editor_set_available (GulTbEditor *tbe, GulToolbar *tb)
 
 	gul_toolbar_set_fixed_order (p->available, TRUE);
 
+	gul_tb_editor_set_treeview_toolbar (tbe, GTK_TREE_VIEW (p->available_view), p->available);
+
 	if (p->tb)
 	{
 		gul_tb_editor_remove_used_items (tbe);
+		gul_tb_editor_update_arrows_sensitivity (tbe);
 	}
 
-	gul_tb_editor_set_treeview_toolbar (tbe, GTK_TREE_VIEW (p->available_view), p->available);
-
 	gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (p->available_view),
 						GDK_BUTTON1_MASK,
 						tree_view_row_targets,
@@ -444,9 +496,11 @@ gul_tb_editor_setup_treeview (GulTbEditor *tbe, GtkTreeView *tv)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
+	GtkTreeSelection *selection;
 	
 	column = gtk_tree_view_column_new ();
 	renderer = gtk_cell_renderer_pixbuf_new ();
+	selection = gtk_tree_view_get_selection (tv);
 	
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	gtk_tree_view_column_set_attributes (column, renderer,
@@ -464,6 +518,8 @@ gul_tb_editor_setup_treeview (GulTbEditor *tbe, GtkTreeView *tv)
 
 	g_signal_connect (tv, "button-press-event", 
 			  G_CALLBACK (gul_tb_editor_treeview_button_press_event_cb), tbe);
+	g_signal_connect (selection, "changed",
+			  G_CALLBACK (gul_tb_editor_treeview_selection_changed_cb), tbe);
 }
 
 GulToolbar *
