@@ -234,17 +234,19 @@ gul_toolbar_listen_to_gconf_cb (GConfClient* client,
 				gpointer user_data)
 {
 	GulToolbar *tb = user_data;
-	GConfValue *value;
-	const char *str;
+	char *str;
 	
 	g_return_if_fail (GUL_IS_TOOLBAR (tb));
 	
-	value = gconf_entry_get_value (entry);
-	str = gconf_value_get_string (value);
+	str = eel_gconf_get_string (entry->key);
+	if( str )
+	{
+		DEBUG_MSG (("in gul_toolbar_listen_to_gconf_cb\n"));
 
-	DEBUG_MSG (("in gul_toolbar_listen_to_gconf_cb\n"));
-
-	gul_toolbar_parse (tb, str);
+		gul_toolbar_parse (tb, str);
+		
+		g_free (str);
+	}
 }
 
 /**
