@@ -27,6 +27,7 @@
 #include <gtk/gtkwindow.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtkclipboard.h>
 #include <libgnomeui/gnome-entry.h>
 #include <string.h>
 
@@ -331,9 +332,28 @@ real_set_location (GaleonLocationEntry *w,
 {
 	GaleonLocationEntryPrivate *p = w->priv;
 	int pos = 0;
+	int start, end;
+	gchar* selection = NULL;
+
+	/* Get the current selection, it is removed when we do the delete_text 
+	 * below */
+	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (p->entry), &start, &end))
+	{
+		selection = gtk_editable_get_chars (GTK_EDITABLE (p->entry), start, end);
+	}
+
 	gtk_editable_delete_text (GTK_EDITABLE (p->entry), 0, -1);
 	gtk_editable_insert_text (GTK_EDITABLE (p->entry), new_location, 
 			          strlen (new_location), &pos);
+
+	/* Restore the selection into the PRIMARY clipboard as it will have just
+	 * been wiped */
+	if (selection)
+	{
+		gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
+					selection, strlen (selection));
+		g_free (selection);
+	}
 }
 
 static void
