@@ -23,7 +23,10 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <gdk/gdk.h>
+#include <gdk/gdkpixbuf.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkiconfactory.h>
+
 #include <libgnome/gnome-util.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
@@ -129,7 +132,8 @@ gul_pixbuf_cache_register_stocks (void)
 		{ GALEON_STOCK_DEFAULT       ,"_Default",    0, 0, NULL },								     { GALEON_STOCK_DEFAULT_OPEN  ,"_Default",    0, 0, NULL },
 		{ GALEON_STOCK_HISTORY       ,"_History",    0, 0, NULL },
 		{ GALEON_STOCK_VIEWSOURCE    ,"_Source",     0, 0, NULL },
-		{ GALEON_STOCK_SENDLINK      ,"_Send link",  0, 0, NULL }
+		{ GALEON_STOCK_SENDLINK      ,"_Send link",  0, 0, NULL },
+		{ GALEON_STOCK_FULLSCREEN    ,"_Fullscreen", 0, 0, NULL }
 	};
 
 	if (done) 
